﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_CONFIG_H_
#define INCLUDE_NN_NLIB_CONFIG_H_

// NOTE:
// DO NOT INCLUDE THE STANDARD C/C++ LIBRARY HEADERS
// on the line always included in this header.
// This would make it difficult to avoid compile errors
// on the environment which lacks the standard headers.

#include "nn/nlib/Platform.h"

#if defined(_MSC_VER)
#if defined(n_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#elif defined(nx_misc_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif
#endif

#ifndef NLIB_NAMESPACE_BEGIN
#define NLIB_NAMESPACE_BEGIN namespace nn { namespace nlib {
#endif

#ifndef NLIB_NAMESPACE_END
#define NLIB_NAMESPACE_END }}
#endif

#ifndef NLIB_NS
#define NLIB_NS ::nn::nlib
#endif

NLIB_NAMESPACE_BEGIN
NLIB_NAMESPACE_END
namespace nlib_ns = NLIB_NS;

#if defined(CAFE)
# include "nn/nlib/Config_cafe.h"  // NOLINT
#elif defined(NN_PLATFORM_CTR)
# include "nn/nlib/Config_ctr.h"  // NOLINT
#elif defined(__NX__)
# include "nn/nlib/Config_nx.h"  // NOLINT
#elif defined(_MSC_VER)
# include "nn/nlib/Config_win32.h"  // NOLINT
#elif defined(__CYGWIN__)
# include "nn/nlib/Config_cygwin.h"  // NOLINT
#elif defined(__linux__)
# include "nn/nlib/Config_linux.h"  // NOLINT
#elif defined(__FreeBSD__)
# include "nn/nlib/Config_freebsd.h"  // NOLINT
#elif defined(__APPLE__) && defined(__MACH__)
# include "nn/nlib/Config_osx.h"  // NOLINT
#endif

#ifdef NLIB_CXX11_STDLIB_ATOMIC
// NOTE: redefine them to use std C++ library for memory barrier
# undef NLIB_MEMORY_ORDER_RELEASE
# undef NLIB_MEMORY_ORDER_ACQUIRE
# undef NLIB_MEMORY_ORDER_ACQ_REL
# include <atomic>  // NOLINT
# define NLIB_MEMORY_ORDER_RELEASE ::std::atomic_thread_fence(::std::memory_order_release)
# define NLIB_MEMORY_ORDER_ACQUIRE ::std::atomic_thread_fence(::std::memory_order_acquire)
# define NLIB_MEMORY_ORDER_ACQ_REL ::std::atomic_thread_fence(::std::memory_order_acq_rel)
#endif

#ifdef __cpp_rvalue_references
# define NLIB_RREF &&
# define NLIB_FWD(T, v) std::forward<T>(v)
# define NLIB_MOVE(x) std::move(x)
#else
# define NLIB_RREF &
# define NLIB_FWD(T, v) (v)
# define NLIB_MOVE(x) (x)
#endif

#ifdef __cpp_constexpr
# define NLIB_CEXPR constexpr
# if __cpp_constexpr >= 201304L
#  define NLIB_CEXPR14 constexpr
# else
#  define NLIB_CEXPR14
# endif
#else
# define NLIB_CEXPR
# define NLIB_CEXPR14
#endif

#ifdef NLIB_DOXYGEN
#define NLIB_NOEXCEPT noexcept
#define NLIB_NOEXCEPT_FUNCPTR
#define NLIB_CEXPR constexpr
#define NLIB_CEXPR14 constexpr
#define NLIB_DEPRECATED [[deprecated]]  // NOLINT
#define NLIB_DEPRECATED_MSG(x) [[deprecated(x)]]  // NOLINT
#endif

#ifndef NLIB_NOEXCEPT
#ifdef NLIB_CXX11_NOEXCEPT
# define NLIB_NOEXCEPT noexcept
#else
# define NLIB_NOEXCEPT throw()
#endif
#elif defined(__INTELLISENSE__)
# undef NLIB_NOEXCEPT
# define NLIB_NOEXCEPT
#endif

#ifndef NLIB_NOEXCEPT_FUNCPTR
#define NLIB_NOEXCEPT_FUNCPTR
#endif

struct nlib_unwind_exception {};  // Do not throw this directly
#ifndef NLIB_RETHROW_UNWIND
# define NLIB_RETHROW_UNWIND catch (nlib_unwind_exception&)
#endif

#ifndef NLIB_ASSERT_NOERR
# define NLIB_ASSERT_NOERR(e) NLIB_ASSERT((e) == 0)
#endif

#ifdef __cpp_exceptions
# define NLIB_TRY try
# define NLIB_CATCH(x) catch(x)
# define NLIB_THROW throw
#else
# define NLIB_TRY if (true)
# define NLIB_CATCH(x) if (false)
# define NLIB_THROW
#endif

#ifndef NLIB_STATIC_ASSERT
#ifndef __cpp_static_assert
NLIB_NAMESPACE_BEGIN
namespace detail {
template <bool>
struct STATIC_ASSERTION_FAILURE;
template <>
struct STATIC_ASSERTION_FAILURE<true> {};
template <int x>
struct static_assert_test {};
}  // namespace detail
NLIB_NAMESPACE_END

#define NLIB_ASSERT_H_STRING_JOIN_(X, Y) NLIB_ASSERT_H_STRING_JOIN1_(X, Y)
#define NLIB_ASSERT_H_STRING_JOIN1_(X, Y) X##Y

#define NLIB_STATIC_ASSERT(exp)                                           \
    typedef ::nlib_ns::detail::static_assert_test<                       \
        sizeof(::nlib_ns::detail::STATIC_ASSERTION_FAILURE<(exp) != 0>)> \
        NLIB_ASSERT_H_STRING_JOIN_(nn_static_assert_typedef_, __LINE__)
#elif __cpp_static_assert >= 201411L
# define NLIB_STATIC_ASSERT(exp) static_assert(exp)
#else
# define NLIB_STATIC_ASSERT(exp) static_assert((exp), "NLIB_STATIC_ASSERT error: " #exp)
#endif
#endif

#ifndef NLIB_CXX11_DEFAULTED_AND_DELETED_FUNCTIONS
#define NLIB_DISALLOW_COPY_AND_ASSIGN(TypeName) \
TypeName(const TypeName&);                      \
void operator=(const TypeName&)
#else
#define NLIB_DISALLOW_COPY_AND_ASSIGN(TypeName) \
TypeName(const TypeName&) = delete;             \
void operator=(const TypeName&) = delete
#endif

#ifndef NLIB_CXX11_EXPLICIT_CONVERSION_OPERATORS
#define NLIB_SAFE_BOOL(class_name, exp)                                         \
private:                                                                        \
    typedef void (class_name::*bool_type)() const;                              \
    void this_type_does_not_support_comparisons() const NLIB_NOEXCEPT {}        \
public:                                                                         \
    operator bool_type() const NLIB_NOEXCEPT {                                  \
        return (exp) ? &class_name::this_type_does_not_support_comparisons : 0; \
    }
#else
#define NLIB_SAFE_BOOL(class_name, exp) \
public:                                 \
    explicit NLIB_ALWAYS_INLINE operator bool() const NLIB_NOEXCEPT { return (exp); }
#endif

#include <wchar.h>  // NOLINT
#if defined(WCHAR_MIN) && defined(WCHAR_MAX)
# if WCHAR_MAX <= 0xFFFF
#  define NLIB_WCHAR_SIZE (2)
NLIB_STATIC_ASSERT(sizeof(wchar_t) == 2);
# else
#  define NLIB_WCHAR_SIZE (4)
NLIB_STATIC_ASSERT(sizeof(wchar_t) == 4);
# endif
#else
# error WCHAR_MIN, WCHAR_MAX not defined
#endif

#ifndef NLIB_CXX11_NULL_POINTER_CONSTANT
NLIB_NAMESPACE_BEGIN
class nullptr_t {
 public:
    template <class T>
    operator T*() const {
        return 0;
    }
    template <class C, class T>
    operator T C::*() const {
        return 0;
    }

 private:
    void operator&() const;  // NOLINT
};
#ifndef nullptr
// if nullptr macro is defined, just use it.
const nullptr_t nullptr = {};
#endif
NLIB_NAMESPACE_END
#else
#include <cstddef>  // NOLINT this is because stddef.h does not define std::nullptr_t
NLIB_NAMESPACE_BEGIN
typedef std::nullptr_t nullptr_t;
NLIB_NAMESPACE_END
#endif

#ifndef NLIB_CXX11_EXPLICIT_VIRTUAL_OVERRIDES
# define NLIB_OVERRIDE
# define NLIB_FINAL
#else
# define NLIB_OVERRIDE override
# define NLIB_FINAL final
#endif

#ifdef NLIB_CXX11_ALIGNMENT_SUPPORT
# ifdef NLIB_ALIGNAS
#  undef NLIB_ALIGNAS
# endif
# ifdef NLIB_ALIGNOF
#  undef NLIB_ALIGNOF
# endif
# define NLIB_ALIGNAS(x) alignas(x)
# define NLIB_ALIGNOF(tp) alignof(tp)
#endif

#ifdef __cpp_rvalue_references
// for std::move
# include <utility>  // NOLINT
#endif

NLIB_NAMESPACE_BEGIN
struct move_tag {};
NLIB_NAMESPACE_END

#ifdef __cpp_rvalue_references
#define NLIB_DEFMOVE_PIMPL(tp) \
    NLIB_ALWAYS_INLINE tp(tp&& rhs) NLIB_NOEXCEPT : prv_(rhs.prv_) { \
        rhs.prv_ = nullptr;                                          \
    }                                                                \
    NLIB_ALWAYS_INLINE tp& operator=(tp&& rhs) NLIB_NOEXCEPT {       \
        Reset();                                                     \
        prv_ = rhs.prv_;                                             \
        rhs.prv_ = nullptr;                                          \
        return *this;                                                \
    }                                                                \
    NLIB_ALWAYS_INLINE                                               \
    tp(tp& rhs, ::nlib_ns::move_tag) NLIB_NOEXCEPT : prv_(rhs.prv_) /* NOLINT */ { \
        rhs.prv_ = nullptr;                                             \
    }                                                                \
    NLIB_ALWAYS_INLINE                                               \
    tp& assign(tp& rhs, ::nlib_ns::move_tag) NLIB_NOEXCEPT { /* NOLINT */  \
        Reset();                                                     \
        prv_ = rhs.prv_;                                             \
        rhs.prv_ = nullptr;                                             \
        return *this;                                                \
    }
#else
#define NLIB_DEFMOVE_PIMPL(tp) \
    NLIB_ALWAYS_INLINE                                               \
    tp(tp& rhs, ::nlib_ns::move_tag) NLIB_NOEXCEPT : prv_(rhs.prv_) /* NOLINT */ { \
        rhs.prv_ = nullptr;                                             \
    }                                                                \
    NLIB_ALWAYS_INLINE                                               \
    tp& assign(tp& rhs, ::nlib_ns::move_tag) NLIB_NOEXCEPT { /* NOLINT */  \
        Reset();                                                     \
        prv_ = rhs.prv_;                                             \
        rhs.prv_ = nullptr;                                             \
        return *this;                                                \
    }
#endif

#ifndef NLIB_MEMCHECKER
# define NLIB_MEMCHECKER NLIB_STATIC_ASSERT(sizeof(char) == 1)        // NOLINT
# define NLIB_MEMCHECKER_START NLIB_STATIC_ASSERT(sizeof(char) == 1)  // NOLINT
# define NLIB_MEMCHECKER_CHECK true
#endif

// The following macro "NLIB_NOEMPTYFILE()" can be put into a file
// in order suppress the MS Visual C++ Linker warning 4221
#ifndef NLIB_NOEMPTYFILE
# define NLIB_NOEMPTYFILE()
#endif

#ifdef NLIB_SOCKET_ENABLED
# ifdef _MSC_VER
#  ifndef NLIB_WINDLL
#   ifdef _DEBUG
#    define NLIB_SOCKPORT_SAMPLE (17974 + _MSC_VER + 50)
#   else
#    define NLIB_SOCKPORT_SAMPLE (17974 + _MSC_VER)
#   endif
#  else
#   ifdef _DEBUG
#    define NLIB_SOCKPORT_SAMPLE (17974 + _MSC_VER + 75)
#   else
#    define NLIB_SOCKPORT_SAMPLE (17974 + _MSC_VER + 25)
#   endif
#  endif
# elif defined(__clang__)
#  ifdef NDEBUG
#   define NLIB_SOCKPORT_SAMPLE (18174)
#  else
#   define NLIB_SOCKPORT_SAMPLE (18074)
#  endif
# else
#  ifdef NDEBUG
#   define NLIB_SOCKPORT_SAMPLE (17874)
#  else
#   define NLIB_SOCKPORT_SAMPLE (17974)
#  endif
# endif
#endif

//
// You can reduce the loopcount for the performance test
// when it runs on on a continuous integration system like Jenkins.
//
// for (int i = 0; i < NLIB_TESTLOOPCOUNT(10000); ++i) {
//     .....
// }
//
#if !defined(NLIB_CIBUILD) && defined(NDEBUG)
# define NLIB_TESTLOOPCOUNT(x) (x)
#else
# define NLIB_TESTLOOPCOUNT(x) (1)
#endif

#ifndef NLIB_OVERRIDE_NEW
#define NLIB_OVERRIDE_NEW \
    static void* operator new(size_t size); \
    static void operator delete(void* ptr); \
    static void* operator new(size_t size, void* ptr) NLIB_NOEXCEPT; \
    static void operator delete(void* mem, void* ptr) NLIB_NOEXCEPT; \
    static void* operator new(size_t size, const std::nothrow_t& nt) NLIB_NOEXCEPT; \
    static void operator delete(void* mem, const std::nothrow_t& nt) NLIB_NOEXCEPT
#endif

#ifndef NLIB_OVERRIDE_NEW_LIBNEW_CPP
#define NLIB_OVERRIDE_NEW_LIBNEW_CPP(type) \
    void* type::operator new(size_t size) { return ::operator new(size); } \
    void type::operator delete(void* ptr) { ::operator delete(ptr); } \
    void* type::operator new(size_t size, void* ptr) NLIB_NOEXCEPT { \
        return ::operator new(size, ptr); \
    } \
    void type::operator delete(void* mem, void* ptr) NLIB_NOEXCEPT { \
        ::operator delete(mem, ptr); \
    } \
    void* type::operator new(size_t size, const std::nothrow_t& nt) NLIB_NOEXCEPT { \
        return ::operator new(size, nt); \
    } \
    void type::operator delete(void* mem, const std::nothrow_t& nt) NLIB_NOEXCEPT { \
        NLIB_UNUSED(nt); \
        ::operator delete(mem); \
    }
#endif

#define NLIB_EQUAL_OPERATOR(TP) \
NLIB_ALWAYS_INLINE bool operator!=(const TP& lhs, const TP& rhs) NLIB_NOEXCEPT { \
    return !(lhs == rhs); \
}
#define NLIB_COMPARE_OPERATOR(TP) \
NLIB_ALWAYS_INLINE bool operator>(const TP& lhs, const TP& rhs) NLIB_NOEXCEPT { \
    return rhs < lhs; \
} \
NLIB_ALWAYS_INLINE bool operator<=(const TP& lhs, const TP& rhs) NLIB_NOEXCEPT { \
    return !(rhs < lhs); \
} \
NLIB_ALWAYS_INLINE bool operator>=(const TP& lhs, const TP& rhs) NLIB_NOEXCEPT { \
    return !(lhs < rhs); \
}

NLIB_NAMESPACE_BEGIN
class NLIB_VIS_PUBLIC ErrnoT NLIB_FINAL {
 public:
    ErrnoT() NLIB_NOEXCEPT : vs_errno_(0) {}
    NLIB_CEXPR ErrnoT(errno_t e) NLIB_NOEXCEPT : vs_errno_(e) {}  // NOLINT
    ErrnoT& operator=(errno_t e) NLIB_NOEXCEPT {
        vs_errno_ = e;
        return *this;
    }
    operator errno_t() const NLIB_NOEXCEPT { return vs_errno_; }
    // const char* c_str() const NLIB_NOEXCEPT { return nlib_error_string(vs_errno_); }

 private:
    // to suppress the mix-up with boolean value
    bool operator!() const;  // FORBIDDEN
    // operator bool() const;  // FORBIDDEN
    errno_t vs_errno_;
};

NLIB_ALWAYS_INLINE bool operator==(const ErrnoT& lhs, errno_t rhs) NLIB_NOEXCEPT {
    return lhs.operator errno_t() == rhs;
}

NLIB_ALWAYS_INLINE bool operator==(errno_t lhs, const ErrnoT& rhs) NLIB_NOEXCEPT {
    return rhs.operator errno_t() == lhs;
}

NLIB_ALWAYS_INLINE bool operator!=(const ErrnoT& lhs, errno_t rhs) NLIB_NOEXCEPT {
    return lhs.operator errno_t() != rhs;
}

NLIB_ALWAYS_INLINE bool operator!=(errno_t lhs, const ErrnoT& rhs) NLIB_NOEXCEPT {
    return rhs.operator errno_t() != lhs;
}

class NLIB_VIS_PUBLIC Utf8Ptr NLIB_FINAL {
 public:
    Utf8Ptr() NLIB_NOEXCEPT : str_(nullptr) {}
    Utf8Ptr(const char* str) NLIB_NOEXCEPT : str_(str) {}  // NOLINT
    operator const char*() const NLIB_NOEXCEPT { return str_; }
    Utf8Ptr& operator=(const char* str) NLIB_NOEXCEPT { str_ = str; return *this; }
    const char& operator[](size_t idx) const NLIB_NOEXCEPT { return str_[idx]; }

 private:
    const char* str_;
};

template<size_t N>
class Utf8Array NLIB_FINAL {
 public:
    Utf8Array() NLIB_NOEXCEPT {}
    operator char*() NLIB_NOEXCEPT { return &str_[0]; }
    operator const char*() const NLIB_NOEXCEPT { return &str_[0]; }
    const char& operator[](size_t idx) const NLIB_NOEXCEPT { return str_[idx]; }
    char& operator[](size_t idx) NLIB_NOEXCEPT { return str_[idx]; }

 private:
    char str_[N];
};
NLIB_NAMESPACE_END

//
// C++ functions in global namespace
//
#ifdef __cplusplus
template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_vsnprintf(
    size_t* count,
    char (&buf)[N],
    _Printf_format_string_ const char* fmt,
    va_list args) NLIB_NOEXCEPT {
    return nlib_vsnprintf(count, buf, N, fmt, args);
}

template<size_t N>
inline NLIB_VIS_HIDDEN errno_t nlib_snprintf(
    size_t* count,
    char (&buf)[N],
    _Printf_format_string_ const char* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    errno_t e = nlib_vsnprintf(count, buf, N, fmt, args);
    va_end(args);
    return e;
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_vsnwprintf(
    size_t* count,
    wchar_t (&buf)[N],
    _Printf_format_string_ const wchar_t* fmt,
    va_list args) NLIB_NOEXCEPT {
    return nlib_vsnwprintf(count, buf, N, fmt, args);
}

template<size_t N>
inline NLIB_VIS_HIDDEN errno_t nlib_snwprintf(
    size_t* count,
    wchar_t(&buf)[N],
    _Printf_format_string_ const wchar_t* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    errno_t e = nlib_vsnwprintf(count, buf, N, fmt, args);
    va_end(args);
    return e;
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_vsnprintf_fallback(
    size_t* count,
    char (&buf)[N],
    _Printf_format_string_ const char* fmt,
    va_list args) NLIB_NOEXCEPT {
    return nlib_vsnprintf_fallback(count, buf, N, fmt, args);
}

template<size_t N>
inline NLIB_VIS_HIDDEN errno_t nlib_snprintf_fallback(
    size_t* count,
    char (&buf)[N],
    _Printf_format_string_ const char* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    errno_t e = nlib_vsnprintf_fallback(count, buf, N, fmt, args);
    va_end(args);
    return e;
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_vsnwprintf_fallback(
    size_t* count,
    wchar_t (&buf)[N],
    _Printf_format_string_ const wchar_t* fmt,
    va_list args) NLIB_NOEXCEPT {
    return nlib_vsnwprintf_fallback(count, buf, N, fmt, args);
}

template<size_t N>
inline NLIB_VIS_HIDDEN errno_t nlib_snwprintf_fallback(
    size_t* count,
    wchar_t(&buf)[N],
    _Printf_format_string_ const wchar_t* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    errno_t e = nlib_vsnwprintf_fallback(count, buf, N, fmt, args);
    va_end(args);
    return e;
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_strcpy(
    char (&s1)[N],
    const char* s2) NLIB_NOEXCEPT {
    return nlib_strcpy(&s1[0], N, s2);
}

template<size_t N>
NLIB_ALWAYS_INLINE size_t nlib_strlcpy(char (&s1)[N], const char* s2) NLIB_NOEXCEPT {
    // CTR armcc needs cast
    return nlib_strlcpy((char*)&s1[0], (const char*)s2, N);  // NOLINT
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_strncpy(
    char (&s1)[N],
    const char* s2,
    size_t n) NLIB_NOEXCEPT {
    return nlib_strncpy(&s1[0], N, s2, n);
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_wcscpy(
    wchar_t (&s1)[N],
    const wchar_t* s2) NLIB_NOEXCEPT {
    return nlib_wcscpy(&s1[0], N, s2);
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_wcsncpy(
    wchar_t (&s1)[N],
    const wchar_t* s2,
    size_t n) NLIB_NOEXCEPT {
    return nlib_wcsncpy(&s1[0], N, s2, n);
}

template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_wide_to_utf8(size_t* result, nlib_utf8_t (&utf8)[N],
                                             const wchar_t* wcstr) NLIB_NOEXCEPT {
    return nlib_wide_to_utf8(result, &utf8[0], N, wcstr);
}

template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_utf8_to_wide(size_t* result, wchar_t (&wcstr)[N],
                                             const nlib_utf8_t* utf8) NLIB_NOEXCEPT {
    return nlib_utf8_to_wide(result, &wcstr[0], N, utf8);
}

template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_utf16_to_utf8(size_t* utf8count, nlib_utf8_t (&utf8)[N],
                                              const nlib_utf16_t* utf16) NLIB_NOEXCEPT {
    return nlib_utf16_to_utf8(utf8count, &utf8[0], N, utf16);
}

template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_utf8_to_utf16(size_t* utf16count, nlib_utf16_t (&utf16)[N],
                                              const nlib_utf8_t* utf8) NLIB_NOEXCEPT {
    return nlib_utf8_to_utf16(utf16count, &utf16[0], N, utf8);
}

template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_utf32_to_utf8(size_t* utf8count, nlib_utf8_t (&utf8)[N],
                                              const nlib_utf32_t* utf32) NLIB_NOEXCEPT {
    return nlib_utf32_to_utf8(utf8count, &utf8[0], N, utf32);
}

template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_utf8_to_utf32(size_t* utf32count, nlib_utf32_t(&utf32)[N],
                                              const nlib_utf8_t* utf8) NLIB_NOEXCEPT {
    return nlib_utf8_to_utf32(utf32count, &utf32[0], N, utf8);
}

template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_memutf16_to_utf8(
    size_t* to_count, size_t* from_count,
    nlib_utf8_t (&to)[N],
    const nlib_utf16_t* from, size_t from_size) NLIB_NOEXCEPT {
    return nlib_memutf16_to_utf8(to_count, from_count, &to[0], N, from, from_size);
}
template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_memutf8_to_utf16(
    size_t* to_count, size_t* from_count,
    nlib_utf16_t (&to)[N],
    const nlib_utf8_t* from, size_t from_size) NLIB_NOEXCEPT {
    return nlib_memutf8_to_utf16(to_count, from_count, &to[0], N, from, from_size);
}
template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_memutf32_to_utf8(
    size_t* to_count, size_t* from_count,
    nlib_utf8_t (&to)[N],
    const nlib_utf32_t* from, size_t from_size) NLIB_NOEXCEPT {
    return nlib_memutf32_to_utf8(to_count, from_count, &to[0], N, from, from_size);
}
template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_memutf8_to_utf32(
    size_t* to_count, size_t* from_count,
    nlib_utf32_t (&to)[N],
    const nlib_utf8_t* from, size_t from_size) NLIB_NOEXCEPT {
    return nlib_memutf8_to_utf32(to_count, from_count, &to[0], N, from, from_size);
}
template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_memwide_to_utf8(
    size_t* to_count, size_t* from_count,
    nlib_utf8_t (&to)[N],
    const wchar_t* from, size_t from_size) NLIB_NOEXCEPT {
    return nlib_memwide_to_utf8(to_count, from_count, &to[0], N, from, from_size);
}
template<size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_memutf8_to_wide(
    size_t* to_count, size_t* from_count,
    wchar_t (&to)[N],
    const nlib_utf8_t* from, size_t from_size) NLIB_NOEXCEPT {
    return nlib_memutf8_to_wide(to_count, from_count, &to[0], N, from, from_size);
}

#if !defined(NN_PLATFORM_CTR) && !defined(CAFE)
// This may throw C++ exception defined by nlib, See also nlib_thread_exit()
NLIB_NORETURN NLIB_VIS_PUBLIC void nlib_thread_exit_cpp();
#endif

NLIB_ALWAYS_INLINE bool nlib_is_error(bool result) NLIB_NOEXCEPT { return !result; }
NLIB_ALWAYS_INLINE bool nlib_is_error(errno_t e) NLIB_NOEXCEPT { return e != 0; }
NLIB_ALWAYS_INLINE bool nlib_is_error(const NLIB_NS::ErrnoT& e) NLIB_NOEXCEPT { return e != 0; }
template<class T>
NLIB_ALWAYS_INLINE bool nlib_is_error(const T& obj) NLIB_NOEXCEPT {
#ifdef NLIB_CXX11_EXPLICIT_CONVERSION_OPERATORS
    return !obj.operator bool();
#else
    return !obj;
#endif
}
#ifdef NLIB_CXX11_DEFAULTED_AND_DELETED_FUNCTIONS
template<class T>
bool nlib_is_error(T*) = delete;  // NOLINT
#else
template<class T>
bool nlib_is_error(T*);  // NOLINT
#endif

#endif  // __cplusplus

#if defined(_MSC_VER)
#if defined(n_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#elif defined(nx_misc_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif
#endif

#endif  // INCLUDE_NN_NLIB_CONFIG_H_
