﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_CSTRING_H_
#define INCLUDE_NN_NLIB_CSTRING_H_

#include <stdarg.h>
#include <string.h>

#include "nn/nlib/Config.h"

NLIB_NAMESPACE_BEGIN

inline size_t StrLen(const char* str) NLIB_NOEXCEPT { return nlib_strlen(str); }
inline size_t StrLen(const nlib_utf16_t* str) NLIB_NOEXCEPT { return nlib_utf16len(str); }
inline size_t StrLen(const nlib_utf32_t* str) NLIB_NOEXCEPT { return nlib_utf32len(str); }
inline size_t StrLen(const wchar_t* str) NLIB_NOEXCEPT { return nlib_wcslen(str); }

inline errno_t CodePointCount(
    const char* str,
    size_t* count) NLIB_NOEXCEPT {
    return nlib_strcplen(count, nullptr, nullptr, str);
}
inline errno_t CodePointCount(
    const nlib_utf16_t* str,
    size_t* count) NLIB_NOEXCEPT {
    return nlib_utf16cplen(count, str);
}
inline errno_t CodePointCount(
    const nlib_utf32_t* str,
    size_t* count) NLIB_NOEXCEPT {
    return nlib_utf32cplen(count, str);
}
inline errno_t CodePointCount(
    const wchar_t* str,
    size_t* count) NLIB_NOEXCEPT {
    return nlib_wcscplen(count, str);
}

inline int StrCmp(const char* s1, const char* s2) NLIB_NOEXCEPT {
    return strcmp(s1, s2);
}
inline int StrCmp(const wchar_t* s1, const wchar_t* s2) NLIB_NOEXCEPT {
    return wcscmp(s1, s2);
}
inline int StrCmp(const nlib_utf16_t* s1, const nlib_utf16_t* s2) NLIB_NOEXCEPT {
    for (; *s1 == *s2; ++s1, ++s2) {
        if (NLIB_UNLIKELY(*s1 == '\0')) return 0;
    }
    return *s1 < *s2 ? -1 : 1;
}
inline int StrCmp(const nlib_utf32_t* s1, const nlib_utf32_t* s2) NLIB_NOEXCEPT {
    for (; *s1 == *s2; ++s1, ++s2) {
        if (NLIB_UNLIKELY(*s1 == '\0')) return 0;
    }
    return *s1 < *s2 ? -1 : 1;
}

inline int StrNcmp(const char* s1,
                   const char* s2,
                   size_t n) NLIB_NOEXCEPT {
    return strncmp(s1, s2, n);
}
inline int StrNcmp(const wchar_t* s1,
                   const wchar_t* s2,
                   size_t n) NLIB_NOEXCEPT {
    return wcsncmp(s1, s2, n);
}
inline int StrNcmp(const nlib_utf16_t* s1,
                   const nlib_utf16_t* s2,
                   size_t n) NLIB_NOEXCEPT {
    for (; n > 0; ++s1, ++s2, --n) {
        if (NLIB_UNLIKELY(*s1 != *s2)) {
            return *s1 < *s2 ? -1 : 1;
        } else if (NLIB_UNLIKELY(*s1 == '\0')) {
            return 0;
        }
    }
    return 0;
}
inline int StrNcmp(const nlib_utf32_t* s1,
                   const nlib_utf32_t* s2,
                   size_t n) NLIB_NOEXCEPT {
    for (; n > 0; ++s1, ++s2, --n) {
        if (NLIB_UNLIKELY(*s1 != *s2)) {
            return *s1 < *s2 ? -1 : 1;
        } else if (NLIB_UNLIKELY(*s1 == '\0')) {
            return 0;
        }
    }
    return 0;
}

inline errno_t StrCpy(char* dest,
                      size_t dest_size,
                      const char* src) NLIB_NOEXCEPT {
    return nlib_strcpy(dest, dest_size, src);
}
inline errno_t StrCpy(nlib_utf16_t* dest,
                      size_t dest_size,
                      const nlib_utf16_t* src) NLIB_NOEXCEPT {
    return nlib_utf16cpy(dest, dest_size, src);
}
inline errno_t StrCpy(nlib_utf32_t* dest,
                      size_t dest_size,
                      const nlib_utf32_t* src) NLIB_NOEXCEPT {
    return nlib_utf32cpy(dest, dest_size, src);
}
inline errno_t StrCpy(wchar_t* dest,
                      size_t dest_size,
                      const wchar_t* src) NLIB_NOEXCEPT {
    return nlib_wcscpy(dest, dest_size, src);
}

template<class T, size_t N>
NLIB_ALWAYS_INLINE errno_t StrCpy(T (&dest)[N],
                                  const T* src) NLIB_NOEXCEPT {
    return StrCpy(&dest[0], N, src);
}

inline errno_t StrNcpy(char* dest,
                       size_t dest_size,
                       const char* src,
                       size_t n) NLIB_NOEXCEPT {
    return nlib_strncpy(dest, dest_size, src, n);
}
inline errno_t StrNcpy(nlib_utf16_t* dest,
                       size_t dest_size,
                       const nlib_utf16_t* src,
                       size_t n) NLIB_NOEXCEPT {
    return nlib_utf16ncpy(dest, dest_size, src, n);
}
inline errno_t StrNcpy(nlib_utf32_t* dest,
                       size_t dest_size,
                       const nlib_utf32_t* src,
                       size_t n) NLIB_NOEXCEPT {
    return nlib_utf32ncpy(dest, dest_size, src, n);
}
inline errno_t StrNcpy(wchar_t* dest,
                       size_t dest_size,
                       const wchar_t* src,
                       size_t n) NLIB_NOEXCEPT {
    return nlib_wcsncpy(dest, dest_size, src, n);
}

template<class T, size_t N>
NLIB_ALWAYS_INLINE errno_t StrNcpy(T (&dest)[N],
                                   const T* src,
                                   size_t n) NLIB_NOEXCEPT {
    return StrNcpy(&dest[0], N, src, n);
}

template<class T, size_t N>
NLIB_ALWAYS_INLINE errno_t StrCat(T (&dest)[N],
                                  const T* src) NLIB_NOEXCEPT {
    return StrCat(&dest[0], N, src);
}

template<class T, size_t N>
NLIB_ALWAYS_INLINE errno_t StrNcat(T (&dest)[N],
                                   const T* src,
                                   size_t n) NLIB_NOEXCEPT {
    return StrNcat(&dest[0], N, src, n);
}

// NOTE: It is possible that NLIB_VSNPRINTF is VsnPrintfFallback.
NLIB_VIS_PUBLIC int VsnPrintfFallback(
    char* buf,
    size_t size,
    _Printf_format_string_ const char* fmt,
    va_list args) NLIB_NOEXCEPT;

inline int VsnPrintf(
    char* buf,
    size_t size,
    _Printf_format_string_ const char* fmt,
    va_list args) NLIB_NOEXCEPT {
    size_t count;
    errno_t e = nlib_vsnprintf(&count, buf, size, fmt, args);
    if (e != 0) errno = e;
    return (e == 0) ? static_cast<int>(count) : -1;
}

template <size_t N>
NLIB_ALWAYS_INLINE int VsnPrintf(
    char (&buf)[N],
    _Printf_format_string_ const char* fmt,
    va_list args) NLIB_NOEXCEPT {
    return VsnPrintf(buf, N, fmt, args);
}

inline int SnPrintf(
    char* buf,
    size_t size,
    _Printf_format_string_ const char* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintf(buf, size, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
inline NLIB_VIS_HIDDEN int SnPrintf(
    char (&buf)[N],
    _Printf_format_string_ const char* fmt,
    ...) NLIB_NOEXCEPT {
    // gcc cannot inline vaargs functions
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintf(buf, N, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
NLIB_ALWAYS_INLINE int VsnPrintfFallback(
    char (&buf)[N],
    _Printf_format_string_ const char* fmt,
    va_list args) NLIB_NOEXCEPT {
    return VsnPrintfFallback(buf, N, fmt, args);
}

inline int SnPrintfFallback(
    char* buf,
    size_t size,
    _Printf_format_string_ const char* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintfFallback(buf, size, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
inline NLIB_VIS_HIDDEN int SnPrintfFallback(
    char (&buf)[N],
    _Printf_format_string_ const char* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintfFallback(buf, N, fmt, args);
    va_end(args);
    return rval;
}

NLIB_VIS_PUBLIC int VsnPrintfFallback(
    wchar_t* buf,
    size_t size,
    _Printf_format_string_ const wchar_t* fmt,
    va_list args) NLIB_NOEXCEPT;

inline int VsnPrintf(
    wchar_t* buf,
    size_t size,
    _Printf_format_string_ const wchar_t* fmt,
    va_list args) NLIB_NOEXCEPT {
    size_t count;
    errno_t e = nlib_vsnwprintf(&count, buf, size, fmt, args);
    if (e != 0) errno = e;
    return (e == 0) ? static_cast<int>(count) : -1;
}

template <size_t N>
NLIB_ALWAYS_INLINE int VsnPrintf(
    wchar_t (&buf)[N],
    _Printf_format_string_ const wchar_t* fmt,
    va_list args) NLIB_NOEXCEPT {
    return VsnPrintf(buf, N, fmt, args);
}

inline int SnPrintf(
    wchar_t* buf,
    size_t size,
    _Printf_format_string_ const wchar_t* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintf(buf, size, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
inline NLIB_VIS_HIDDEN int SnPrintf(
    wchar_t (&buf)[N],
    _Printf_format_string_ const wchar_t* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintf(buf, N, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
NLIB_ALWAYS_INLINE int VsnPrintfFallback(
    wchar_t (&buf)[N],
    _Printf_format_string_ const wchar_t* fmt,
    va_list args) NLIB_NOEXCEPT {
    return VsnPrintfFallback(buf, N, fmt, args);
}

inline int SnPrintfFallback(
    wchar_t* buf,
    size_t size,
    _Printf_format_string_ const wchar_t* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintfFallback(buf, size, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
inline NLIB_VIS_HIDDEN int SnPrintfFallback(
    wchar_t (&buf)[N],
    _Printf_format_string_ const wchar_t* fmt,
    ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintfFallback(buf, N, fmt, args);
    va_end(args);
    return rval;
}

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_CSTRING_H_
