﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_CURLINPUTSTREAM_H_
#define INCLUDE_NN_NLIB_CURLINPUTSTREAM_H_

#include "nn/nlib/Platform.h"
#ifdef NLIB_HAS_LIBCURL

#ifndef CAFE
#include <curl/curl.h>  // NOLINT
#else
#include <cafe/curl/curl.h>  // NOLINT
#endif
#include "nn/nlib/InputStream.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC CurlInputStream NLIB_FINAL : public InputStream {
 public:
    enum BufferSize {
        kBufferSize1x = 0,
        kBufferSize2x,
        kBufferSize3x,
        kBufferSize4x,
        BUFFER_SIZE_1X = kBufferSize1x,
        BUFFER_SIZE_2X = kBufferSize2x,
        BUFFER_SIZE_3X = kBufferSize3x,
        BUFFER_SIZE_4X = kBufferSize4x
    };

 public:
    NLIB_CEXPR CurlInputStream() NLIB_NOEXCEPT : prv_(nullptr) {}
    virtual ~CurlInputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    CURL* GetEasyHandle() const NLIB_NOEXCEPT;
    CURLcode GetCurlError() const NLIB_NOEXCEPT;
    CURLMcode GetCurlMultiError() const NLIB_NOEXCEPT;
    NLIB_CHECK_RESULT errno_t Init(BufferSize buffer_size) NLIB_NOEXCEPT;
    NLIB_CHECK_RESULT errno_t Init() NLIB_NOEXCEPT { return Init(kBufferSize1x); }
    CURLcode SetUrl(const char* url) NLIB_NOEXCEPT;
    bool StartDownload() NLIB_NOEXCEPT;

 private:
    NLIB_VIS_HIDDEN static size_t ParseStreamCallback(char* contents, size_t length,
                                                      size_t nmemb, void *userp) NLIB_NOEXCEPT;
    virtual size_t FillBuffer_(void* p, size_t nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer_(size_t* nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE;
    struct CallbackInfo {
        unsigned char* buf;
        size_t pos;
        size_t bufsize;
    };

 private:
    struct CurlInputStreamPrivate;
    CurlInputStreamPrivate* prv_;
};

NLIB_NAMESPACE_END

#endif  // NLIB_HAS_LIBCURL
#endif  // INCLUDE_NN_NLIB_CURLINPUTSTREAM_H_
