﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_DYNAMICALIGNEDSTORAGE_H_
#define INCLUDE_NN_NLIB_DYNAMICALIGNEDSTORAGE_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/Swap.h"

NLIB_NAMESPACE_BEGIN

class DynamicAlignedStorage NLIB_FINAL {
 public:
    NLIB_CEXPR DynamicAlignedStorage() NLIB_NOEXCEPT : storage_(nullptr) {}
#ifdef __cpp_rvalue_references
    DynamicAlignedStorage(DynamicAlignedStorage&& rhs) NLIB_NOEXCEPT
        : storage_(rhs.storage_) {
        rhs.storage_ = nullptr;
    }
    DynamicAlignedStorage& operator=(DynamicAlignedStorage&& rhs) NLIB_NOEXCEPT {
        storage_ = rhs.storage_;
        rhs.storage_ = nullptr;
        return *this;
    }
#endif
    DynamicAlignedStorage(DynamicAlignedStorage& rhs, move_tag) NLIB_NOEXCEPT  // NOLINT
#ifdef NN_PLATFORM_CTR
        : storage_(rhs.storage_), align_(rhs.align_)
#else
        : storage_(rhs.storage_)
#endif
    {
        rhs.storage_ = nullptr;
    }
    DynamicAlignedStorage& assign(DynamicAlignedStorage& rhs, move_tag) NLIB_NOEXCEPT {  // NOLINT
#ifdef NN_PLATFORM_CTR
        align_ = rhs.align_;
#endif
        storage_ = rhs.storage_;
        rhs.storage_ = nullptr;
        return *this;
    }
    NLIB_DEPRECATED void swap(DynamicAlignedStorage& rhs) NLIB_NOEXCEPT {
        void* tmp = rhs.storage_;
        rhs.storage_ = storage_;
        storage_ = tmp;
    }
    errno_t Init(size_t size, size_t align) NLIB_NOEXCEPT {
        if (align == 0) return EINVAL;
        if ((align & (align - 1)) != 0) return EINVAL;
#ifdef NN_PLATFORM_CTR
        void* p = nlib_malloc(size + align);
#elif defined(_MSC_VER)
        void* p = _aligned_malloc(size, align);
#else
        void* p = nlib_memalign(align, size);
#endif
        if (!p) return ENOMEM;
        storage_ = p;
#ifdef NN_PLATFORM_CTR
        align_ = align;
#endif
        return 0;
    }
    ~DynamicAlignedStorage() NLIB_NOEXCEPT {
#ifdef _MSC_VER
        _aligned_free(storage_);
#else
        nlib_free(storage_);
#endif
    }
    void* Get() NLIB_NOEXCEPT {
#ifndef NN_PLATFORM_CTR
        return storage_;
#else
        uintptr_t pos = (reinterpret_cast<uintptr_t>(storage_) + (align_ - 1)) & ~(align_ - 1);
        return reinterpret_cast<void*>(pos);
#endif
    }

 private:
    void* storage_;
#ifdef NN_PLATFORM_CTR
    size_t align_;
#endif
    NLIB_DISALLOW_COPY_AND_ASSIGN(DynamicAlignedStorage);
};

NLIB_NAMESPACE_END

NLIB_DEFINE_STD_SWAP(NLIB_NS::DynamicAlignedStorage)

#endif  // INCLUDE_NN_NLIB_DYNAMICALIGNEDSTORAGE_H_
