﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MEMORYINPUTSTREAM_H_
#define INCLUDE_NN_NLIB_MEMORYINPUTSTREAM_H_

#include "nn/nlib/InputStream.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC MemoryInputStream NLIB_FINAL : public InputStream {
 public:
    NLIB_CEXPR MemoryInputStream() NLIB_NOEXCEPT : mis_buf_(nullptr), mis_size_(0) {}
    virtual ~MemoryInputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    template <class T, size_t N>
    NLIB_CEXPR explicit MemoryInputStream(const T (&buf)[N]) NLIB_NOEXCEPT
        : mis_buf_(static_cast<const nlib_byte_t*>(static_cast<const void*>(&buf[0]))),
          mis_size_(N * sizeof(T)) {
        // T must be pod
    }
    NLIB_CEXPR MemoryInputStream(const void* buf, size_t n) NLIB_NOEXCEPT
        : mis_buf_(static_cast<const nlib_byte_t*>(buf)), mis_size_(n) {}
    template <class T, size_t N>
    errno_t Init(const T (&buf)[N]) NLIB_NOEXCEPT {
        return this->Init(&buf[0], N * sizeof(T));
    }
    errno_t Init(const nlib_byte_t* buf, size_t n) NLIB_NOEXCEPT NLIB_NONNULL;
    errno_t Init(const void* buf, size_t n) NLIB_NOEXCEPT {
        return Init(static_cast<const nlib_byte_t*>(buf), n);
    }

 private:
    virtual size_t FillBuffer_(void* p, size_t nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE {
        mis_buf_ = nullptr;
        mis_size_ = 0;
        return true;
    }

 private:
    const nlib_byte_t* mis_buf_;
    size_t mis_size_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(MemoryInputStream);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_MEMORYINPUTSTREAM_H_
