﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MEMORYOUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_MEMORYOUTPUTSTREAM_H_

#include "nn/nlib/OutputStream.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC MemoryOutputStream NLIB_FINAL : public OutputStream {
 public:
    NLIB_CEXPR MemoryOutputStream() NLIB_NOEXCEPT : mos_buf_(nullptr), mos_pos_(0), mos_size_(0) {}
    NLIB_CEXPR MemoryOutputStream(void* buf, size_t n) NLIB_NOEXCEPT
        : mos_buf_(static_cast<nlib_byte_t*>(buf)), mos_pos_(0), mos_size_(n) {}
    template <class T, size_t N>
    NLIB_CEXPR explicit MemoryOutputStream (T (&buf)[N]) NLIB_NOEXCEPT
        : mos_buf_(static_cast<nlib_byte_t*>(static_cast<void*>(&buf[0]))),
          mos_pos_(0),
          mos_size_(N * sizeof(T)) {}
    virtual ~MemoryOutputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    const void* data() const NLIB_NOEXCEPT { return mos_buf_; }
    errno_t Init(nlib_byte_t* buf, size_t n) NLIB_NOEXCEPT NLIB_NONNULL;
    errno_t Init(void* buf, size_t n) NLIB_NOEXCEPT NLIB_NONNULL {
        return Init(static_cast<nlib_byte_t*>(buf), n);
    }
    template <class T, size_t N>
    errno_t Init(T (&buf)[N]) NLIB_NOEXCEPT {
        return this->Init(&buf[0], N * sizeof(T));
    }

 private:
    virtual bool PushBuffer_(const void* p, size_t nbytes,
                             bool do_flush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE {
        mos_buf_ = nullptr;
        mos_pos_ = 0;
        mos_size_ = 0;
        return true;
    }
    virtual void* GetWorkBuffer_(size_t* nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    nlib_byte_t* mos_buf_;
    size_t mos_pos_;
    size_t mos_size_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(MemoryOutputStream);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_MEMORYOUTPUTSTREAM_H_
