﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_PLATFORM_WIN32_H_
#define INCLUDE_NN_NLIB_PLATFORM_WIN32_H_
#ifndef INCLUDE_NN_NLIB_PLATFORM_H_
# error do not include directly
#endif

#if defined(_MSC_VER)

#if !defined(_M_IX86) && !defined(_M_X64) && !defined(_M_AMD64)
#error Sorry, not supported
#endif

#if _MSC_VER < 1800
# error Use Visual Studio 2013 or later
#endif

#ifndef WIN32_LEAN_AND_MEAN
# pragma message(__FILE__ ": WIN32_LEAN_AND_MEAN not defined, compile may fail")
# define WIN32_LEAN_AND_MEAN
#endif

#ifndef NOMINMAX
# pragma message(__FILE__ ": NOMINMAX not defined, compile may fail")
# define NOMINMAX
#endif

#ifndef _USE_MATH_DEFINES
# pragma message(__FILE__ ": _USE_MATH_DEFINES not defined, compile may fail")
# define _USE_MATH_DEFINES
#endif

#ifdef _WIN64
# define NLIB_64BIT
#endif

#include <intrin.h>

#pragma intrinsic(_BitScanReverse)
#pragma intrinsic(_BitScanForward)
#ifdef NLIB_64BIT
# pragma intrinsic(_BitScanReverse64)
# pragma intrinsic(_BitScanForward64)
#endif

#include <stdlib.h>  // for malloc, free, calloc, realloc
#include <limits.h>  // for RSIZE_MAX
#include <WinSock2.h>
#include <MSWsock.h>
#include <Windows.h>  // for HANDLE, CONDITION_VARIABLE, ....
#include <fcntl.h>  // for O_RDONLY, O_RDWR, O_CREAT, ....

#include <in6addr.h>
#include <ws2ipdef.h>

#if WINVER < 0x0603
# pragma message(__FILE__ ": Please update your Windows SDK")
#endif

#if (!defined(_M_IX86_FP) || _M_IX86_FP < 2) && !defined(_M_X64) && !defined(_M_AMD64)
# error /arch:SSE2 or higher must be set
#endif

#ifndef NLIB_SSE41
# define NLIB_SSE41
#endif

#ifndef NLIB_SSE42
# define NLIB_SSE42
#endif

// http://www.open-std.org/jtc1/sc22/wg21/docs/papers/2016/p0096r3.html
// http://en.cppreference.com/w/cpp/compiler_support
// http://en.cppreference.com/w/User:D41D8CD98F/feature_testing_macros
// https://blogs.msdn.microsoft.com/vcblog/2016/01/22/vs-2015-update-2s-stl-is-c17-so-far-feature-complete/
#ifdef __cplusplus
#define NLIB_CXX11_SWAPHEADER
#define NLIB_CXX11_STDLIB_CHRONO
#define NLIB_CXX11_STDLIB_ATOMIC
#define NLIB_CXX11_STDLIB_ARRAY
#define NLIB_CXX11_STDLIB_UNORDERED
#define NLIB_CXX11_STDLIB_TUPLE
#define NLIB_HAS_NATIVE_TYPETRAITS
#define NLIB_CXX11_UNIQUEPTR
#define NLIB_CXX11_EXPLICIT_VIRTUAL_OVERRIDES
#define NLIB_CXX11_EXPLICIT_CONVERSION_OPERATORS
#define NLIB_CXX11_STDLIB_RANDOM
#define NLIB_CXX11_DEFAULT_TEMPLATE_ARGUMENT_FOR_FUNCTION_TEMPLATES
#define NLIB_CXX11_NULL_POINTER_CONSTANT

#define NLIB_CXX11_STATIC_ASSERTIONS
#ifndef __cpp_static_assert
#define __cpp_static_assert 200410L  // N1720
#endif

#define NLIB_CXX11_RVALUE_REFERENCES
#ifndef __cpp_rvalue_references
#define __cpp_rvalue_references 200610L  // N2118
#endif

#define NLIB_CXX11_LAMBDAS
#ifndef __cpp_lambdas
#define __cpp_lambdas 200907L  // N2927
#endif

#define NLIB_CXX11_RANGE_BASED_FOR
#ifndef __cpp_range_based_for
#define __cpp_range_based_for 200907L  // N2930
#endif

#define NLIB_CXX11_TEMPLATE_ALIAS
#ifndef __cpp_alias_templates
#define __cpp_alias_templates 200704L  // N2258
#endif

#define NLIB_CXX11_VARIADIC_TEMPLATES
#ifndef __cpp_variadic_templates
#define __cpp_variadic_templates 200704L  // N2242
#endif

#define NLIB_CXX11_DELEGATING_CONSTRUCTORS
#ifndef __cpp_delegating_constructors
#define __cpp_delegating_constructors 200604L  // N1986
#endif

#ifndef __cpp_lib_make_unique
#define __cpp_lib_make_unique 201304L  // N3656
#endif
#ifndef __cpp_lib_transparent_operators
#define __cpp_lib_transparent_operators 201210L  // N3421
#endif
#ifndef __cpp_lib_transformation_trait_aliases
#define __cpp_lib_transformation_trait_aliases 201304L  // N3655
#endif
#ifndef __cpp_lib_incomplete_container_elements
#define __cpp_lib_incomplete_container_elements 201505L  // N4510
#endif

#if _MSC_VER >= 1900
#define NLIB_CXX11_NOEXCEPT
#define NLIB_CXX11_ALIGNMENT_SUPPORT
#define NLIB_CXX11_DEFAULTED_AND_DELETED_FUNCTIONS

#define NLIB_CXX11_CONSTEXPR
#ifndef __cpp_constexpr
#define __cpp_constexpr 200704  // N2235
#endif

#define NLIB_CXX11_NEW_CHARACTER_TYPES
#ifndef __cpp_unicode_characters
#define __cpp_unicode_characters 200704L  // N2249
#endif

// C++11
#ifndef __cpp_raw_strings
#define __cpp_raw_strings 200710L  // N2442
#endif
#ifndef __cpp_unicode_literals
#define __cpp_unicode_literals 200710L  // N2442
#endif
#ifndef __cpp_user_defined_literals
#define __cpp_user_defined_literals 200809L  // N2765
#endif
#ifndef __cpp_decltype
#define __cpp_decltype 200707L  // N2343
#endif
#ifndef __cpp_attributes
#define __cpp_attributes 200809L  // N2761
#endif
#ifndef __cpp_initializer_lists
#define __cpp_initializer_lists 200806L  // N2672
#endif
#ifndef __cpp_nsdmi
#define __cpp_nsdmi 200809L  // N2756
#endif
#ifndef __cpp_inheriting_constructors
#define __cpp_inheriting_constructors 200802L  // N2540
#endif
#ifndef __cpp_ref_qualifiers
#define __cpp_ref_qualifiers 200710L  // N2439
#endif

// C++14
#ifndef __cpp_binary_literals
#define __cpp_binary_literals 201304L  // N3472
#endif
#ifndef __cpp_init_captures
#define __cpp_init_captures 201304L  // N3648
#endif
#ifndef __cpp_generic_lambdas
#define __cpp_generic_lambdas 201304L  // N3649
#endif
#ifndef __cpp_sized_deallocation
#define __cpp_sized_deallocation 201309L  // N3778
#endif
#ifndef __cpp_decltype_auto
#define __cpp_decltype_auto 201304L  // N3638
#endif
#ifndef __cpp_return_type_deduction
#define __cpp_return_type_deduction 201304L  // N3638
#endif
#ifndef __cpp_variable_templates
#define __cpp_variable_templates 201304L  // N3651
#endif
#ifndef __cpp_lib_is_null_pointer
#define __cpp_lib_is_null_pointer 201309L  // LWG2247
#endif
#ifndef __cpp_lib_integer_sequence
#define __cpp_lib_integer_sequence 201304L  // N3658
#endif
#ifndef __cpp_lib_exchange_function
#define __cpp_lib_exchange_function 201304L  // N3668
#endif
#ifndef __cpp_lib_tuples_by_type
#define __cpp_lib_tuples_by_type 201304L  // N3670
#endif
#ifndef __cpp_lib_tuple_element_t
#define __cpp_lib_tuple_element_t 201402L  // N3887
#endif
#ifndef __cpp_lib_integral_constant_callable
#define __cpp_lib_integral_constant_callable 201304L  // N3545
#endif
#ifndef __cpp_lib_result_of_sfinae
#define __cpp_lib_result_of_sfinae 201210L  // N3642
#endif
#ifndef __cpp_lib_is_final
#define __cpp_lib_is_final 201402L  // LWG2112
#endif
#ifndef __cpp_lib_chrono_udls
#define __cpp_lib_chrono_udls 201304L  // N3642
#endif
#ifndef __cpp_lib_string_udls
#define __cpp_lib_string_udls 201304L  // N3642
#endif
#ifndef __cpp_lib_generic_associative_lookup
#define __cpp_lib_generic_associative_lookup 201304L  // N3657
#endif
#ifndef __cpp_lib_null_iterators
#define __cpp_lib_null_iterators 201304L  // N3644
#endif
#ifndef __cpp_lib_make_reverse_iterator
#define __cpp_lib_make_reverse_iterator 201402L  // LWG2285
#endif
#ifndef __cpp_lib_robust_nonmodifying_seq_ops
#define __cpp_lib_robust_nonmodifying_seq_ops 201304L  // N3671
#endif
#ifndef __cpp_lib_complex_udls
#define __cpp_lib_complex_udls 201309L  // N3779
#endif
#ifndef __cpp_lib_quoted_string_io
#define __cpp_lib_quoted_string_io 201304L  // N3654
#endif
#ifndef __cpp_lib_shared_timed_mutex
#define __cpp_lib_shared_timed_mutex 201402L  // N3891
#endif
#if _MSVC_LANG > 201402L
#ifndef __cpp_namespace_attributes
#define __cpp_namespace_attributes 201411L  // N4266
#endif
#ifndef __cpp_enumerator_attributes
#define __cpp_enumerator_attributes 201411L  // N4266
#endif
#ifndef __cpp_lib_type_trait_variable_templates
#define __cpp_lib_type_trait_variable_templates  // P0006R0
#endif
#ifndef __cpp_lib_logical_traits
#define __cpp_lib_logical_traits 201510  // P0013R1
#endif
#ifndef __cpp_lib_as_const
#define __cpp_lib_as_const 201510  // P0007R1
#endif
#ifndef __cpp_lib_transparent_operators
#define __cpp_lib_transparent_operators 201510L  // P0074R0
#endif
#ifndef __cpp_lib_chrono
#define __cpp_lib_chrono 201510  // P0092R1
#endif
#ifndef __cpp_lib_lock_guard_variadic
#define __cpp_lib_lock_guard_variadic 201510L  // P0156R0
#endif
#ifndef __cpp_lib_uncaught_exceptions
#define __cpp_lib_uncaught_exceptions 201411L  // N4259
#endif
#ifndef __cpp_lib_shared_mutex
#define __cpp_lib_shared_mutex 201505L  // N4508
#endif
#endif

#endif
#if _MSC_VER >= 1910
#define NLIB_CXX14_CONSTEXPR
#undef __cpp_constexpr
#define __cpp_constexpr 201304L  // N3652
#ifndef __cpp_aggregate_nsdmi
#define __cpp_aggregate_nsdmi 201304L  // N3653
#endif
#if _MSVC_LANG > 201402L
// #define __cpp_hex_float 201603L  // P0245R1
// #define __cpp_noexcept_function_type 201510L  // P0012R1
// #define __cpp_fold_expressions 201411L  // N4295
// #ifndef __cpp_capture_star_this
// #define __cpp_capture_star_this 201603L  // P0018R3
// #endif
// #undef __cpp_constexpr
// #define __cpp_constexpr 201603L  // P0170R1
#undef __cpp_range_based_for
#define __cpp_range_based_for 201603L  // P0184R0
#undef __cpp_static_assert
#define __cpp_static_assert 201411L  // N3928
#ifndef __cpp_nested_namespace_definitions
#define __cpp_nested_namespace_definitions 201411L  // N4230
#endif
// #define __cpp_inheriting_constructors 201511L  // P0136R1
// #define __cpp_aggregate_bases 201603L  // P0017R1
// #define __cpp_nontype_template_args 201411L  // N4268
// #define __cpp_fold_expressions 201603L  // P0036R0
// #define __cpp_lib_hardware_interference_size 201603L  // P0154R1
#endif
#endif
#endif

#include <stdint.h>
#include <inttypes.h>
#if (_MSC_VER < 1900)
# ifndef __func__
#  define __func__ __FUNCTION__
# endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef va_copy
# define va_copy(dest, src) (dest = src)
#endif
#define NLIB_ALWAYS_INLINE __forceinline
#define NLIB_NEVER_INLINE __declspec(noinline)
#define NLIB_LIKELY(x) (x)
#define NLIB_UNLIKELY(x) (x)
#define NLIB_EXPECT(var, exp_value) (var)
#define NLIB_CHECK_RESULT
#if defined(__cplusplus) && _MSC_VER >= 1900
#define NLIB_NORETURN [[noreturn]]  // NOLINT
#else
#define NLIB_NORETURN
#endif
#if defined(__cplusplus) && _MSC_VER >= 1910 && _MSVC_LANG > 201402L
#define NLIB_FALLTHROUGH [[fallthrough]]  // NOLINT
#else
#define NLIB_FALLTHROUGH
#endif
#define NLIB_NONNULL
#define NLIB_NONNULL_1
#define NLIB_NONNULL_2
#define NLIB_NONNULL_3
#define NLIB_NONNULL_4
#define NLIB_NONNULL_5
#define NLIB_ATTRIBUTE_MALLOC
#define NLIB_ATTRIBUTE_PURE
#define NLIB_ATTRIBUTE_CONST
#define NLIB_ATTRIBUTE_ALLOC_SIZE1(n)
#define NLIB_ATTRIBUTE_ALLOC_SIZE2(n0, n1)
#define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn)
#define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n)
#ifndef NLIB_DEPRECATED
#if defined(__cplusplus) && _MSC_VER >= 1910
#define NLIB_DEPRECATED [[deprecated]]  // NOLINT
#else
#define NLIB_DEPRECATED __declspec(deprecated)
#endif
#endif
#ifndef NLIB_DEPRECATED_MSG
#if defined(__cplusplus) && _MSC_VER >= 1910
#define NLIB_DEPRECATED_MSG(msg) [[deprecated(msg)]]  // NOLINT
#else
#define NLIB_DEPRECATED_MSG(msg) __declspec(deprecated(msg))
#endif
#endif
#define NLIB_WEAKSYMBOL
#define NLIB_VIS_HIDDEN
#ifdef NLIB_WINDLL
# define NLIB_EXPIMP_TEMPLATE_(x) template class NLIB_VIS_PUBLIC x
# define NLIB_WINEXPORT __declspec(dllexport)
# define NLIB_WINIMPORT __declspec(dllimport)
# define NLIB_VIS_PUBLIC NLIB_WINIMPORT
# define NLIB_EXPIMP_TEMPLATE(x) extern NLIB_EXPIMP_TEMPLATE_(x)
#else
# define NLIB_VIS_PUBLIC
# define NLIB_EXPIMP_TEMPLATE(x)
#endif

#if defined(n_EXPORTS)
# undef NLIB_VIS_PUBLIC
# define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#elif defined(nx_misc_EXPORTS)
# undef NLIB_EXPIMP_TEMPLATE
# define NLIB_EXPIMP_TEMPLATE(x) NLIB_EXPIMP_TEMPLATE_(x)
#endif

#define NLIB_VIS_PUBLIC_ALT

#define NLIB_LITTLE_ENDIAN

// NOTE:
//   _WriteBarrier() for the compiler, _mm_sfence() for the processor, ...
// NOTE:
//   You have to comment out error lines in intrin.h if you use VC++2005.
//   See:
//     https://connect.microsoft.com/VisualStudio/feedback/ViewFeedback.aspx?FeedbackID=262047
//     http://www.hydrogenaudio.org/forums/index.php?showtopic=50972
#define NLIB_MEMORY_ORDER_RELEASE                   \
    __pragma(warning(push))                         \
    __pragma(warning(disable:4127))                 \
    do { _WriteBarrier(); _mm_sfence(); } while (0) \
    __pragma(warning(pop))
#define NLIB_MEMORY_ORDER_ACQUIRE                  \
    __pragma(warning(push))                        \
    __pragma(warning(disable:4127))                \
    do { _ReadBarrier(); _mm_lfence(); } while (0) \
    __pragma(warning(pop))
#define NLIB_MEMORY_ORDER_ACQ_REL                       \
    __pragma(warning(push))                             \
    __pragma(warning(disable:4127))                     \
    do { _ReadWriteBarrier(); _mm_mfence(); } while (0) \
    __pragma(warning(pop))
#define NLIB_MEMORY_ORDER_SEQ_CST NLIB_MEMORY_ORDER_ACQ_REL

// because "%zu" not supported ...
#ifdef NLIB_64BIT
# define __PRIS_PREFIX "ll"
#else
# define __PRIS_PREFIX
#endif
#define NLIB_WARN__(x) #x
#define NLIB_WARN_(x) NLIB_WARN__(x)
#define NLIB_WARN(exp) (__FILE__ "(" NLIB_WARN_(__LINE__) ") : WARNING: " exp)

typedef unsigned int nlib_tls;

#if _MSC_VER >= 1900
#define NLIB_TIMESPEC_HAS_NATIVE
#endif

#pragma pack(push, 8)
typedef struct nlib_mutex_ {
    int32_t mark;
    uint32_t cat;
    union {
        HANDLE h;
        CRITICAL_SECTION cs;
    } data;
} nlib_mutex;
#pragma pack(pop)
#define NLIB_MUTEX_INITIALIZER { 0x6768696AU, 0 }
#define NLIB_RECURSIVE_MUTEX_INITIALIZER { 0x6768696AU, 1 }
#define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER { 0x6768696AU, 2 }

typedef HANDLE nlib_semaphore;

typedef struct nlib_cond_ {
    CONDITION_VARIABLE cond;
    nlib_mutex mutex;
} nlib_cond;
#define NLIB_COND_INITIALIZER { 0, NLIB_MUTEX_INITIALIZER }

typedef HANDLE nlib_thread;

#define NLIB_ATTRIBUTE_PRINTF(x, y)

#define NLIB_ASSUME(cond) __assume(cond)

#define NLIB_RWLOCK_HAS_NATIVE
#ifdef NLIB_RWLOCK_HAS_NATIVE
typedef SRWLOCK nlib_rwlock;
#define NLIB_RWLOCK_INITIALIZER SRWLOCK_INIT
#endif
#define NLIB_TIMER_HAS_NATIVE
#define NLIB_ONCE_HAS_NATIVE

NLIB_VIS_PUBLIC void nlib_free_size_(void* ptr, size_t size);

#define NLIB_LIBC_nlib_memcmp
#define NLIB_LIBC_nlib_strlen
#define NLIB_LIBC_nlib_strnlen
#define NLIB_LIBC_nlib_wcslen
#define NLIB_LIBC_nlib_wcsnlen
#define NLIB_LIBC_nlib_strcpy
#define NLIB_LIBC_nlib_strncpy
#define NLIB_LIBC_nlib_wcscpy
#define NLIB_LIBC_nlib_wcsncpy
#define NLIB_LIBC_nlib_strchr
#define NLIB_LIBC_nlib_strrchr

#if !defined(__INTELLISENSE__)
#define NLIB_ATOMIC_RELAXED (0)
#define NLIB_ATOMIC_ACQUIRE (1)
#define NLIB_ATOMIC_RELEASE (2)
#define NLIB_ATOMIC_ACQ_REL (3)
#define NLIB_ATOMIC_SEQ_CST (7)

//
// NOTE:
// Windows 8 has xxxNoFence version
//

static __inline int32_t nlib_atomic_load32(const int32_t* ptr, int memorder) {
    int32_t result;
    _ReadWriteBarrier();
    result = *ptr;
    (void)memorder;
    // if (memorder & NLIB_ATOMIC_ACQUIRE)
    //    NLIB_MEMORY_ORDER_ACQUIRE;
    _ReadWriteBarrier();
    return result;
}

static __inline void nlib_atomic_store32(int32_t* ptr, int32_t val, int memorder) {
    _ReadWriteBarrier();
    if (memorder != NLIB_ATOMIC_SEQ_CST) {
        // if (memorder & NLIB_ATOMIC_RELEASE)
        //    NLIB_MEMORY_ORDER_RELEASE;
        *ptr = val;
    } else {
        _InterlockedExchange((volatile long*)ptr, val);  // NOLINT
    }
    _ReadWriteBarrier();
}

static __inline int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val,
                                               int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = (int32_t)_InterlockedExchange((volatile long*)ptr, val);  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected,
                                                   int32_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    int32_t result;
    _ReadWriteBarrier();
    result = _InterlockedCompareExchange((volatile long*)ptr, desired, *expected);  // NOLINT
    _ReadWriteBarrier();
    (void)weak;
    (void)success_memorder;
    (void)failure_memorder;
    if (result == *expected) {
        return 1;
    } else {
        *expected = result;
        return 0;
    }
}

static __inline int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedExchangeAdd((volatile long*)ptr, val) + val;  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedExchangeAdd((volatile long*)ptr, -val) - val;  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedAnd((volatile long*)ptr, val) & val;  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedXor((volatile long*)ptr, val) ^ val;  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val,
                                               int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedOr((volatile long*)ptr, val) | val;  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedExchangeAdd((volatile long*)ptr, val);  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedExchangeAdd((volatile long*)ptr, -val);  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedAnd((volatile long*)ptr, val);  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val,
                                                int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedXor((volatile long*)ptr, val);  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val,
                                               int memorder) {
    int32_t result;
    (void)memorder;
    _ReadWriteBarrier();
    result = _InterlockedOr((volatile long*)ptr, val);  // NOLINT
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_load64(const int64_t* ptr, int memorder) {
    int64_t result;
    _ReadWriteBarrier();
    result = *ptr;
    _ReadWriteBarrier();
    (void)memorder;
    // if (memorder & NLIB_ATOMIC_ACQUIRE)
    //    NLIB_MEMORY_ORDER_ACQUIRE;
    return result;
}

static __inline void nlib_atomic_store64(int64_t* ptr, int64_t val, int memorder) {
    _ReadWriteBarrier();
    if (memorder != NLIB_ATOMIC_SEQ_CST) {
        // if (memorder & NLIB_ATOMIC_RELEASE)
        //    NLIB_MEMORY_ORDER_RELEASE;
        *ptr = val;
    } else {
#ifndef NLIB_64BIT
        InterlockedExchange64((volatile long long*)ptr, val);  // NOLINT
#else
        _InterlockedExchange64((volatile long long*)ptr, val);  // NOLINT
#endif
    }
    _ReadWriteBarrier();
}

static __inline int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val,
                                               int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = (int64_t)InterlockedExchange64((volatile long long*)ptr, val);  // NOLINT
#else
    result = (int64_t)_InterlockedExchange64((volatile long long*)ptr, val);  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline void* nlib_atomic_exchangeptr(void** ptr, void* val, int memorder) {
#ifndef NLIB_64BIT
    return (void*)nlib_atomic_exchange32((int32_t*)ptr, (int32_t)val, memorder);  // NOLINT
#else
    return (void*)nlib_atomic_exchange64((int64_t*)ptr, (int64_t)val, memorder);  // NOLINT
#endif
}

static __inline int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                                   int64_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    int64_t result;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedCompareExchange64((volatile long long*)ptr, desired, *expected);  // NOLINT
#else
    result = _InterlockedCompareExchange64((volatile long long*)ptr, desired, *expected);  // NOLINT
#endif
    _ReadWriteBarrier();
    (void)weak;
    (void)success_memorder;
    (void)failure_memorder;
    if (result == *expected) {
        return 1;
    } else {
        *expected = result;
        return 0;
    }
}

static __inline int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedExchangeAdd64((volatile long long*)ptr, val) + val;  // NOLINT
#else
    result = _InterlockedExchangeAdd64((volatile long long*)ptr, val) + val;  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedExchangeAdd64((volatile long long*)ptr, -val) - val;  // NOLINT
#else
    result = _InterlockedExchangeAdd64((volatile long long*)ptr, -val) - val;  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedAnd64((volatile long long*)ptr, val) & val;  // NOLINT
#else
    result = _InterlockedAnd64((volatile long long*)ptr, val) & val;  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedXor64((volatile long long*)ptr, val) ^ val;  // NOLINT
#else
    result = _InterlockedXor64((volatile long long*)ptr, val) ^ val;  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val,
                                               int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedOr64((volatile long long*)ptr, val) | val;  // NOLINT
#else
    result = _InterlockedOr64((volatile long long*)ptr, val) | val;  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedExchangeAdd64((volatile long long*)ptr, val);  // NOLINT
#else
    result = _InterlockedExchangeAdd64((volatile long long*)ptr, val);  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedExchangeAdd64((volatile long long*)ptr, -val);  // NOLINT
#else
    result = _InterlockedExchangeAdd64((volatile long long*)ptr, -val);  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedAnd64((volatile long long*)ptr, val);  // NOLINT
#else
    result = _InterlockedAnd64((volatile long long*)ptr, val);  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val,
                                                int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedXor64((volatile long long*)ptr, val);  // NOLINT
#else
    result = _InterlockedXor64((volatile long long*)ptr, val);  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val,
                                               int memorder) {
    int64_t result;
    (void)memorder;
    _ReadWriteBarrier();
#ifndef NLIB_64BIT
    result = InterlockedOr64((volatile long long*)ptr, val);  // NOLINT
#else
    result = _InterlockedOr64((volatile long long*)ptr, val);  // NOLINT
#endif
    _ReadWriteBarrier();
    return result;
}

static __inline void* nlib_atomic_loadptr(void* const* ptr, int memorder) {
    void* result;
    _ReadWriteBarrier();
    result = *ptr;
    _ReadWriteBarrier();
    (void)memorder;
    // if (memorder & NLIB_ATOMIC_ACQUIRE)
    //    NLIB_MEMORY_ORDER_ACQUIRE;
    return result;
}

static __inline void nlib_atomic_storeptr(void** ptr, void* val, int memorder) {
    _ReadWriteBarrier();
    if (memorder != NLIB_ATOMIC_SEQ_CST) {
        // if (memorder & NLIB_ATOMIC_RELEASE)
        //    NLIB_MEMORY_ORDER_RELEASE;
        *ptr = val;
    } else {
#if (_MSC_VER == 1800 && !defined(NLIB_64BIT))
        InterlockedExchangePointer((void* volatile*)ptr, val);  // NOLINT
#else
        _InterlockedExchangePointer((void* volatile*)ptr, val);  // NOLINT
#endif
    }
    _ReadWriteBarrier();
}

static __inline int nlib_atomic_compare_exchangeptr(void** ptr, void** expected,
                                                    void* desired, int weak,
                                                    int success_memorder, int failure_memorder) {
    void* result;
    _ReadWriteBarrier();
    result = _InterlockedCompareExchangePointer((void* volatile *)ptr, desired, *expected);
    _ReadWriteBarrier();
    (void)weak;
    (void)success_memorder;
    (void)failure_memorder;
    if (result == *expected) {
        return 1;
    } else {
        *expected = result;
        return 0;
    }
}

static __inline void nlib_atomic_thread_fence(int memorder) {
    switch (memorder) {
    case NLIB_ATOMIC_RELAXED:
        break;
    case NLIB_ATOMIC_ACQUIRE:
        NLIB_MEMORY_ORDER_ACQUIRE;
        break;
    case NLIB_ATOMIC_RELEASE:
        NLIB_MEMORY_ORDER_RELEASE;
        break;
    default:
        NLIB_MEMORY_ORDER_SEQ_CST;
        break;
    }
}
#endif

#ifdef __cplusplus
}
#endif

#ifndef _WIN64
#define NLIB_ALTNAME(func1, func2) \
    __pragma(comment(linker, "/alternatename:_" NLIB_STRINGIFY(func1) "=_" NLIB_STRINGIFY(func2)))
#else
#define NLIB_ALTNAME(func1, func2) \
    __pragma(comment(linker, "/alternatename:" NLIB_STRINGIFY(func1) "=" NLIB_STRINGIFY(func2)))
#endif

NLIB_ALTNAME(nlib_malloc, malloc)
NLIB_ALTNAME(nlib_free, free)
NLIB_ALTNAME(nlib_calloc, calloc)
NLIB_ALTNAME(nlib_realloc, realloc)
NLIB_ALTNAME(nlib_free_size, nlib_free_size_)

#if defined(NLIB_SIMD)
# ifndef NLIB_LIBC_nlib_strlen
NLIB_ALTNAME(nlib_strlen, nlib_strlen_simd)
# endif
# ifndef NLIB_LIBC_nlib_strlen
NLIB_ALTNAME(nlib_strnlen, nlib_strnlen_simd)
# endif
NLIB_ALTNAME(nlib_utf16len_, nlib_utf16len_simd)
NLIB_ALTNAME(nlib_utf16nlen_, nlib_utf16nlen_simd)
NLIB_ALTNAME(nlib_utf32len_, nlib_utf32len_generic)
NLIB_ALTNAME(nlib_utf32nlen_, nlib_utf32nlen_generic)
NLIB_ALTNAME(nlib_utf16cplen_ex_, nlib_utf16cplen_ex_simd)
NLIB_ALTNAME(nlib_strchr, nlib_strchr_simd)
NLIB_ALTNAME(nlib_strrchr, strrchr)
NLIB_ALTNAME(nlib_strchr_mb, nlib_strchr_mb_simd)
NLIB_ALTNAME(nlib_memcmp, nlib_memcmp_simd)
NLIB_ALTNAME(nlib_memchr, nlib_memchr_simd)
NLIB_ALTNAME(nlib_memrchr, nlib_memrchr_simd)
NLIB_ALTNAME(nlib_memchr_not, nlib_memchr_not_simd)
NLIB_ALTNAME(nlib_memchr_range_not, nlib_memchr_range_not_simd)
NLIB_ALTNAME(nlib_memchr_lt, nlib_memchr_lt_simd)
NLIB_ALTNAME(nlib_memchr_gt, nlib_memchr_gt_simd)
NLIB_ALTNAME(nlib_memchr_mb, nlib_memchr_mb_simd)
NLIB_ALTNAME(nlib_skipws, nlib_skipws_simd)
NLIB_ALTNAME(nlib_swapendian_16, nlib_swapendian_16_simd)
NLIB_ALTNAME(nlib_swapendian_32, nlib_swapendian_32_simd)
NLIB_ALTNAME(nlib_swapendian_64, nlib_swapendian_64_simd)
#else
# ifndef NLIB_LIBC_nlib_strlen
NLIB_ALTNAME(nlib_strlen, nlib_strlen_generic)
# endif
# ifndef NLIB_LIBC_nlib_strlen
NLIB_ALTNAME(nlib_strnlen, nlib_strnlen_generic)
# endif
NLIB_ALTNAME(nlib_utf16len_, nlib_utf16len_generic)
NLIB_ALTNAME(nlib_utf16nlen_, nlib_utf16nlen_generic)
NLIB_ALTNAME(nlib_utf32len_, nlib_utf32len_generic)
NLIB_ALTNAME(nlib_utf32nlen_, nlib_utf32nlen_generic)
NLIB_ALTNAME(nlib_utf16cplen_ex_, nlib_utf16cplen_ex_generic)
NLIB_ALTNAME(nlib_strchr, nlib_strchr_generic)
NLIB_ALTNAME(nlib_strrchr, nlib_strrchr_generic)
NLIB_ALTNAME(nlib_strchr_mb, nlib_strchr_mb_generic)
NLIB_ALTNAME(nlib_memcmp, nlib_memcmp_generic)
NLIB_ALTNAME(nlib_memchr, nlib_memchr_generic)
NLIB_ALTNAME(nlib_memrchr, nlib_memrchr_generic)
NLIB_ALTNAME(nlib_memchr_not, nlib_memchr_not_generic)
NLIB_ALTNAME(nlib_memchr_range_not, nlib_memchr_range_not_generic)
NLIB_ALTNAME(nlib_memchr_lt, nlib_memchr_lt_generic)
NLIB_ALTNAME(nlib_memchr_gt, nlib_memchr_gt_generic)
NLIB_ALTNAME(nlib_memchr_mb, nlib_memchr_mb_generic)
NLIB_ALTNAME(nlib_skipws, nlib_skipws_generic)
NLIB_ALTNAME(nlib_swapendian_16, nlib_swapendian_16_generic)
NLIB_ALTNAME(nlib_swapendian_32, nlib_swapendian_32_generic)
NLIB_ALTNAME(nlib_swapendian_64, nlib_swapendian_64_generic)
#endif

#endif
#endif  // INCLUDE_NN_NLIB_PLATFORM_WIN32_H_
