﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SWAP_H_
#define INCLUDE_NN_NLIB_SWAP_H_

#include "nn/nlib/Config.h"
#ifdef NLIB_CXX11_SWAPHEADER
#include <utility>  // NOLINT
#else
#include <algorithm>  // NOLINT
#endif

#ifdef __cpp_rvalue_references
#define NLIB_DEFINE_STD_SWAP(tp)
#define NLIB_DEFINE_STD_SWAP_T_BEGIN2(ns1, ns2)
#define NLIB_DEFINE_STD_SWAP_T_BEGIN3(ns1, ns2, ns3)
#define NLIB_DEFINE_STD_SWAP_T_BEGIN4(ns1, ns2, ns3, ns4)
#define NLIB_DEFINE_STD_SWAP_T_END2(ns1, ns2)
#define NLIB_DEFINE_STD_SWAP_T_END3(ns1, ns2, ns3)
#define NLIB_DEFINE_STD_SWAP_T_END4(ns1, ns2, ns3, ns4)
#define NLIB_DEFINE_STD_SWAP_T1(targ1, tp)
#define NLIB_DEFINE_STD_SWAP_T2(targ1, targ2, tp)
#else
#define NLIB_DEFINE_STD_SWAP(tp) \
namespace std { \
    template<> inline void swap<tp>(tp& lhs, tp& rhs) { /* NOLINT */ \
        tp x(lhs, nlib_ns::move_tag()); \
        lhs.assign(rhs, nlib_ns::move_tag()); \
        rhs.assign(x, nlib_ns::move_tag()); \
    } \
}
#ifndef NLIB_STD_SWAP_WORKAROUND
# define NLIB_DEFINE_STD_SWAP_T_BEGIN2(ns1, ns2) NLIB_NAMESPACE_BEGIN // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN3(ns1, ns2, ns3) \
    NLIB_NAMESPACE_BEGIN namespace ns3 { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN4(ns1, ns2, ns3, ns4) \
    NLIB_NAMESPACE_BEGIN namespace ns3 { namespace ns4 { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END2(ns1, ns2) NLIB_NAMESPACE_END // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END3(ns1, ns2, ns3) } NLIB_NAMESPACE_END // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END4(ns1, ns2, ns3, ns4) }} NLIB_NAMESPACE_END // NOLINT
#else
// This code is for the compilers which std::swap has priority over ADL.
# define NLIB_DEFINE_STD_SWAP_T_BEGIN2(ns1, ns2) namespace std { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN3(ns1, ns2, ns3) namespace std { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_BEGIN4(ns1, ns2, ns3, ns4) namespace std { // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END2(ns1, ns2) } // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END3(ns1, ns2, ns3) } // NOLINT
# define NLIB_DEFINE_STD_SWAP_T_END4(ns1, ns2, ns3, ns4) } // NOLINT
#endif
#if 1
# define NLIB_DEFINE_STD_SWAP_T1(targ1, tp) \
    template<class targ1> NLIB_ALWAYS_INLINE \
    void swap(tp<targ1>& lhs, tp<targ1>& rhs) /* NOLINT */ { \
        tp<targ1> x(lhs, nlib_ns::move_tag()); \
        lhs.assign(rhs, nlib_ns::move_tag()); \
        rhs.assign(x, nlib_ns::move_tag()); \
    }
# define NLIB_DEFINE_STD_SWAP_T2(targ1, targ2, tp) \
    template<class targ1, class targ2> NLIB_ALWAYS_INLINE \
    void swap(tp<targ1, targ2>& lhs, tp<targ1, targ2>& rhs) /* NOLINT */ { \
        tp<targ1, targ2> x(lhs, nlib_ns::move_tag()); \
        lhs.assign(rhs, nlib_ns::move_tag()); \
        rhs.assign(x, nlib_ns::move_tag()); \
    }
#else
# define NLIB_DEFINE_STD_SWAP_T1(targ1, tp) \
    template<class targ1> NLIB_ALWAYS_INLINE \
    void swap(tp<targ1>& lhs, tp<targ1>& rhs) /* NOLINT */ { \
        lhs.swap(rhs); \
    }
# define NLIB_DEFINE_STD_SWAP_T2(targ1, targ2, tp) \
    template<class targ1, class targ2> NLIB_ALWAYS_INLINE \
    void swap(tp<targ1, targ2>& lhs, tp<targ1, targ2>& rhs) /* NOLINT */ { \
        lhs.swap(rhs); \
    }
#endif
#endif

#endif  // INCLUDE_NN_NLIB_SWAP_H_
