﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_TCPINPUTSTREAM_H_
#define INCLUDE_NN_NLIB_TCPINPUTSTREAM_H_

#include "nn/nlib/Platform.h"
#ifdef NLIB_SOCKET_ENABLED

#include "nn/nlib/Config.h"
#include "nn/nlib/InputStream.h"

NLIB_NAMESPACE_BEGIN

struct TcpInputStreamSettings {
    void* buf;
    size_t buffer_size;
    TcpInputStreamSettings() : buf(nullptr), buffer_size(8192) {}
};

class NLIB_VIS_PUBLIC TcpInputStream NLIB_FINAL : public InputStream {
 public:
    TcpInputStream() NLIB_NOEXCEPT : socket_(NLIB_SOCKET_INVALID), buf_(nullptr), buf_size_(0),
        buffer_holder_(nullptr) {}
    virtual ~TcpInputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init(nlib_sock sockfd) NLIB_NOEXCEPT {
        TcpInputStreamSettings settings;
        return this->Init(sockfd, settings);
    }
    errno_t Init(nlib_sock sockfd,
                 const TcpInputStreamSettings& settings) NLIB_NOEXCEPT;

 private:
    virtual size_t FillBuffer_(void* p, size_t nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer_(size_t* nbytes) NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    nlib_sock socket_;
    uint8_t* buf_;
    size_t buf_size_;
    uint8_t* buffer_holder_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(TcpInputStream);
};

NLIB_NAMESPACE_END

#endif

#endif  // INCLUDE_NN_NLIB_TCPINPUTSTREAM_H_
