﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_HEAP_CENTRALHEAP_H_
#define INCLUDE_NN_NLIB_HEAP_CENTRALHEAP_H_

#include "nn/nlib/heap/NMalloc.h"

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace heap {

class CachedHeap;
class TlsHeapCentral;
class NLIB_VIS_PUBLIC CentralHeap NLIB_FINAL {
 public:
    typedef void (*DestructorHandler)(void* start, void* end) NLIB_NOEXCEPT_FUNCPTR;

    NLIB_CEXPR CentralHeap() NLIB_NOEXCEPT
        : central_(nullptr), use_virtual_memory_(false), heap_option_(0),
          start_(nullptr), end_(nullptr) {}
    ~CentralHeap() NLIB_NOEXCEPT { this->Finalize(); }
    errno_t Init(void* start, size_t size, uint32_t heap_option) NLIB_NOEXCEPT;
    void Finalize() NLIB_NOEXCEPT;
    NLIB_ALWAYS_INLINE void* Alloc(size_t n) NLIB_NOEXCEPT { return this->Alloc(n, 8); }
    void* Alloc(size_t n, size_t algn) NLIB_NOEXCEPT;
    errno_t Free(void* p) NLIB_NOEXCEPT;
    errno_t FreeWithSize(void* p, size_t size) NLIB_NOEXCEPT;
    size_t GetAllocSize(const void* p) NLIB_NOEXCEPT;
    errno_t Realloc(void* ptr, size_t size, void** p) NLIB_NOEXCEPT;

    bool MakeCache(CachedHeap* ptr) NLIB_NOEXCEPT;
    errno_t WalkAllocatedPtrs(nmalloc_heapwalk_callback func, void* user_ptr) NLIB_NOEXCEPT;
    errno_t QueryV(int query, va_list ap) NLIB_NOEXCEPT;
    errno_t Query(int query, ...) NLIB_NOEXCEPT;
    static const size_t kPageSize;

 private:
    TlsHeapCentral* central_;
    bool use_virtual_memory_;
    uint32_t heap_option_;
    uint8_t* start_;
    uint8_t* end_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(CentralHeap);
};

}  // namespace heap
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_HEAP_CENTRALHEAP_H_
