﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_JSONPATCH_H_
#define INCLUDE_NN_NLIB_MSGPACK_JSONPATCH_H_

#include <utility>
#include "nn/nlib/msgpack/MpObject.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {

class NLIB_VIS_PUBLIC JsonPatchBuilder NLIB_FINAL {
 public:
    JsonPatchBuilder() NLIB_NOEXCEPT;
    ~JsonPatchBuilder() NLIB_NOEXCEPT;
#ifdef __cpp_rvalue_references
#ifdef NLIB_CXX11_DEFAULTED_AND_DELETED_FUNCTIONS
    JsonPatchBuilder(JsonPatchBuilder&& rhs) = default;
    JsonPatchBuilder& operator=(JsonPatchBuilder&& rhs) = default;
#else
    JsonPatchBuilder(JsonPatchBuilder&& rhs) NLIB_NOEXCEPT : oplist_(std::move(rhs.oplist_)) {}
    JsonPatchBuilder& operator=(JsonPatchBuilder&& rhs) NLIB_NOEXCEPT {
        oplist_ = std::move(rhs.oplist_);
        return *this;
    }
#endif
#endif
    JsonPatchBuilder(JsonPatchBuilder& rhs, move_tag) NLIB_NOEXCEPT  // NOLINT
        : oplist_(rhs.oplist_, move_tag()) {}
    JsonPatchBuilder& assign(JsonPatchBuilder& rhs, move_tag) NLIB_NOEXCEPT {  // NOLINT
        oplist_.assign(rhs.oplist_, move_tag());
        return *this;
    }
    NLIB_ALWAYS_INLINE errno_t Add(const nlib_utf8_t* path, MpObject* value) NLIB_NOEXCEPT {
        return PathValueOp("add", path, value);
    }
    errno_t Remove(const nlib_utf8_t* path) NLIB_NOEXCEPT;
    NLIB_ALWAYS_INLINE errno_t Replace(const nlib_utf8_t* path, MpObject* value) NLIB_NOEXCEPT {
        return PathValueOp("replace", path, value);
    }
    NLIB_ALWAYS_INLINE errno_t Move(const nlib_utf8_t* path,
                                    const nlib_utf8_t* from) NLIB_NOEXCEPT {
        return PathFromOp("move", path, from);
    }
    NLIB_ALWAYS_INLINE errno_t Copy(const nlib_utf8_t* path,
                                    const nlib_utf8_t* from) NLIB_NOEXCEPT {
        return PathFromOp("copy", path, from);
    }
    NLIB_ALWAYS_INLINE errno_t Test(const nlib_utf8_t* path, MpObject* value) NLIB_NOEXCEPT {
        return PathValueOp("test", path, value);
    }
    errno_t Export(MpObject* patch) NLIB_NOEXCEPT;
#ifdef __cpp_rvalue_references
    std::pair<errno_t, MpObject> Export() NLIB_NOEXCEPT {
        std::pair<errno_t, MpObject> rval;
        rval.first = Export(&rval.second);
        return rval;
    }
#endif
    NLIB_DEPRECATED void swap(JsonPatchBuilder& rhs) NLIB_NOEXCEPT {
        using std::swap;
        swap(oplist_, rhs.oplist_);
    }

 private:
    errno_t ResolvePath(const nlib_utf8_t* path, MpObject* path_obj) NLIB_NOEXCEPT;
    errno_t PathValueOp(const nlib_utf8_t* opstr,
                        const nlib_utf8_t* path, MpObject* path_obj) NLIB_NOEXCEPT;
    errno_t PathFromOp(const nlib_utf8_t* opstr,
                       const nlib_utf8_t* path, const nlib_utf8_t* from) NLIB_NOEXCEPT;

 private:
    MpObject oplist_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonPatchBuilder);
};

class NLIB_VIS_PUBLIC JsonPatch NLIB_FINAL {
 public:
    static errno_t Apply(MpObject* obj, MpObject* patch, size_t* index) NLIB_NOEXCEPT;

 private:
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonPatch);
};

}  // namespace msgpack
NLIB_NAMESPACE_END
NLIB_DEFINE_STD_SWAP(::nlib_ns::msgpack::JsonPatchBuilder)
#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_JSONPATCH_H_

