﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_MPDIRECTWRITER_H_
#define INCLUDE_NN_NLIB_MSGPACK_MPDIRECTWRITER_H_

#include "nn/nlib/Config.h"

NLIB_NAMESPACE_BEGIN
namespace msgpack {
struct MpDirectWriter {
    static nlib_byte_t* WriteUint8(nlib_byte_t* p, nlib_byte_t* pend, uint8_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteUint16(nlib_byte_t* p, nlib_byte_t* pend, uint16_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteUint32(nlib_byte_t* p, nlib_byte_t* pend, uint32_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteUint64(nlib_byte_t* p, nlib_byte_t* pend, uint64_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteInt8(nlib_byte_t* p, nlib_byte_t* pend, int8_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteInt16(nlib_byte_t* p, nlib_byte_t* pend, int16_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteInt32(nlib_byte_t* p, nlib_byte_t* pend, int32_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteInt64(nlib_byte_t* p, nlib_byte_t* pend, int64_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteNil(nlib_byte_t* p, nlib_byte_t* pend) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteBoolean(nlib_byte_t* p, nlib_byte_t* pend, bool val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteFloat(nlib_byte_t* p, nlib_byte_t* pend, float val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteDouble(nlib_byte_t* p, nlib_byte_t* pend, double val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteString(nlib_byte_t* p, nlib_byte_t* pend,
                                    const char* str, uint32_t len) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteString(nlib_byte_t* p, nlib_byte_t* pend,
                                    const char* str) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteStringHeader(nlib_byte_t* p, nlib_byte_t* pend,
                                          uint32_t len) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteBin(nlib_byte_t* p, nlib_byte_t* pend,
                                 const nlib_byte_t* bin, uint32_t n) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteBinHeader(nlib_byte_t* p, nlib_byte_t* pend,
                                       uint32_t n) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteExt(nlib_byte_t* p, nlib_byte_t* pend, int8_t type,
                                 const nlib_byte_t* data, uint32_t n) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteExtHeader(nlib_byte_t* p, nlib_byte_t* pend, int8_t type,
                                       uint32_t n) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteArrayHeader(nlib_byte_t* p, nlib_byte_t* pend,
                                         uint32_t count) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteMapHeader(nlib_byte_t* p, nlib_byte_t* pend,
                                       uint32_t count) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteUnsignedInt(nlib_byte_t* p, nlib_byte_t* pend,
                                         uint8_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteUnsignedInt(nlib_byte_t* p, nlib_byte_t* pend,
                                         uint16_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteUnsignedInt(nlib_byte_t* p, nlib_byte_t* pend,
                                         uint32_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteUnsignedInt(nlib_byte_t* p, nlib_byte_t* pend,
                                         uint64_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteInt(nlib_byte_t* p, nlib_byte_t* pend, int8_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteInt(nlib_byte_t* p, nlib_byte_t* pend, int16_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteInt(nlib_byte_t* p, nlib_byte_t* pend, int32_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteInt(nlib_byte_t* p, nlib_byte_t* pend, int64_t val) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteTimestamp(nlib_byte_t* p, nlib_byte_t* pend,
                                       int64_t sec, uint32_t nsec) NLIB_NOEXCEPT;
    static nlib_byte_t* WriteTimestamp(nlib_byte_t* p, nlib_byte_t* pend,
                                       nlib_time val) NLIB_NOEXCEPT;

 private:
    static NLIB_ALWAYS_INLINE void EncU16(nlib_byte_t* p, uint16_t val) NLIB_NOEXCEPT {
#ifdef NLIB_LITTLE_ENDIAN
        val = nlib_bswap16(val);
#endif
        nlib_memcpy(p, 2, &val, 2);
    }
    static NLIB_ALWAYS_INLINE void EncU32(nlib_byte_t* p, uint32_t val) NLIB_NOEXCEPT {
#ifdef NLIB_LITTLE_ENDIAN
        val = nlib_bswap32(val);
#endif
        nlib_memcpy(p, 4, &val, 4);
    }
    static NLIB_ALWAYS_INLINE void EncU64(nlib_byte_t* p, uint64_t val) NLIB_NOEXCEPT {
#ifdef NLIB_LITTLE_ENDIAN
        val = nlib_bswap64(val);
#endif
        nlib_memcpy(p, 8, &val, 8);
    }

 private:
    MpDirectWriter();
};
#define NLIB_CHK_(p, n, pend) if (NLIB_UNLIKELY((p) + (n) > (pend))) { \
    NLIB_ASSERT((p) + (n) <= (pend)); \
    return nullptr; \
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteUint8(nlib_byte_t* p, nlib_byte_t* pend, uint8_t val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 2, pend);
    if (NLIB_UNLIKELY(p + 2 > pend)) return nullptr;
    p[0] = static_cast<nlib_byte_t>(0xcc);
    p[1] = static_cast<nlib_byte_t>(val);
    return p + 2;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteUint16(nlib_byte_t* p, nlib_byte_t* pend, uint16_t val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 3, pend);
    p[0] = static_cast<nlib_byte_t>(0xcd);
    EncU16(p + 1, val);
    return p + 3;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteUint32(nlib_byte_t* p, nlib_byte_t* pend, uint32_t val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 5, pend);
    p[0] = static_cast<nlib_byte_t>(0xce);
    EncU32(p + 1, val);
    return p + 5;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteUint64(nlib_byte_t* p, nlib_byte_t* pend, uint64_t val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 9, pend);
    p[0] = static_cast<nlib_byte_t>(0xcf);
    EncU64(p + 1, val);
    return p + 9;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteInt8(nlib_byte_t* p, nlib_byte_t* pend, int8_t val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 2, pend);
    p[0] = static_cast<nlib_byte_t>(0xd0);
    p[1] = static_cast<nlib_byte_t>(val);
    return p + 2;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteInt16(nlib_byte_t* p, nlib_byte_t* pend, int16_t val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 3, pend);
    p[0] = static_cast<nlib_byte_t>(0xd1);
    EncU16(p + 1, static_cast<uint16_t>(val));
    return p + 3;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteInt32(nlib_byte_t* p, nlib_byte_t* pend, int32_t val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 5, pend);
    p[0] = static_cast<nlib_byte_t>(0xd2);
    EncU32(p + 1, static_cast<uint32_t>(val));
    return p + 5;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteInt64(nlib_byte_t* p, nlib_byte_t* pend, int64_t val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 9, pend);
    p[0] = static_cast<nlib_byte_t>(0xd3);
    EncU64(p + 1, static_cast<uint64_t>(val));
    return p + 9;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteNil(nlib_byte_t* p, nlib_byte_t* pend) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 1, pend);
    *p = static_cast<nlib_byte_t>(0xc0);
    return p + 1;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteBoolean(nlib_byte_t* p, nlib_byte_t* pend, bool val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 1, pend);
    *p = static_cast<nlib_byte_t>(val ? 0xc3 : 0xc2);
    return p + 1;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteFloat(nlib_byte_t* p, nlib_byte_t* pend, float val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 5, pend);
    p[0] = static_cast<nlib_byte_t>(0xca);
    union {
        float f32;
        uint32_t u32;
    } tmp;
    tmp.f32 = val;
    EncU32(p + 1, tmp.u32);
    return p + 5;
}
NLIB_ALWAYS_INLINE nlib_byte_t*
MpDirectWriter::WriteDouble(nlib_byte_t* p, nlib_byte_t* pend, double val) NLIB_NOEXCEPT {
    NLIB_CHK_(p, 9, pend);
    p[0] = static_cast<nlib_byte_t>(0xcb);
    union {
        double f64;
        uint64_t u64;
    } tmp;
    tmp.f64 = val;
    EncU64(p + 1, tmp.u64);
    return p + 9;
}
inline nlib_byte_t*
MpDirectWriter::WriteStringHeader(nlib_byte_t* p, nlib_byte_t* pend,
                                  uint32_t len) NLIB_NOEXCEPT {
    if (len < 32U) {
        NLIB_CHK_(p, 1, pend);
        p[0] = static_cast<nlib_byte_t>(0xa0 | len);
        return p + 1;
    } else if (len < 256U) {
        NLIB_CHK_(p, 2, pend);
        p[0] = static_cast<nlib_byte_t>(0xd9);
        p[1] = static_cast<nlib_byte_t>(len);
        return p + 2;
    } else if (len < 65536U) {
        NLIB_CHK_(p, 3, pend);
        p[0] = static_cast<nlib_byte_t>(0xda);
        EncU16(p + 1, static_cast<uint16_t>(len));
        return p + 3;
    } else {
        NLIB_CHK_(p, 5, pend);
        p[0] = static_cast<nlib_byte_t>(0xdb);
        EncU32(p + 1, len);
        return p + 5;
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteString(nlib_byte_t* p, nlib_byte_t* pend,
                            const char* str, uint32_t len) NLIB_NOEXCEPT {
    if (len < 32U) {
        NLIB_CHK_(p, 1 + len, pend);
        p[0] = static_cast<nlib_byte_t>(0xa0 | len);
        nlib_memcpy(p + 1, len, str, len);
        return p + 1 + len;
    } else if (len < 256U) {
        NLIB_CHK_(p, 2 + len, pend);
        p[0] = static_cast<nlib_byte_t>(0xd9);
        p[1] = static_cast<nlib_byte_t>(len);
        nlib_memcpy(p + 2, len, str, len);
        return p + 2 + len;
    } else if (len < 65536U) {
        NLIB_CHK_(p, 3 + len, pend);
        p[0] = static_cast<nlib_byte_t>(0xda);
        EncU16(p + 1, static_cast<uint16_t>(len));
        nlib_memcpy(p + 3, len, str, len);
        return p + 3 + len;
    } else {
        NLIB_CHK_(p, 5 + len, pend);
        p[0] = static_cast<nlib_byte_t>(0xdb);
        EncU32(p + 1, len);
        nlib_memcpy(p + 5, len, str, len);
        return p + 5 + len;
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteString(nlib_byte_t* p, nlib_byte_t* pend,
                            const char* str) NLIB_NOEXCEPT {
    return WriteString(p, pend, str, static_cast<uint32_t>(nlib_strlen(str)));
}
inline nlib_byte_t*
MpDirectWriter::WriteBinHeader(nlib_byte_t* p, nlib_byte_t* pend, uint32_t n) NLIB_NOEXCEPT {
    if (n < 256U) {
        NLIB_CHK_(p, 2, pend);
        p[0] = static_cast<nlib_byte_t>(0xc4);
        p[1] = static_cast<nlib_byte_t>(n);
        return p + 2;
    } else if (n < 65536U) {
        NLIB_CHK_(p, 3, pend);
        p[0] = static_cast<nlib_byte_t>(0xc5);
        EncU16(p + 1, static_cast<uint16_t>(n));
        return p + 3;
    } else {
        NLIB_CHK_(p, 5, pend);
        p[0] = static_cast<nlib_byte_t>(0xc6);
        EncU32(p + 1, n);
        return p + 5;
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteBin(nlib_byte_t* p, nlib_byte_t* pend,
                         const nlib_byte_t* bin, uint32_t n) NLIB_NOEXCEPT {
    if (n < 256U) {
        NLIB_CHK_(p, 2 + n, pend);
        p[0] = static_cast<nlib_byte_t>(0xc4);
        p[1] = static_cast<nlib_byte_t>(n);
        nlib_memcpy(p + 2, n, bin, n);
        return p + 2 + n;
    } else if (n < 65536U) {
        NLIB_CHK_(p, 3 + n, pend);
        p[0] = static_cast<nlib_byte_t>(0xc5);
        EncU16(p + 1, static_cast<uint16_t>(n));
        nlib_memcpy(p + 3, n, bin, n);
        return p + 3 + n;
    } else {
        NLIB_CHK_(p, 5 + n, pend);
        p[0] = static_cast<nlib_byte_t>(0xc6);
        EncU32(p + 1, n);
        nlib_memcpy(p + 5, n, bin, n);
        return p + 5 + n;
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteExtHeader(nlib_byte_t* p, nlib_byte_t* pend, int8_t type,
                               uint32_t n) NLIB_NOEXCEPT {
    if (n < 256U) {
        switch (n) {
        case 1:
            NLIB_CHK_(p, 2, pend);
            p[0] = static_cast<nlib_byte_t>(0xd4);
            p[1] = static_cast<nlib_byte_t>(type);
            return p + 2;
        case 2:
            NLIB_CHK_(p, 2, pend);
            p[0] = static_cast<nlib_byte_t>(0xd5);
            p[1] = static_cast<nlib_byte_t>(type);
            return p + 2;
        case 4:
            NLIB_CHK_(p, 2, pend);
            p[0] = static_cast<nlib_byte_t>(0xd6);
            p[1] = static_cast<nlib_byte_t>(type);
            return p + 2;
        case 8:
            NLIB_CHK_(p, 2, pend);
            p[0] = static_cast<nlib_byte_t>(0xd7);
            p[1] = static_cast<nlib_byte_t>(type);
            return p + 2;
        case 16:
            NLIB_CHK_(p, 2, pend);
            p[0] = static_cast<nlib_byte_t>(0xd8);
            p[1] = static_cast<nlib_byte_t>(type);
            return p + 2;
        default:
            NLIB_CHK_(p, 3 + n, pend);
            p[0] = static_cast<nlib_byte_t>(0xc7);
            p[1] = static_cast<nlib_byte_t>(n);
            p[2] = static_cast<nlib_byte_t>(type);
            return p + 3;
        }
    } else if (n < 65536U) {
        NLIB_CHK_(p, 4, pend);
        p[0] = static_cast<nlib_byte_t>(0xc8);
        EncU16(p + 1, static_cast<uint16_t>(n));
        p[3] = static_cast<nlib_byte_t>(type);
        return p + 4;
    } else {
        NLIB_CHK_(p, 6, pend);
        p[0] = static_cast<nlib_byte_t>(0xc9);
        EncU32(p + 1, n);
        p[5] = static_cast<nlib_byte_t>(type);
        return p + 6;
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteExt(nlib_byte_t* p, nlib_byte_t* pend, int8_t type,
                         const nlib_byte_t* data, uint32_t n) NLIB_NOEXCEPT {
    if (n < 256U) {
        switch (n) {
        case 1:
            NLIB_CHK_(p, 3, pend);
            p[0] = static_cast<nlib_byte_t>(0xd4);
            p[1] = static_cast<nlib_byte_t>(type);
            p[2] = data[0];
            return p + 3;
        case 2:
            NLIB_CHK_(p, 4, pend);
            p[0] = static_cast<nlib_byte_t>(0xd5);
            p[1] = static_cast<nlib_byte_t>(type);
            p[2] = data[0];
            p[3] = data[1];
            return p + 4;
        case 4:
            NLIB_CHK_(p, 6, pend);
            p[0] = static_cast<nlib_byte_t>(0xd6);
            p[1] = static_cast<nlib_byte_t>(type);
            nlib_memcpy(p + 2, 4, data, 4);
            return p + 6;
        case 8:
            NLIB_CHK_(p, 10, pend);
            p[0] = static_cast<nlib_byte_t>(0xd7);
            p[1] = static_cast<nlib_byte_t>(type);
            nlib_memcpy(p + 2, 8, data, 8);
            return p + 10;
        case 16:
            NLIB_CHK_(p, 18, pend);
            p[0] = static_cast<nlib_byte_t>(0xd8);
            p[1] = static_cast<nlib_byte_t>(type);
            nlib_memcpy(p + 2, 16, data, 16);
            return p + 18;
        default:
            NLIB_CHK_(p, 3 + n, pend);
            p[0] = static_cast<nlib_byte_t>(0xc7);
            p[1] = static_cast<nlib_byte_t>(n);
            p[2] = static_cast<nlib_byte_t>(type);
            nlib_memcpy(p + 3, n, data, n);
            return p + 3 + n;
        }
    } else if (n < 65536U) {
        NLIB_CHK_(p, 4 + n, pend);
        p[0] = static_cast<nlib_byte_t>(0xc8);
        EncU16(p + 1, static_cast<uint16_t>(n));
        p[3] = static_cast<nlib_byte_t>(type);
        nlib_memcpy(p + 4, n, data, n);
        return p + 4 + n;
    } else {
        NLIB_CHK_(p, 6 + n, pend);
        p[0] = static_cast<nlib_byte_t>(0xc9);
        EncU32(p + 1, n);
        p[5] = static_cast<nlib_byte_t>(type);
        nlib_memcpy(p + 6, n, data, n);
        return p + 6 + n;
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteArrayHeader(nlib_byte_t* p, nlib_byte_t* pend,
                                 uint32_t count) NLIB_NOEXCEPT {
    if (count < 16U) {
        NLIB_CHK_(p, 1, pend);
        *p = static_cast<nlib_byte_t>(0x90 | count);
        return p + 1;
    } else if (count < 65536U) {
        NLIB_CHK_(p, 3, pend);
        p[0] = static_cast<nlib_byte_t>(0xdc);
        EncU16(p + 1, static_cast<uint16_t>(count));
        return p + 3;
    } else {
        NLIB_CHK_(p, 5, pend);
        p[0] = static_cast<nlib_byte_t>(0xdd);
        EncU32(p + 1, count);
        return p + 5;
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteMapHeader(nlib_byte_t* p, nlib_byte_t* pend,
                               uint32_t count) NLIB_NOEXCEPT {
    if (count < 16U) {
        NLIB_CHK_(p, 1, pend);
        *p = static_cast<nlib_byte_t>(0x80 | count);
        return p + 1;
    } else if (count < 65536U) {
        NLIB_CHK_(p, 3, pend);
        p[0] = static_cast<nlib_byte_t>(0xde);
        EncU16(p + 1, static_cast<uint16_t>(count));
        return p + 3;
    } else {
        NLIB_CHK_(p, 5, pend);
        p[0] = static_cast<nlib_byte_t>(0xdf);
        EncU32(p + 1, count);
        return p + 5;
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteUnsignedInt(nlib_byte_t* p, nlib_byte_t* pend,
                                 uint8_t val) NLIB_NOEXCEPT {
    if (val < 128U) {
        NLIB_CHK_(p, 1, pend);
        p[0] = static_cast<nlib_byte_t>(val);
        return p + 1;
    } else {
        return WriteUint8(p, pend, static_cast<uint8_t>(val));
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteUnsignedInt(nlib_byte_t* p, nlib_byte_t* pend,
                                 uint16_t val) NLIB_NOEXCEPT {
    if (val < 128U) {
        NLIB_CHK_(p, 1, pend);
        p[0] = static_cast<nlib_byte_t>(val);
        return p + 1;
    } else if (val < 256U) {
        return WriteUint8(p, pend, static_cast<uint8_t>(val));
    } else {
        return WriteUint16(p, pend, static_cast<uint16_t>(val));
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteUnsignedInt(nlib_byte_t* p, nlib_byte_t* pend,
                                 uint32_t val) NLIB_NOEXCEPT {
    if (val < 256U) {
        if (val < 128U) {
            NLIB_CHK_(p, 1, pend);
            p[0] = static_cast<nlib_byte_t>(val);
            return p + 1;
        } else {
            return WriteUint8(p, pend, static_cast<uint8_t>(val));
        }
    } else if (val < 65536U) {
        return WriteUint16(p, pend, static_cast<uint16_t>(val));
    } else {
        return WriteUint32(p, pend, val);
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteUnsignedInt(nlib_byte_t* p, nlib_byte_t* pend,
                                 uint64_t val) NLIB_NOEXCEPT {
    if (val < 0x100000000ULL) {
        return WriteUnsignedInt(p, pend, static_cast<uint32_t>(val));
    } else {
        return WriteUint64(p, pend, val);
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteInt(nlib_byte_t* p, nlib_byte_t* pend, int8_t val) NLIB_NOEXCEPT {
    if (val < -32) {
        return WriteInt8(p, pend, static_cast<int8_t>(val));
    } else {
        NLIB_CHK_(p, 1, pend);
        p[0] = static_cast<nlib_byte_t>(val);
        return p + 1;
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteInt(nlib_byte_t* p, nlib_byte_t* pend, int16_t val) NLIB_NOEXCEPT {
    if (val < 0) {
        if (val < INT8_MIN) {
            return WriteInt16(p, pend, static_cast<int16_t>(val));
        } else if (val < -32) {
            return WriteInt8(p, pend, static_cast<int8_t>(val));
        } else {
            NLIB_CHK_(p, 1, pend);
            p[0] = static_cast<nlib_byte_t>(val);
            return p + 1;
        }
    } else {
        return WriteUnsignedInt(p, pend, static_cast<uint16_t>(val));
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteInt(nlib_byte_t* p, nlib_byte_t* pend, int32_t val) NLIB_NOEXCEPT {
    if (val < 0) {
        if (val < INT8_MIN) {
            if (val < INT16_MIN) {
                return WriteInt32(p, pend, val);
            } else {
                return WriteInt16(p, pend, static_cast<int16_t>(val));
            }
        } else if (val < -32) {
            return WriteInt8(p, pend, static_cast<int8_t>(val));
        } else {
            NLIB_CHK_(p, 1, pend);
            p[0] = static_cast<nlib_byte_t>(val);
            return p + 1;
        }
    } else {
        return WriteUnsignedInt(p, pend, static_cast<uint32_t>(val));
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteInt(nlib_byte_t* p, nlib_byte_t* pend, int64_t val) NLIB_NOEXCEPT {
    if (val < 0) {
        if (val < INT32_MIN) {
            return WriteInt64(p, pend, val);
        } else {
            return WriteInt(p, pend, static_cast<int32_t>(val));
        }
    } else {
        return WriteUnsignedInt(p, pend, static_cast<uint64_t>(val));
    }
}
inline nlib_byte_t*
MpDirectWriter::WriteTimestamp(nlib_byte_t* p, nlib_byte_t* pend,
                               int64_t sec, uint32_t nsec) NLIB_NOEXCEPT {
    if (NLIB_LIKELY((sec >> 34) == 0)) {
        uint64_t data64 = (static_cast<uint64_t>(nsec) << 34) | static_cast<uint64_t>(sec);
        if (data64 < 0x100000000ULL) {
            // timestamp 32
            NLIB_CHK_(p, 6, pend);
            p[0] = static_cast<nlib_byte_t>(0xd6);
            p[1] = static_cast<nlib_byte_t>(-1);
            EncU32(p + 2, static_cast<uint32_t>(data64));
            return p + 6;
        } else {
            // timestamp 64
            NLIB_CHK_(p, 10, pend);
            p[0] = static_cast<nlib_byte_t>(0xd7);
            p[1] = static_cast<nlib_byte_t>(-1);
            EncU64(p + 2, data64);
            return p + 10;
        }
    }
    // timestamp96
    NLIB_CHK_(p, 15, pend);
    p[0] = static_cast<nlib_byte_t>(0xc7);
    p[1] = static_cast<nlib_byte_t>(12);
    p[2] = static_cast<nlib_byte_t>(-1);
    EncU32(p + 3, static_cast<uint32_t>(nsec));
    EncU64(p + 7, static_cast<uint64_t>(sec));
    return p + 15;
}
inline nlib_byte_t*
MpDirectWriter::WriteTimestamp(nlib_byte_t* p, nlib_byte_t* pend,
                               nlib_time val) NLIB_NOEXCEPT {
    int64_t sec;
    uint32_t nsec;
    NLIB_TO_SEC_NSEC(sec, nsec, val);
    return WriteTimestamp(p, pend, sec, nsec);
}

#undef NLIB_CHK_
}  // namespace msgpack
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_MSGPACK_MPDIRECTWRITER_H_
