﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCCLIENT_H_
#define INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCCLIENT_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/Swap.h"
#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/threading/Future.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {
namespace jsonrpc {
typedef uint32_t reqid_t;

class JsonRpcResponseHandler;
class JsonRpcResponse;

class NLIB_VIS_PUBLIC JsonRpcClient NLIB_FINAL {
 public:
    static JsonRpcClient OpenClient(const char* name);
    static JsonRpcClient SearchClient(const char* name);
    static void CloseClient(const JsonRpcClient& client);
    static void Shutdown();

 public:
    typedef void (*NoTargetResponseHandler)(UniquePtr<JsonRpcResponse>& rhs) NLIB_NOEXCEPT_FUNCPTR;  // NOLINT
    NLIB_CEXPR JsonRpcClient() NLIB_NOEXCEPT : handle_(0) {}
    JsonRpcClient(JsonRpcClient& rhs, move_tag) NLIB_NOEXCEPT : handle_(rhs.handle_) {}
    JsonRpcClient& assign(JsonRpcClient& rhs, move_tag) NLIB_NOEXCEPT {
        handle_ = rhs.handle_;
        return *this;
    }
    NLIB_DEPRECATED void swap(JsonRpcClient& rhs) {  // NOLINT
        using std::swap;
        swap(handle_, rhs.handle_);
    }

 public:
    reqid_t GenerateId();

    // future becomes valid after response or timeout
    typedef ::nlib_ns::threading::Future<UniquePtr<JsonRpcResponse> > FutureType;
    errno_t GetFutureForId(FutureType* future, reqid_t reqid);
    errno_t ResolveResponse(const void* buf, size_t nbytes);
    errno_t Abort(reqid_t reqid);
    errno_t SetNoTargetResponseHandler(NoTargetResponseHandler handler);

    bool operator==(const JsonRpcClient& rhs) const { return handle_ == rhs.handle_; }
    bool operator!=(const JsonRpcClient& rhs) const { return handle_ != rhs.handle_; }
    NLIB_SAFE_BOOL(JsonRpcClient, handle_ != 0);

 private:
    explicit NLIB_VIS_HIDDEN JsonRpcClient(uint32_t handle) : handle_(handle) {}

 private:
    uint32_t handle_;
    friend class JsonRpcClientImpl;
};

}  // namespace jsonrpc
}  // namespace msgpack
NLIB_NAMESPACE_END

NLIB_DEFINE_STD_SWAP(::nlib_ns::msgpack::jsonrpc::JsonRpcClient)

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCCLIENT_H_
