﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SUCCINCT_WORDFILTER_H_
#define INCLUDE_NN_NLIB_SUCCINCT_WORDFILTER_H_

#include "nn/nlib/Swap.h"
#include "nn/nlib/succinct/AhoCorasick.h"

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace succinct {

class NLIB_VIS_PUBLIC WordFilter NLIB_FINAL {
 public:
    typedef bool (*MatchCallback)(const char* first, const char* last, uint32_t nodeid,
                                  void* user_obj);
    NLIB_CEXPR WordFilter() NLIB_NOEXCEPT : prv_(nullptr) {}
    ~WordFilter() NLIB_NOEXCEPT { Reset(); }
    NLIB_DEFMOVE_PIMPL(WordFilter);
    NLIB_DEPRECATED void swap(WordFilter& rhs) NLIB_NOEXCEPT {
        using std::swap;
        swap(prv_, rhs.prv_);
    }
    bool Init() NLIB_NOEXCEPT;
    void Match(const char* doc, MatchCallback callback, void* user_obj) NLIB_NOEXCEPT {
        this->Match(doc, nlib_strlen(doc), callback, user_obj);
    }
    void Match(const char* doc, MatchCallback callback) NLIB_NOEXCEPT {
        this->Match(doc, callback, nullptr);
    }
    void Match(const char* data, size_t n, MatchCallback callback, void* user_obj) NLIB_NOEXCEPT;
    void Match(const char* data, size_t n, MatchCallback callback) NLIB_NOEXCEPT {
        this->Match(data, n, callback, nullptr);
    }
    void Reset() NLIB_NOEXCEPT;
    bool Export(BinaryWriter* w) const NLIB_NOEXCEPT;
    bool Import(BinaryReader* r) NLIB_NOEXCEPT;

 private:
    class WordFilterPrivate;
    WordFilterPrivate* prv_;
    friend class WordFilterBuilder;
    NLIB_DISALLOW_COPY_AND_ASSIGN(WordFilter);
};

class NLIB_VIS_PUBLIC WordFilterBuilder NLIB_FINAL {
 public:
    NLIB_CEXPR WordFilterBuilder() NLIB_NOEXCEPT : prv_(nullptr) {}
    ~WordFilterBuilder() NLIB_NOEXCEPT { Reset(); }
    NLIB_DEFMOVE_PIMPL(WordFilterBuilder);
    NLIB_DEPRECATED void swap(WordFilterBuilder& rhs) NLIB_NOEXCEPT {
        using std::swap;
        swap(prv_, rhs.prv_);
    }
    void Reset() NLIB_NOEXCEPT;
    bool Init() NLIB_NOEXCEPT;
    WordFilter* Build() NLIB_NOEXCEPT;
    bool AddWord(const char* str) NLIB_NOEXCEPT;
    bool AddWords(const char* str, size_t len) NLIB_NOEXCEPT;
    bool AddWords(const char* str) NLIB_NOEXCEPT {
        return AddWords(str, nlib_strlen(str));
    }
    bool AddExcludeWord(const char* str) NLIB_NOEXCEPT;
    bool AddExcludeWords(const char* str, size_t len) NLIB_NOEXCEPT;
    bool AddExcludeWords(const char* str) NLIB_NOEXCEPT {
        return AddExcludeWords(str, nlib_strlen(str));
    }

 private:
    struct WordFilterBuilderPrivate;
    WordFilterBuilderPrivate* prv_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(WordFilterBuilder);
};

}  // namespace succinct
NLIB_NAMESPACE_END
NLIB_DEFINE_STD_SWAP(::nlib_ns::succinct::WordFilter)
NLIB_DEFINE_STD_SWAP(::nlib_ns::succinct::WordFilterBuilder)
#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_SUCCINCT_WORDFILTER_H_
