﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_UNICODE_CONVERT_H_
#define INCLUDE_NN_NLIB_UNICODE_CONVERT_H_

#include "nn/nlib/Config.h"

// functions below may be deprecated in the future
NLIB_NAMESPACE_BEGIN
namespace unicode {

class NLIB_VIS_PUBLIC ToUtf8Obj NLIB_FINAL {
 public:
    explicit ToUtf8Obj(const wchar_t* wstr) NLIB_NOEXCEPT;
    explicit ToUtf8Obj(const nlib_utf16_t* utf16str) NLIB_NOEXCEPT;
    explicit ToUtf8Obj(const nlib_utf32_t* utf32str) NLIB_NOEXCEPT;
    explicit ToUtf8Obj(const nlib_utf8_t* str) NLIB_NOEXCEPT {
        // does not work if the encoding is not UTF-8.
        buf_ = str;
        is_internal_alloc_ = false;
    }
    ~ToUtf8Obj() NLIB_NOEXCEPT;
    operator const nlib_utf8_t*() NLIB_NOEXCEPT { return buf_ ? buf_ : ""; }
    const char* c_str() NLIB_NOEXCEPT { return buf_ ? buf_ : ""; }
    NLIB_SAFE_BOOL(ToUtf8Obj, buf_ != nullptr)

 private:
    const nlib_utf8_t* buf_;
    bool is_internal_alloc_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(ToUtf8Obj);
};

class NLIB_VIS_PUBLIC ToWideObj NLIB_FINAL {
 public:
    explicit ToWideObj(const nlib_utf8_t* str) NLIB_NOEXCEPT;
    explicit ToWideObj(const wchar_t* wstr) NLIB_NOEXCEPT {
        buf_ = wstr;
        is_internal_alloc_ = false;
    }
    ~ToWideObj() NLIB_NOEXCEPT;
    operator const wchar_t*() NLIB_NOEXCEPT { return buf_ ? buf_ : L""; }
    const wchar_t* c_str() NLIB_NOEXCEPT { return buf_ ? buf_ : L""; }
    NLIB_SAFE_BOOL(ToWideObj, buf_ != nullptr)

 private:
    const wchar_t* buf_;
    bool is_internal_alloc_;
    NLIB_DISALLOW_COPY_AND_ASSIGN(ToWideObj);
};

}  // namespace unicode
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_UNICODE_CONVERT_H_
