<#
.SYNOPSIS
    Downloads, builds, and installs zlib and libcurl

.DESCRIPTION
    This script downloads zlib and libcurl, and builds and installs them,
    using the newest Visual Studio on your computer.
    The PC version of nlib may require zlib and libcurl installed.

    PowerShell must be 64 bit process, and run the script as admin.

.PARAMETER expat
    Builds and installs libexpat if $true

.EXAMPLE
    .\setup-msvc-libraries.ps1
#>

Param([bool]$expat=$False)

if (![System.Environment]::Is64BitProcess) {
    Write-Error "use 64bit powershell"
}

Push-Location .
$scriptDir = Split-Path -Parent $MyInvocation.MyCommand.Path
$restore_path = $Env:Path
$restore_encode = [Console]::OutputEncoding
try {
Set-Location $scriptDir
[Console]::OutputEncoding = [Text.Encoding]::UTF8
Set-Item Env:Path ((($Env:Path -split ';') -match '^(?!.*cygwin)') -join ';')

if (Test-Path Env:VS140COMNTOOLS) {
    $gen64 = "Visual Studio 14 2015 Win64"
    $gen32 = "Visual Studio 14 2015"
} elseif (Test-Path Env:VS120COMNTOOLS) {
    $gen64 = "Visual Studio 12 2013 Win64"
    $gen32 = "Visual Studio 14 2015"
} else {
    $gen64 = "Visual Studio 15 2017 Win64"
    $gen32 = "Visual Studio 15 2017"
}

if (!(Test-Path ".\.cache")) {
    New-Item ".\.cache" -ItemType Directory
}
#
# zlib
#
Push-Location .
try {
    if (!(Test-Path .\.cache\zlib.zip)) {
        Invoke-WebRequest -Uri http://www.zlib.net/zlib128.zip -OutFile .\.cache\zlib.zip
    }
    Expand-Archive .\.cache\zlib.zip -Force
    Set-Location zlib\zlib-1.2.8
    cmake -G $gen32 -H"." -Bvc32
    cmake --build vc32 --config Release --target INSTALL
    cmake --build vc32 --config Debug --target INSTALL
    cmake -G $gen64 -H"." -Bvc64
    cmake --build vc64 --config Release --target INSTALL
    cmake --build vc64 --config Debug --target INSTALL
} catch {
    Write-Warning "Setup zlib -- FAILED"
} finally {
    Pop-Location
}

#
# libexpat
#
if (Test-Path ..\_common\software\expat-2.1.0.zip) {
Push-Location .
try {
    Expand-Archive ..\_common\software\expat-2.1.0.zip -Force
    Set-Location .\expat-2.1.0\expat-2.1.0
    cmake -G $gen32 -H"." -Bvc32
    cmake --build vc32 --config Release --target INSTALL
    cmake -G $gen64 -H"." -Bvc64
    cmake --build vc64 --config Release --target INSTALL
} catch {
    Write-Warning "Setup libexpat -- FAILED"
} finally {
    Pop-Location
}
}

#
# libcurl
#
Push-Location .
try {
    if (!(Test-Path .\.cache\curl.zip)) {
        Invoke-WebRequest -Uri https://curl.haxx.se/download/curl-7.48.0.zip -OutFile .\.cache\curl.zip
    }
    Expand-Archive .\.cache\curl.zip -Force
    Set-Location curl\curl-7.48.0
    cmake -G $gen32 -H"." -Bvc32 -DBUILD_CURL_TESTS=OFF -DBUILD_CURL_EXE=OFF -DZLIB_ROOT="${Env:ProgramFiles(x86)}/zlib"
    cmake --build vc32 --config Release --target INSTALL
    cmake -G $gen64 -H"." -Bvc64 -DBUILD_CURL_TESTS=OFF -DBUILD_CURL_EXE=OFF -DZLIB_ROOT="${Env:ProgramFiles}/zlib"
    cmake --build vc64 --config Release --target INSTALL
} catch {
    Write-Warning "Setup libcurl -- FAILED"
} finally {
    Pop-Location
}

} finally {
    Set-Item Env:Path $restore_path
    [Console]::OutputEncoding = $restore_encode
    Pop-Location
}



