﻿
/*--------------------------------------------------------------------------------*
  Project: CrossRoad
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SIMD_SIMDFLOAT_H_
#define INCLUDE_NN_NLIB_SIMD_SIMDFLOAT_H_

#ifdef NN_PLATFORM_CTR
# ifndef __USE_C99_MATH
#  define __USE_C99_MATH
# endif
#endif
#include <math.h>
#include <float.h>

#include "nn/nlib/Config.h"
#include "nn/nlib/simd/SimdInt.h"

#ifndef INFINITY
#define INFINITY ((float)(1e+300 * 1e+300))
#endif

#if !defined(NLIB_SIMD) && !defined(CAFE)
#define NLIB_F128_SIMD_NOUSE
#endif

#ifdef NLIB_F128_SIMD_NOUSE
typedef struct {
    union {
        float v[4];
        uint32_t u[4];
    } vec;
} nlib_f128_t;
typedef struct { nlib_f128_t val[2]; } nlib_f128x2_t;
#elif defined(NLIB_SSE41)
typedef __m128 nlib_f128_t;
typedef struct { nlib_f128_t val[2]; } nlib_f128x2_t;
#elif defined(NLIB_NEON)
typedef float32x4_t nlib_f128_t;
typedef float32x4x2_t nlib_f128x2_t;
#elif defined(CAFE)
typedef struct {
    union {
        f32x2 ps[2];
        float v[4];
        uint32_t u[4];
    } vec;
} nlib_f128_t;
typedef struct { nlib_f128_t val[2]; } nlib_f128x2_t;
#endif

NLIB_NAMESPACE_BEGIN
namespace simd {

// use each_float for the argument value
struct each_float_tag {};
NLIB_CEXPR const each_float_tag each_float = {};

// __m128(SSE), float32x4_t(NEON)
typedef nlib_f128_t f128;
// float32x4x2_t(NEON)
typedef nlib_f128x2_t f128x2;

#if !defined(NLIB_SIMD) || defined(NLIB_F128_SIMD_NOUSE)
typedef const f128& f128arg;
#else
typedef const f128 f128arg;
#endif

#if defined(_MSC_VER) || !defined(NLIB_SIMD) || defined(NLIB_F128_SIMD_NOUSE)
typedef const f128& f128arg_ex;
#else
typedef const f128 f128arg_ex;
#endif

#if !defined(_MSC_VER) && !defined(__vectorcall)
#define __vectorcall
#endif

struct NLIB_VIS_HIDDEN F128 {
 public:
    static f128 __vectorcall SetValue(float v, each_float_tag) NLIB_NOEXCEPT;
    static f128 __vectorcall SetValue(uint32_t v, each_uint32_tag) NLIB_NOEXCEPT;
    static f128 __vectorcall SetValue(float a, float b, float c, float d) NLIB_NOEXCEPT;
    template <size_t N>
    static f128 __vectorcall SetValue(f128arg value, each_select32_tag) NLIB_NOEXCEPT;
    static f128 __vectorcall SetZero() NLIB_NOEXCEPT;
    static f128 __vectorcall Set1000() NLIB_NOEXCEPT;
    static f128 __vectorcall Set0100() NLIB_NOEXCEPT;
    static f128 __vectorcall Set0010() NLIB_NOEXCEPT;
    static f128 __vectorcall Set0001() NLIB_NOEXCEPT;
    template <size_t N>
    static f128 __vectorcall SetZeroToLane(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall SetOne() NLIB_NOEXCEPT;
    static f128 __vectorcall SetNegativeOne() NLIB_NOEXCEPT;
    static f128 __vectorcall SetEpsilon() NLIB_NOEXCEPT;
    static f128 __vectorcall SetInfinity() NLIB_NOEXCEPT;
    static f128 __vectorcall SetNaN() NLIB_NOEXCEPT;
    static f128 __vectorcall SetSignMask() NLIB_NOEXCEPT;

    static f128 __vectorcall LoadA16(const float* p) NLIB_NOEXCEPT;
    static f128 __vectorcall LoadA8(const float* p) NLIB_NOEXCEPT;
    static f128 __vectorcall LoadA4(const float* p) NLIB_NOEXCEPT;
    static f128 __vectorcall LoadA16(uintptr_t p) NLIB_NOEXCEPT;
    static f128 __vectorcall LoadA8(uintptr_t p) NLIB_NOEXCEPT;
    static f128 __vectorcall LoadA4(uintptr_t p) NLIB_NOEXCEPT;
    static f128 __vectorcall LoadA16(intptr_t p) NLIB_NOEXCEPT;
    static f128 __vectorcall LoadA8(intptr_t p) NLIB_NOEXCEPT;
    static f128 __vectorcall LoadA4(intptr_t p) NLIB_NOEXCEPT;

    static void __vectorcall StoreA16(float* p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA8(float* p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA4(float* p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA16(uintptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA8(uintptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA4(uintptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA16(intptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA8(intptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA4(intptr_t p, f128arg value) NLIB_NOEXCEPT;

    static void __vectorcall StoreLoA8(float* p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreLoA4(float* p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreLoA8(uintptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreLoA4(uintptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreLoA8(intptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreLoA4(intptr_t p, f128arg value) NLIB_NOEXCEPT;

    static void __vectorcall StoreHiA8(float* p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreHiA4(float* p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreHiA8(uintptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreHiA4(uintptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreHiA8(intptr_t p, f128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreHiA4(intptr_t p, f128arg value) NLIB_NOEXCEPT;

    /*
    static f128 __vectorcall ConvertFromFp16Lo(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall ConvertFromFp16High(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall ConvertToFp16(f128arg a, f128arg b) NLIB_NOEXCEPT;
    */

#if !defined(NLIB_F128_SIMD_NOUSE) && !defined(CAFE)
    static f128 __vectorcall ConvertFromI128(i128 value) NLIB_NOEXCEPT;
    static i128 __vectorcall ConvertToI128Round(f128 value) NLIB_NOEXCEPT;
    static i128 __vectorcall ConvertToI128Truncate(f128 value) NLIB_NOEXCEPT;

    static f128 __vectorcall CastFromI128(i128 value) NLIB_NOEXCEPT;
    static i128 __vectorcall CastToI128(f128 value) NLIB_NOEXCEPT;

    template<int N>
    static f128 __vectorcall ConvertFromFixedPoint(i128arg value) NLIB_NOEXCEPT;
    template<int N>
    static i128 __vectorcall ConvertToFixedPoint(f128arg value) NLIB_NOEXCEPT;
#endif

    static f128 __vectorcall Add(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Sub(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Mult(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Mult(float a, f128arg b) NLIB_NOEXCEPT;
    template <size_t N>
    static  f128 __vectorcall Mult(f128arg a, f128arg b, each_select32_tag) NLIB_NOEXCEPT;
    static f128 __vectorcall Div(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Negate(f128arg value) NLIB_NOEXCEPT;
    template <bool NegateLane0, bool NegateLane1, bool NegateLane2, bool NegateLane3>
    static f128 __vectorcall NegateEx(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall MultAdd(f128arg a, f128arg b, f128arg c) NLIB_NOEXCEPT;
    static f128 __vectorcall MultAdd(float a, f128arg b, f128arg c) NLIB_NOEXCEPT;
    template <size_t N>
    static f128 __vectorcall MultAdd(f128arg a, f128arg b, f128arg c,
                                     each_select32_tag) NLIB_NOEXCEPT;
    static f128 __vectorcall MultSub(f128arg a, f128arg b, f128arg c) NLIB_NOEXCEPT;
    static f128 __vectorcall MultSub(float a, f128arg b, f128arg c) NLIB_NOEXCEPT;
    template <size_t N>
    static f128 __vectorcall MultSub(f128arg a, f128arg b, f128arg c,
                                     each_select32_tag) NLIB_NOEXCEPT;
    static f128 __vectorcall PairwiseAdd(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Abs(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall AbsDiff(f128arg a, f128arg b) NLIB_NOEXCEPT;

    //
    // Min/Max
    //

    static f128 __vectorcall Max(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Min(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall PairwiseMax(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall PairwiseMin(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Clamp(f128arg value, f128arg min, f128arg max) NLIB_NOEXCEPT;
    static f128 __vectorcall Saturate(f128arg value) NLIB_NOEXCEPT;

    //
    // Reciprocal/Sqrt
    //

    static f128 __vectorcall Recp(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall RecpEst(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall Sqrt(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall SqrtEst(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall RecpSqrt(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall RecpSqrtEst(f128arg value) NLIB_NOEXCEPT;

    //
    // Round/Truncate
    //

    static f128 __vectorcall Round(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall Truncate(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall Floor(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall Ceil(f128arg value) NLIB_NOEXCEPT;

    //
    // Logical Operations
    //

    static f128 __vectorcall And(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Or(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Xor(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall Not(f128arg a) NLIB_NOEXCEPT;
    static f128 __vectorcall AndNot(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall OrNot(f128arg a, f128arg b) NLIB_NOEXCEPT;

    //
    // Comparison Operations
    //

    static f128 __vectorcall CmpEq(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpLt(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpLe(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpGt(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpGe(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpNe(f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpNearEq(f128arg a, f128arg b, f128arg eps) NLIB_NOEXCEPT;
    static f128 __vectorcall InBound(f128arg value, f128arg bounds) NLIB_NOEXCEPT;

    static f128 __vectorcall CmpEqZero(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpLtZero(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpLeZero(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpGtZero(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpGeZero(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpNeZero(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall CmpNearEqZero(f128arg value, f128arg eps) NLIB_NOEXCEPT;

    //
    // Trigonometric function
    //
    static f128 __vectorcall AddAngle(f128arg angle1, f128arg angle2) NLIB_NOEXCEPT;
    static f128 __vectorcall SubAngle(f128arg angle1, f128arg angle2) NLIB_NOEXCEPT;
    static f128 __vectorcall ModAngle(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall Sin(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall Cos(f128arg value) NLIB_NOEXCEPT;
    static f128x2 __vectorcall SinCos(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall Tan(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall SinH(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall CosH(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall TanH(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall ArcSin(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall ArcCos(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall ArcTan(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall ArcTan2(f128arg y, f128arg x) NLIB_NOEXCEPT;
    // and Est version needed?

    //
    // Interpolation
    //

    static f128 __vectorcall Lerp(f128arg a, f128arg b, f128arg t) NLIB_NOEXCEPT;
    static f128 __vectorcall
        Hermite(f128arg p0, f128arg v0, f128arg p1, f128arg_ex v1, f128arg_ex t) NLIB_NOEXCEPT;
    static f128 __vectorcall
        CatmullRom(f128arg p0, f128arg p1, f128arg p2, f128arg_ex p3, f128arg_ex t) NLIB_NOEXCEPT;
    static f128 __vectorcall
        BaryCentric(f128arg p0, f128arg p1, f128arg p2, f128arg_ex f, f128arg_ex g) NLIB_NOEXCEPT;

    //
    // Exp/Log
    //
    static f128 __vectorcall Exp2(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall ExpE(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall Log2(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall LogE(f128arg value) NLIB_NOEXCEPT;  // not implemented

    //
    // Misc
    //

    static int __vectorcall MoveMask(f128arg value) NLIB_NOEXCEPT;
    static bool __vectorcall IsAllMaskFalse(f128arg value) NLIB_NOEXCEPT;
    static bool __vectorcall IsAllMaskTrue(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall Select(f128arg mask, f128arg a, f128arg b) NLIB_NOEXCEPT;
    static f128 __vectorcall IsNaN(f128arg value) NLIB_NOEXCEPT;
    static f128 __vectorcall IsInfinite(f128arg value) NLIB_NOEXCEPT;

    //
    // Get/Set
    //

    template <size_t N>
    static float __vectorcall GetFloatFromLane(f128arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static uint32_t __vectorcall GetUint32FromLane(f128arg value) NLIB_NOEXCEPT;
    static float __vectorcall GetFloatByIndex(f128arg value, size_t idx) NLIB_NOEXCEPT;
    static uint32_t __vectorcall GetUint32ByIndex(f128arg value, size_t idx) NLIB_NOEXCEPT;

    template <size_t N>
    static f128 __vectorcall SetFloatToLane(f128arg value, float v) NLIB_NOEXCEPT;
    static f128 __vectorcall SetFloatByIndex(f128arg value, float v, size_t i) NLIB_NOEXCEPT;

    //
    // Swizzle/Permute
    //

    template <int V0, int V1, int V2, int V3>
    static f128 __vectorcall Swizzle(f128arg value) NLIB_NOEXCEPT;
    template <int V0, int V1, int V2, int V3>
    static f128 __vectorcall Permute(f128arg a, f128arg b) NLIB_NOEXCEPT;
    template <bool SplatLane0, bool SplatLane1, bool SplatLane2, bool SplatLane3>
    static f128 __vectorcall Splat(f128arg value, f128arg splat) NLIB_NOEXCEPT;

    template <size_t N>
    // left <- value[3], value[2], value[1], value[0] -> right
    static f128 __vectorcall RotateLeft(f128arg value) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(N < 4);
        const size_t NN = 4 - N;
        return Swizzle<(NN & 3), ((NN + 1) & 3), ((NN + 2) & 3), ((NN + 3) & 3)>(value);
    }
    template <size_t N>
    // left <- value[3], value[2], value[1], value[0] -> right
    static f128 __vectorcall RotateRight(f128arg value) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(N < 4);
        return Swizzle<(N & 3), ((N + 1) & 3), ((N + 2) & 3), ((N + 3) & 3)>(value);
    }
    template <size_t N>
    // left <- b[3], ..., b[0], a[3], ..., a[0] -> right
    static f128 __vectorcall ShiftRight(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(N < 4);
        return Permute<N, (N + 1), (N + 2), (N + 3)>(a, b);
    }

 private:
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float v1000_[4];  // 1.f, 0.f, 0.f, 0.f
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float v0100_[4];  // 0.f, 1.f, 0.f, 0.f
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float v0010_[4];  // 0.f, 0.f, 1.f, 0.f
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float v0001_[4];  // 0.f, 0.f, 0.f, 1.f

    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float pi_values_[4];   // pi, -pi, 2pi, pi/2
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const uint32_t nan_inf_[4];  // nan, inf, -nan, -inf

    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float hermite_R0_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float hermite_R1_[4];

    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float catmull_R0_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float catmull_R1_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float catmull_R2_[4];

    // Those coefficients are from Cephes Math Library
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float sin_cvalue_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float sin_coeff_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float cos_cvalue_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float cos_coeff_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float atan_coeff_[8];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float atan2_cvalue_[4];

    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float exp2_P_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float exp2_Q_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float tanh_cvalue_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float tan_p_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float tan_q_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float tan_c_[4];
    NLIB_ALIGNAS(16) static NLIB_VIS_PUBLIC const float log2_PQ_[12];

    F128();  // forbidden
    friend class Vector3;
    friend class Vector4;
    friend class Matrix;
    friend class Plane;
    friend class Quaternion;
    friend class Sphere;
    friend class AxisAlignedBox;
    friend class OrientedBox;
    friend class Frustum;

    friend class DistanceSq;
    friend class Intersection;
    friend class Containment;
};

#ifndef NLIB_DOXYGEN

#undef NLIB_M
#define NLIB_M(tp) NLIB_ALWAYS_INLINE tp __vectorcall
#define NLIB_M2(tp) inline tp __vectorcall

// r[i] = v
NLIB_M(f128) F128::SetValue(float v, each_float_tag) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = v;
    ret.vec.v[1] = v;
    ret.vec.v[2] = v;
    ret.vec.v[3] = v;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_set1_ps(v);
#elif defined(NLIB_NEON)
    return vdupq_n_f32(v);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_FDUP(v);
    return ret;
#endif
}

// r[i] = *reinterpret_cast<float*>(&v)
NLIB_M(f128) F128::SetValue(uint32_t v, each_uint32_tag) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.u[0] = v;
    ret.vec.u[1] = v;
    ret.vec.u[2] = v;
    ret.vec.u[3] = v;
    return ret;
#elif defined(NLIB_SSE41)
    union {
        float f32;
        uint32_t u32;
    } tmp;
    tmp.u32 = v;
    return _mm_set1_ps(tmp.f32);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vdupq_n_u32(v);
    return vreinterpretq_f32_u32(tmp);
#elif defined(CAFE)
    union {
        float f32;
        uint32_t u32;
    } tmp;
    tmp.u32 = v;
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_FDUP(tmp.f32);
    return ret;
#endif
}

// r[0] = a, r[1] = b, r[2] = c, r[3] = d
NLIB_M(f128) F128::SetValue(float a, float b, float c, float d) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a;
    ret.vec.v[1] = b;
    ret.vec.v[2] = c;
    ret.vec.v[3] = d;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_set_ps(d, c, b, a);
#elif defined(NLIB_NEON)
    union {
        float f32[2];
        uint64_t u64;
    } tmp1, tmp2;
    tmp1.f32[0] = a;
    tmp1.f32[1] = b;
    tmp2.f32[0] = c;
    tmp2.f32[1] = d;
    return vcombine_f32(vcreate_f32(tmp1.u64), vcreate_f32(tmp2.u64));
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0][0] = a;
    ret.vec.ps[0][1] = b;
    ret.vec.ps[1][0] = c;
    ret.vec.ps[1][1] = d;
    return ret;
#endif
}

template <size_t N>
// r[i] = value[N]
NLIB_M(f128) F128::SetValue(f128arg value, each_select32_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = value.vec.v[N];
    ret.vec.v[1] = value.vec.v[N];
    ret.vec.v[2] = value.vec.v[N];
    ret.vec.v[3] = value.vec.v[N];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_shuffle_ps(value, value, _MM_SHUFFLE(N, N, N, N));
#elif defined(NLIB_NEON)
    float32x2_t tmp = vget_low_f32(value);
    return vdupq_lane_f32(tmp, N);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_FDUP(value.vec.ps[N / 2][N % 2]);
    return ret;
#endif
}

#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
template <>
NLIB_M(f128) F128::SetValue<2>(f128arg value, each_select32_tag) NLIB_NOEXCEPT {
    float32x2_t tmp = vget_high_f32(value);
    return vdupq_lane_f32(tmp, 0);
}
template <>
NLIB_M(f128) F128::SetValue<3>(f128arg value, each_select32_tag) NLIB_NOEXCEPT {
    float32x2_t tmp = vget_high_f32(value);
    return vdupq_lane_f32(tmp, 1);
}
#elif defined(CAFE) && !defined(NLIB_F128_SIMD_NOUSE)
template <>
NLIB_M(f128) F128::SetValue<0>(f128arg value, each_select32_tag) NLIB_NOEXCEPT {
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_MERGE00(value.vec.ps[0], value.vec.ps[0]);
    return ret;
}
template <>
NLIB_M(f128) F128::SetValue<1>(f128arg value, each_select32_tag) NLIB_NOEXCEPT {
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_MERGE11(value.vec.ps[0], value.vec.ps[0]);
    return ret;
}
template <>
NLIB_M(f128) F128::SetValue<2>(f128arg value, each_select32_tag) NLIB_NOEXCEPT {
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_MERGE00(value.vec.ps[1], value.vec.ps[1]);
    return ret;
}
template <>
NLIB_M(f128) F128::SetValue<3>(f128arg value, each_select32_tag) NLIB_NOEXCEPT {
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_MERGE11(value.vec.ps[1], value.vec.ps[1]);
    return ret;
}
#endif

// r[i] = 0.f
NLIB_M(f128) F128::SetZero() NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = 0;
    ret.vec.v[1] = 0;
    ret.vec.v[2] = 0;
    ret.vec.v[3] = 0;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_setzero_ps();
#elif defined(NLIB_NEON)
    return vdupq_n_f32(0);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_FDUP(0.f);
    return ret;
#endif
}

NLIB_M(f128) F128::Set1000() NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = 1.f;
    ret.vec.v[1] = 0;
    ret.vec.v[2] = 0;
    ret.vec.v[3] = 0;
    return ret;
#elif defined(NLIB_NEON)
    float32x2_t x10 = vcreate_f32(0x000000003F800000ULL);
    float32x2_t x00 = vcreate_f32(0ULL);
    return vcombine_f32(x10, x00);
#else
    return F128::LoadA16(F128::v1000_);
#endif
}

NLIB_M(f128) F128::Set0100() NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = 0;
    ret.vec.v[1] = 1.f;
    ret.vec.v[2] = 0;
    ret.vec.v[3] = 0;
    return ret;
#elif defined(NLIB_NEON)
    float32x2_t x01 = vcreate_f32(0x3F80000000000000ULL);
    float32x2_t x00 = vcreate_f32(0ULL);
    return vcombine_f32(x01, x00);
#else
    return F128::LoadA16(F128::v0100_);
#endif
}

NLIB_M(f128) F128::Set0010() NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = 0;
    ret.vec.v[1] = 0;
    ret.vec.v[2] = 1.f;
    ret.vec.v[3] = 0;
    return ret;
#elif defined(NLIB_NEON)
    float32x2_t x10 = vcreate_f32(0x000000003F800000ULL);
    float32x2_t x00 = vcreate_f32(0ULL);
    return vcombine_f32(x00, x10);
#else
    return F128::LoadA16(F128::v0010_);
#endif
}

NLIB_M(f128) F128::Set0001() NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = 0;
    ret.vec.v[1] = 0;
    ret.vec.v[2] = 0;
    ret.vec.v[3] = 1.f;
    return ret;
#elif defined(NLIB_NEON)
    float32x2_t x01 = vcreate_f32(0x3F80000000000000ULL);
    float32x2_t x00 = vcreate_f32(0ULL);
    return vcombine_f32(x00, x01);
#else
    return F128::LoadA16(F128::v0001_);
#endif
}

template <size_t N>
// r = value, r[N] = 0.f
NLIB_M(f128) F128::SetZeroToLane(f128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret = value;
    ret.vec.v[N] = 0.f;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_insert_ps(value, value, 1 << N);
#elif defined(NLIB_NEON)
    return F128::Permute<N == 0 ? 4 : 0,
                         N == 1 ? 5 : 1,
                         N == 2 ? 6 : 2,
                         N == 3 ? 7 : 3>(value, vdupq_n_f32(0.f));
    // return vsetq_lane_f32(0.f, value, N);
#elif defined(CAFE)
    f128 ret = value;
    ret.vec.ps[N / 2][N % 2] = 0.f;
    return ret;
#endif
}

// r[i] = 1.f
NLIB_M(f128) F128::SetOne() NLIB_NOEXCEPT {
    return F128::SetValue(1.f, each_float);
}

// r[i] = -1.f
NLIB_M(f128) F128::SetNegativeOne() NLIB_NOEXCEPT {
    return F128::SetValue(-1.f, each_float);
}

// r[i] = 1.0e-7f
NLIB_M(f128) F128::SetEpsilon() NLIB_NOEXCEPT {
    return F128::SetValue(1.0e-7f, each_float);
}

// r[i] = 0x7F800000U
NLIB_M(f128) F128::SetInfinity() NLIB_NOEXCEPT {
    return F128::SetValue(0x7F800000U, each_uint32);
}

// r[i] = 0x7FC00000U
NLIB_M(f128) F128::SetNaN() NLIB_NOEXCEPT {
    return F128::SetValue(0x7FC00000U, each_uint32);
}

// r[i] = -0.f(0x80000000U)
NLIB_M(f128) F128::SetSignMask() NLIB_NOEXCEPT {
    return F128::SetValue(-0.f, each_float);
}

// r[i] = p[i], p is 16 bytes aligned
NLIB_M(f128) F128::LoadA16(const float* p) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = p[0];
    ret.vec.v[1] = p[1];
    ret.vec.v[2] = p[2];
    ret.vec.v[3] = p[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_load_ps(p);
#elif defined(NLIB_NEON)
    const uint64_t* tmp = reinterpret_cast<const uint64_t*>(p);
    uint64x2_t val = vld1q_u64(tmp);
    return vreinterpretq_f32_u64(val);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0][0] = p[0];
    ret.vec.ps[0][1] = p[1];
    ret.vec.ps[1][0] = p[2];
    ret.vec.ps[1][1] = p[3];
    return ret;
#endif
}

// r[i] = p[i], p is 4 bytes aligned
NLIB_M(f128) F128::LoadA4(const float* p) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    return LoadA16(p);
#elif defined(NLIB_SSE41)
    return _mm_loadu_ps(p);
#elif defined(NLIB_NEON)
    return vld1q_f32(p);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0][0] = p[0];
    ret.vec.ps[0][1] = p[1];
    ret.vec.ps[1][0] = p[2];
    ret.vec.ps[1][1] = p[3];
    return ret;
#endif
}

// r[i] = p[i], p is 8 bytes aligned
NLIB_M(f128) F128::LoadA8(const float* p) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
    const uint64_t* tmp = reinterpret_cast<const uint64_t*>(p);
    uint64x2_t val = vld1q_u64(tmp);
    return vreinterpretq_f32_u64(val);
#else
    return LoadA4(p);
#endif
}

// r[i] = p[i], p is 16 bytes aligned
NLIB_M(f128) F128::LoadA16(uintptr_t p) NLIB_NOEXCEPT {
    return LoadA16(reinterpret_cast<const float*>(p));
}

// r[i] = p[i], p is 8 bytes aligned
NLIB_M(f128) F128::LoadA8(uintptr_t p) NLIB_NOEXCEPT {
    return LoadA8(reinterpret_cast<const float*>(p));
}

// r[i] = p[i], p is 4 bytes aligned
NLIB_M(f128) F128::LoadA4(uintptr_t p) NLIB_NOEXCEPT {
    return LoadA4(reinterpret_cast<const float*>(p));
}

// r[i] = p[i], p is 16 bytes aligned
NLIB_M(f128) F128::LoadA16(intptr_t p) NLIB_NOEXCEPT {
    return LoadA16(reinterpret_cast<const float*>(p));
}

// r[i] = p[i], p is 8 bytes aligned
NLIB_M(f128) F128::LoadA8(intptr_t p) NLIB_NOEXCEPT {
    return LoadA8(reinterpret_cast<const float*>(p));
}

// r[i] = p[i], p is 4 bytes aligned
NLIB_M(f128) F128::LoadA4(intptr_t p) NLIB_NOEXCEPT {
    return LoadA4(reinterpret_cast<const float*>(p));
}

// p[i] = value[i], p is 16 bytes aligned
NLIB_M(void) F128::StoreA16(float* p, f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    p[0] = value.vec.v[0];
    p[1] = value.vec.v[1];
    p[2] = value.vec.v[2];
    p[3] = value.vec.v[3];
#elif defined(NLIB_SSE41)
    _mm_store_ps(p, value);
#elif defined(NLIB_NEON)
    uint64x2_t tmp = vreinterpretq_u64_f32(value);
    vst1q_u64(reinterpret_cast<uint64_t*>(p), tmp);
#elif defined(CAFE)
    p[0] = value.vec.ps[0][0];
    p[1] = value.vec.ps[0][1];
    p[2] = value.vec.ps[1][0];
    p[3] = value.vec.ps[1][1];
#endif
}

// p[i] = value[i], p is 4 bytes aligned
NLIB_M(void) F128::StoreA4(float* p, f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    StoreA16(p, value);
#elif defined(NLIB_SSE41)
    _mm_storeu_ps(p, value);
#elif defined(NLIB_NEON)
    vst1q_f32(p, value);
#elif defined(CAFE)
    p[0] = value.vec.ps[0][0];
    p[1] = value.vec.ps[0][1];
    p[2] = value.vec.ps[1][0];
    p[3] = value.vec.ps[1][1];
#endif
}

// p[i] = value[i], p is 8 bytes aligned
NLIB_M(void) F128::StoreA8(float* p, f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
    uint64x2_t tmp = vreinterpretq_u64_f32(value);
    vst1q_u64(reinterpret_cast<uint64_t*>(p), tmp);
#else
    StoreA4(p, value);
#endif
}

// p[i] = value[i], p is 16 bytes aligned
NLIB_M(void) F128::StoreA16(uintptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreA16(reinterpret_cast<float*>(p), value);
}

// p[i] = value[i], p is 8 bytes aligned
NLIB_M(void) F128::StoreA8(uintptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreA8(reinterpret_cast<float*>(p), value);
}

// p[i] = value[i], p is 4 bytes aligned
NLIB_M(void) F128::StoreA4(uintptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreA4(reinterpret_cast<float*>(p), value);
}

// p[i] = value[i], p is 16 bytes aligned
NLIB_M(void) F128::StoreA16(intptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreA16(reinterpret_cast<float*>(p), value);
}

// p[i] = value[i], p is 8 bytes aligned
NLIB_M(void) F128::StoreA8(intptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreA8(reinterpret_cast<float*>(p), value);
}

// p[i] = value[i], p is 4 bytes aligned
NLIB_M(void) F128::StoreA4(intptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreA4(reinterpret_cast<float*>(p), value);
}

// p[0] = value[0], p[1] = value[1], p is 8 bytes aligned
NLIB_M(void) F128::StoreLoA8(float* p, f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    p[0] = value.vec.v[0];
    p[1] = value.vec.v[1];
#elif defined(NLIB_SSE41)
    _mm_storel_pi(reinterpret_cast<__m64*>(p), value);
#elif defined(NLIB_NEON)
    uint64x1_t tmp = vget_low_u64(vreinterpretq_u64_f32(value));
    vst1_u64(reinterpret_cast<uint64_t*>(p), tmp);
#elif defined(CAFE)
    p[0] = value.vec.ps[0][0];
    p[1] = value.vec.ps[0][1];
#endif
}

// p[0] = value[0], p[1] = value[1], p is 4 bytes aligned
NLIB_M(void) F128::StoreLoA4(float* p, f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    p[0] = value.vec.v[0];
    p[1] = value.vec.v[1];
#elif defined(NLIB_SSE41)
    _mm_storel_pi(reinterpret_cast<__m64*>(p), value);
#elif defined(NLIB_NEON)
    float32x2_t tmp = vget_low_f32(value);
    vst1_f32(p, tmp);
#elif defined(CAFE)
    p[0] = value.vec.ps[0][0];
    p[1] = value.vec.ps[0][1];
#endif
}

// p[0] = value[0], p[1] = value[1], p is 8 bytes aligned
NLIB_M(void) F128::StoreLoA8(uintptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreLoA8(reinterpret_cast<float*>(p), value);
}

// p[0] = value[0], p[1] = value[1], p is 4 bytes aligned
NLIB_M(void) F128::StoreLoA4(uintptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreLoA4(reinterpret_cast<float*>(p), value);
}

// p[0] = value[0], p[1] = value[1], p is 8 bytes aligned
NLIB_M(void) F128::StoreLoA8(intptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreLoA8(reinterpret_cast<float*>(p), value);
}

// p[0] = value[0], p[1] = value[1], p is 4 bytes aligned
NLIB_M(void) F128::StoreLoA4(intptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreLoA4(reinterpret_cast<float*>(p), value);
}

// p[0] = value[2], p[1] = value[3], p is 8 bytes aligned
NLIB_M(void) F128::StoreHiA8(float* p, f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    p[0] = value.vec.v[2];
    p[1] = value.vec.v[3];
#elif defined(NLIB_SSE41)
    _mm_storeh_pi(reinterpret_cast<__m64*>(p), value);
#elif defined(NLIB_NEON)
    vst1_f32(p, vget_high_f32(value));
#elif defined(CAFE)
    p[0] = value.vec.ps[1][0];
    p[1] = value.vec.ps[1][1];
#endif
}

// p[0] = value[2], p[1] = value[3], p is 4 bytes aligned
NLIB_M(void) F128::StoreHiA4(float* p, f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    p[0] = value.vec.v[2];
    p[1] = value.vec.v[3];
#elif defined(NLIB_SSE41)
    _mm_storeh_pi(reinterpret_cast<__m64*>(p), value);
#elif defined(NLIB_NEON)
    float32x2_t tmp = vget_high_f32(value);
    vst1_f32(p, tmp);
#elif defined(CAFE)
    p[0] = value.vec.ps[1][0];
    p[1] = value.vec.ps[1][1];
#endif
}

// p[0] = value[2], p[1] = value[3], p is 8 bytes aligned
NLIB_M(void) F128::StoreHiA8(uintptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreHiA8(reinterpret_cast<float*>(p), value);
}

// p[0] = value[2], p[1] = value[3], p is 4 bytes aligned
NLIB_M(void) F128::StoreHiA4(uintptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreHiA4(reinterpret_cast<float*>(p), value);
}

// p[0] = value[2], p[1] = value[3], p is 8 bytes aligned
NLIB_M(void) F128::StoreHiA8(intptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreHiA8(reinterpret_cast<float*>(p), value);
}

// p[0] = value[2], p[1] = value[3], p is 4 bytes aligned
NLIB_M(void) F128::StoreHiA4(intptr_t p, f128arg value) NLIB_NOEXCEPT {
    StoreHiA4(reinterpret_cast<float*>(p), value);
}

// r[i] = fabs(value[i])
NLIB_M(f128) F128::Abs(f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = value.vec.v[0] > 0 ? value.vec.v[0] : -value.vec.v[0];
    ret.vec.v[1] = value.vec.v[1] > 0 ? value.vec.v[1] : -value.vec.v[1];
    ret.vec.v[2] = value.vec.v[2] > 0 ? value.vec.v[2] : -value.vec.v[2];
    ret.vec.v[3] = value.vec.v[3] > 0 ? value.vec.v[3] : -value.vec.v[3];
    return ret;
#elif defined(NLIB_NEON)
    return vabsq_f32(value);
#elif defined(NLIB_SSE41)
    const __m128 signmask = _mm_set1_ps(-0.0f);  // 0x80000000
    return _mm_andnot_ps(signmask, value);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = __PS_ABS(value.vec.ps[0]);
    ret.vec.ps[1] = __PS_ABS(value.vec.ps[1]);
    return ret;
#endif
}

// r[i] = mask[i] ? a[i] : b[i]
NLIB_M(f128) F128::Select(f128arg mask, f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 result;
    result.vec.u[0] = (a.vec.u[0] & mask.vec.u[0]) | (b.vec.u[0] & ~mask.vec.u[0]);
    result.vec.u[1] = (a.vec.u[1] & mask.vec.u[1]) | (b.vec.u[1] & ~mask.vec.u[1]);
    result.vec.u[2] = (a.vec.u[2] & mask.vec.u[2]) | (b.vec.u[2] & ~mask.vec.u[2]);
    result.vec.u[3] = (a.vec.u[3] & mask.vec.u[3]) | (b.vec.u[3] & ~mask.vec.u[3]);
    return result;
#elif defined(NLIB_SSE41)
    return _mm_blendv_ps(b, a, mask);
#elif defined(NLIB_NEON)
    return vbslq_f32(vreinterpretq_u32_f32(mask), a, b);
#elif defined(CAFE)
    // avoid NaN
    f128 mask_ = mask;
    mask_.vec.u[0] &= 0xFF7FFFFFUL;
    mask_.vec.u[1] &= 0xFF7FFFFFUL;
    mask_.vec.u[2] &= 0xFF7FFFFFUL;
    mask_.vec.u[3] &= 0xFF7FFFFFUL;
    // mask_ < 0 ? a : b
    f128 ret;
    ret.vec.ps[0] = __PS_SEL(mask_.vec.ps[0], b.vec.ps[0], a.vec.ps[0]);
    ret.vec.ps[1] = __PS_SEL(mask_.vec.ps[1], b.vec.ps[1], a.vec.ps[1]);
    return ret;
#endif
}

/*
NLIB_M(f128) F128::ConvertFromFp16Lo(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
    float16x4_t lo = vget_low_f16(vreinterpretq_f16_f32(value));
    return vcvt_f32_f16(lo);
#else
    // _mm_cvtph_ps
    (void)value;
    return F128::SetZero();
#endif
}

NLIB_M(f128) F128::ConvertFromFp16High(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
#ifdef __aarch64__
    return vcvt_high_f32_f16(value);
#else
    float16x4_t hi = vget_high_f16(vreinterpretq_f16_f32(value));
    return vcvt_f32_f16(hi);
#endif
#else
    // _mm_cvtph_ps
    (void)value;
    return F128::SetZero();
#endif
}

NLIB_M(f128) F128::ConvertToFp16(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
#ifdef __aarch64__
    float16x4_t lo = vcvt_f16_f32(a);
    float16x8_t x = vcvt_high_f16_f32(lo, b);
    return vreinterpretq_f32_f16(x);
#else
    float16x4_t lo = vcvt_f16_f32(a);
    float16x4_t hi = vcvt_f16_f32(b);
    return vreinterpretq_f32_f16(vcombine_f16(lo, hi));
#endif
#else
    // _mm_cvtps_ph
    (void)a;
    (void)b;
    return F128::SetZero();
#endif
}
*/

#if !defined(NLIB_F128_SIMD_NOUSE) && !defined(CAFE)
// r[i] = static_cast<float>(value[i])
NLIB_M(f128) F128::ConvertFromI128(i128 value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cvtepi32_ps(value);
#elif defined(NLIB_NEON)
    return vcvtq_f32_s32(vreinterpretq_s32_s8(value));
#endif
}

// r[i] = *reinterpret_cast<float*>(&value[i])
NLIB_M(f128) F128::CastFromI128(i128 value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_castsi128_ps(value);
#elif defined(NLIB_NEON)
    return vreinterpretq_f32_s8(value);
#endif
}

// r[i] = static_cast<int>(roundf(value[i]))
NLIB_M(i128) F128::ConvertToI128Round(f128 value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cvtps_epi32(value);
#elif defined(NLIB_NEON)
    uint32x4_t half = vreinterpretq_u32_f32(vdupq_n_f32(0.5f));
    uint32x4_t sgn = vdupq_n_u32(0x80000000U);
    uint32x4_t w = vandq_u32(vreinterpretq_u32_f32(value), sgn);
    w = vorrq_u32(w, half);
    return vreinterpretq_s8_s32(vcvtq_s32_f32(vaddq_f32(value, vreinterpretq_f32_u32(w))));
#endif
}

NLIB_M(i128) F128::ConvertToI128Truncate(f128 value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cvttps_epi32(value);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_s32(vcvtq_s32_f32(value));
#endif
}

// r[i] = *reinterpret_cast<int*>(&value[i])
NLIB_M(i128) F128::CastToI128(f128 value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_castps_si128(value);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_f32(value);
#endif
}

template<int N>
NLIB_M(f128) F128::ConvertFromFixedPoint(i128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(1 <= N && N <= 32);
#if defined(NLIB_NEON)
    return vcvtq_n_f32_s32(vreinterpretq_s32_s8(value), N);
#else
    f128 f = F128::ConvertFromI128(value);
    f128 m = F128::SetValue(((0x7F - N) << 23), each_uint32);  // 0.5f^N
    return F128::Mult(f, m);
#endif
}

template<int N>
NLIB_M(i128) F128::ConvertToFixedPoint(f128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(1 <= N && N <= 32);
#if defined(NLIB_NEON)
    return vreinterpretq_s8_s32(vcvtq_n_s32_f32(value, N));
#else
    f128 m = F128::SetValue(((0x7F + N) << 23), each_uint32);  // 2.f^N
    f128 f = F128::Mult(value, m);
    return F128::ConvertToI128Truncate(f);
#endif
}

#endif

// r[i] = (a[i] < b[i]) ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::CmpLt(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = (a.vec.v[0] < b.vec.v[0]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[1] = (a.vec.v[1] < b.vec.v[1]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[2] = (a.vec.v[2] < b.vec.v[2]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[3] = (a.vec.v[3] < b.vec.v[3]) ? 0xFFFFFFFFUL : 0;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_cmplt_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vcltq_f32(a, b);
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = (a[i] <= b[i]) ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::CmpLe(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = (a.vec.v[0] <= b.vec.v[0]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[1] = (a.vec.v[1] <= b.vec.v[1]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[2] = (a.vec.v[2] <= b.vec.v[2]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[3] = (a.vec.v[3] <= b.vec.v[3]) ? 0xFFFFFFFFUL : 0;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_cmple_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vcleq_f32(a, b);
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = (a[i] > b[i]) ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::CmpGt(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = (a.vec.v[0] > b.vec.v[0]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[1] = (a.vec.v[1] > b.vec.v[1]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[2] = (a.vec.v[2] > b.vec.v[2]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[3] = (a.vec.v[3] > b.vec.v[3]) ? 0xFFFFFFFFUL : 0;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_cmpgt_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vcgtq_f32(a, b);
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = (a[i] >= b[i]) ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::CmpGe(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = (a.vec.v[0] >= b.vec.v[0]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[1] = (a.vec.v[1] >= b.vec.v[1]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[2] = (a.vec.v[2] >= b.vec.v[2]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[3] = (a.vec.v[3] >= b.vec.v[3]) ? 0xFFFFFFFFUL : 0;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_cmpge_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vcgeq_f32(a, b);
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = (a[i] != b[i]) ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::CmpNe(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = (a.vec.v[0] != b.vec.v[0]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[1] = (a.vec.v[1] != b.vec.v[1]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[2] = (a.vec.v[2] != b.vec.v[2]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[3] = (a.vec.v[3] != b.vec.v[3]) ? 0xFFFFFFFFUL : 0;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_cmpneq_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vmvnq_u32(vceqq_f32(a, b));
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = a[i] + b[i]
NLIB_M(f128) F128::Add(f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a.vec.v[0] + b.vec.v[0];
    ret.vec.v[1] = a.vec.v[1] + b.vec.v[1];
    ret.vec.v[2] = a.vec.v[2] + b.vec.v[2];
    ret.vec.v[3] = a.vec.v[3] + b.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_add_ps(a, b);
#elif defined(NLIB_NEON)
    return vaddq_f32(a, b);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = __PS_ADD(a.vec.ps[0], b.vec.ps[0]);
    ret.vec.ps[1] = __PS_ADD(a.vec.ps[1], b.vec.ps[1]);
    return ret;
#endif
}

// r[i] = a[i] - b[i]
NLIB_M(f128) F128::Sub(f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a.vec.v[0] - b.vec.v[0];
    ret.vec.v[1] = a.vec.v[1] - b.vec.v[1];
    ret.vec.v[2] = a.vec.v[2] - b.vec.v[2];
    ret.vec.v[3] = a.vec.v[3] - b.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_sub_ps(a, b);
#elif defined(NLIB_NEON)
    return vsubq_f32(a, b);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = __PS_SUB(a.vec.ps[0], b.vec.ps[0]);
    ret.vec.ps[1] = __PS_SUB(a.vec.ps[1], b.vec.ps[1]);
    return ret;
#endif
}

// r[i] = -value[i]
NLIB_M(f128) F128::Negate(f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = -value.vec.v[0];
    ret.vec.v[1] = -value.vec.v[1];
    ret.vec.v[2] = -value.vec.v[2];
    ret.vec.v[3] = -value.vec.v[3];
    return ret;
#elif defined(NLIB_NEON)
    return vnegq_f32(value);
#elif defined(NLIB_SSE41)
    const __m128 signmask = _mm_set1_ps(-0.0f);  // 0x80000000
    return _mm_xor_ps(signmask, value);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = __PS_NEG(value.vec.ps[0]);
    ret.vec.ps[1] = __PS_NEG(value.vec.ps[1]);
    return ret;
#endif
}

// r[i] = a[i] * b[i]
NLIB_M(f128) F128::Mult(f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a.vec.v[0] * b.vec.v[0];
    ret.vec.v[1] = a.vec.v[1] * b.vec.v[1];
    ret.vec.v[2] = a.vec.v[2] * b.vec.v[2];
    ret.vec.v[3] = a.vec.v[3] * b.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_mul_ps(a, b);
#elif defined(NLIB_NEON)
    return vmulq_f32(a, b);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = __PS_MUL(a.vec.ps[0], b.vec.ps[0]);
    ret.vec.ps[1] = __PS_MUL(a.vec.ps[1], b.vec.ps[1]);
    return ret;
#endif
}

// r[i] = a * b[i]
NLIB_M(f128) F128::Mult(float a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
    return vmulq_n_f32(b, a);
#elif defined(CAFE) && !defined(NLIB_F128_SIMD_NOUSE)
    f128 ret;
    ret.vec.ps[0] = __PS_MULS0F(b.vec.ps[0], a);
    ret.vec.ps[1] = __PS_MULS0F(b.vec.ps[1], a);
    return ret;
#else
    return F128::Mult(b, F128::SetValue(a, each_float));
#endif
}

template <size_t N>
// r[i] = a[N] * b[i]
NLIB_M(f128) F128::Mult(f128arg a, f128arg b, each_select32_tag) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
# if __aarch64__
    return vmulq_laneq_f32(b, a, N);
# else
    float tmp = vget_lane_f32((N < 2 ? vget_low_f32(a) : vget_high_f32(a)), (N & 1));
    return vmulq_n_f32(b, tmp);
# endif
#elif defined(CAFE) && !defined(NLIB_F128_SIMD_NOUSE)
    float t = a.vec.ps[N / 2][N % 2];
    f128 ret;
    ret.vec.ps[0] = __PS_MULS0F(b.vec.ps[0], t);
    ret.vec.ps[1] = __PS_MULS0F(b.vec.ps[1], t);
    return ret;
#else
    return F128::Mult(F128::SetValue<N>(a, each_select32), b);
#endif
}

// r[i] = a[i] / b[i]
NLIB_M(f128) F128::Div(f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a.vec.v[0] / b.vec.v[0];
    ret.vec.v[1] = a.vec.v[1] / b.vec.v[1];
    ret.vec.v[2] = a.vec.v[2] / b.vec.v[2];
    ret.vec.v[3] = a.vec.v[3] / b.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_div_ps(a, b);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    return vdivq_f32(a, b);
# else
    float32x4_t inv0 = vrecpeq_f32(b);
    float32x4_t step0 = vrecpsq_f32(inv0, b);
    float32x4_t inv1 = vmulq_f32(step0, inv0);
    float32x4_t step1 = vrecpsq_f32(inv1, b);
    float32x4_t inv2 = vmulq_f32(step1, inv1);
    uint32x4_t zeromask = vceqq_f32(b, vdupq_n_f32(0));
    inv2 = vbslq_f32(zeromask, F128::SetInfinity(), inv2);
    return vmulq_f32(a, inv2);
# endif
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = __PS_DIV(a.vec.ps[0], b.vec.ps[0]);
    ret.vec.ps[1] = __PS_DIV(a.vec.ps[1], b.vec.ps[1]);
    return ret;
#endif
}

// r[i] = max(a[i], b[i])
NLIB_M(f128) F128::Max(f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a.vec.v[0] > b.vec.v[0] ? a.vec.v[0] : b.vec.v[0];
    ret.vec.v[1] = a.vec.v[1] > b.vec.v[1] ? a.vec.v[1] : b.vec.v[1];
    ret.vec.v[2] = a.vec.v[2] > b.vec.v[2] ? a.vec.v[2] : b.vec.v[2];
    ret.vec.v[3] = a.vec.v[3] > b.vec.v[3] ? a.vec.v[3] : b.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_max_ps(a, b);
#elif defined(NLIB_NEON)
    return vmaxq_f32(a, b);
#elif defined(CAFE)
    f32x2 cmp0 = __PS_SUB(a.vec.ps[0], b.vec.ps[0]);
    f32x2 cmp1 = __PS_SUB(a.vec.ps[1], b.vec.ps[1]);
    f128 ret;
    ret.vec.ps[0] = __PS_SEL(cmp0, a.vec.ps[0], b.vec.ps[0]);
    ret.vec.ps[1] = __PS_SEL(cmp1, a.vec.ps[1], b.vec.ps[1]);
    return ret;
#endif
}

// r[i] = min(a[i], b[i])
NLIB_M(f128) F128::Min(f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a.vec.v[0] < b.vec.v[0] ? a.vec.v[0] : b.vec.v[0];
    ret.vec.v[1] = a.vec.v[1] < b.vec.v[1] ? a.vec.v[1] : b.vec.v[1];
    ret.vec.v[2] = a.vec.v[2] < b.vec.v[2] ? a.vec.v[2] : b.vec.v[2];
    ret.vec.v[3] = a.vec.v[3] < b.vec.v[3] ? a.vec.v[3] : b.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_min_ps(a, b);
#elif defined(NLIB_NEON)
    return vminq_f32(a, b);
#elif defined(CAFE)
    f32x2 cmp0 = __PS_SUB(a.vec.ps[0], b.vec.ps[0]);
    f32x2 cmp1 = __PS_SUB(a.vec.ps[1], b.vec.ps[1]);
    f128 ret;
    ret.vec.ps[0] = __PS_SEL(cmp0, b.vec.ps[0], a.vec.ps[0]);
    ret.vec.ps[1] = __PS_SEL(cmp1, b.vec.ps[1], a.vec.ps[1]);
    return ret;
#endif
}

// r[0] = max(a[0], a[1]), r[1] = max(a[2], a[3]), r[2] = max(b[0], b[1]), r[3] = max(b[2], b[3])
NLIB_M(f128) F128::PairwiseMax(f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a.vec.v[0] > a.vec.v[1] ? a.vec.v[0] : a.vec.v[1];
    ret.vec.v[1] = a.vec.v[2] > a.vec.v[3] ? a.vec.v[2] : a.vec.v[3];
    ret.vec.v[2] = b.vec.v[0] > b.vec.v[1] ? b.vec.v[0] : b.vec.v[1];
    ret.vec.v[3] = b.vec.v[2] > b.vec.v[3] ? b.vec.v[2] : b.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    f128 ax = _mm_max_ps(a, _mm_shuffle_ps(a, a, _MM_SHUFFLE(2, 3, 0, 1)));
    f128 bx = _mm_max_ps(b, _mm_shuffle_ps(b, b, _MM_SHUFFLE(2, 3, 0, 1)));
    return _mm_shuffle_ps(ax, bx, _MM_SHUFFLE(2, 0, 2, 0));
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    return vpmaxq_f32(a, b);
# else
    float32x2_t rl = vpmax_f32(vget_low_f32(a), vget_high_f32(a));
    float32x2_t rh = vpmax_f32(vget_low_f32(b), vget_high_f32(b));
    return vcombine_f32(rl, rh);
# endif
#elif defined(CAFE)
    f32x2 v02, v13, cmp;
    f128 ret;
    v02 = __PS_MERGE00(a.vec.ps[0], a.vec.ps[1]);
    v13 = __PS_MERGE11(a.vec.ps[0], a.vec.ps[1]);
    cmp = __PS_SUB(v02, v13);
    ret.vec.ps[0] = __PS_SEL(cmp, v02, v13);
    v02 = __PS_MERGE00(b.vec.ps[0], b.vec.ps[1]);
    v13 = __PS_MERGE11(b.vec.ps[0], b.vec.ps[1]);
    cmp = __PS_SUB(v02, v13);
    ret.vec.ps[1] = __PS_SEL(cmp, v02, v13);
    return ret;
#endif
}

// r[0] = min(a[0], a[1]), r[1] = min(a[2], a[3]), r[2] = min(b[0], b[1]), r[3] = min(b[2], b[3])
NLIB_M(f128) F128::PairwiseMin(f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a.vec.v[0] < a.vec.v[1] ? a.vec.v[0] : a.vec.v[1];
    ret.vec.v[1] = a.vec.v[2] < a.vec.v[3] ? a.vec.v[2] : a.vec.v[3];
    ret.vec.v[2] = b.vec.v[0] < b.vec.v[1] ? b.vec.v[0] : b.vec.v[1];
    ret.vec.v[3] = b.vec.v[2] < b.vec.v[3] ? b.vec.v[2] : b.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    f128 ax = _mm_min_ps(a, _mm_shuffle_ps(a, a, _MM_SHUFFLE(2, 3, 0, 1)));
    f128 bx = _mm_min_ps(b, _mm_shuffle_ps(b, b, _MM_SHUFFLE(2, 3, 0, 1)));
    return _mm_shuffle_ps(ax, bx, _MM_SHUFFLE(2, 0, 2, 0));
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    return vpminq_f32(a, b);
# else
    float32x2_t rl = vpmin_f32(vget_low_f32(a), vget_high_f32(a));
    float32x2_t rh = vpmin_f32(vget_low_f32(b), vget_high_f32(b));
    return vcombine_f32(rl, rh);
# endif
#elif defined(CAFE)
    f32x2 v02, v13, cmp;
    f128 ret;
    v02 = __PS_MERGE00(a.vec.ps[0], a.vec.ps[1]);
    v13 = __PS_MERGE11(a.vec.ps[0], a.vec.ps[1]);
    cmp = __PS_SUB(v02, v13);
    ret.vec.ps[0] = __PS_SEL(cmp, v13, v02);
    v02 = __PS_MERGE00(b.vec.ps[0], b.vec.ps[1]);
    v13 = __PS_MERGE11(b.vec.ps[0], b.vec.ps[1]);
    cmp = __PS_SUB(v02, v13);
    ret.vec.ps[1] = __PS_SEL(cmp, v13, v02);
    return ret;
#endif
}

// r[0] = a[0] + a[1], r[1] = a[2] + a[3], ...
NLIB_M(f128) F128::PairwiseAdd(f128arg a, f128arg b) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = a.vec.v[0] + a.vec.v[1];
    ret.vec.v[1] = a.vec.v[2] + a.vec.v[3];
    ret.vec.v[2] = b.vec.v[0] + b.vec.v[1];
    ret.vec.v[3] = b.vec.v[2] + b.vec.v[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_hadd_ps(a, b);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    return vpaddq_f32(a, b);
# else
    float32x2_t al = vget_low_f32(a);
    float32x2_t ah = vget_high_f32(a);
    float32x2_t l = vpadd_f32(al, ah);

    float32x2_t bl = vget_low_f32(b);
    float32x2_t bh = vget_high_f32(b);
    float32x2_t h = vpadd_f32(bl, bh);
    return vcombine_f32(l, h);
# endif
#elif defined(CAFE)
    f32x2 v02, v13, cmp;
    f128 ret;
    v02 = __PS_MERGE00(a.vec.ps[0], a.vec.ps[1]);
    v13 = __PS_MERGE11(a.vec.ps[0], a.vec.ps[1]);
    ret.vec.ps[0] = __PS_ADD(v02, v13);
    v02 = __PS_MERGE00(b.vec.ps[0], b.vec.ps[1]);
    v13 = __PS_MERGE11(b.vec.ps[0], b.vec.ps[1]);
    ret.vec.ps[1] = __PS_ADD(v02, v13);
    return ret;
#endif
}

// r[i] = fabs(a[i] - b[i])
NLIB_M(f128) F128::AbsDiff(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
    return vabdq_f32(a, b);
#else
    return F128::Abs(F128::Sub(a, b));
#endif
}

// r[i] = c[i] + a[i] * b[i]
NLIB_M(f128) F128::MultAdd(f128arg a, f128arg b, f128arg c) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
# if __aarch64__
    return vfmaq_f32(c, a, b);
# else
    return vmlaq_f32(c, a, b);
# endif
#elif defined(CAFE) && !defined(NLIB_F128_SIMD_NOUSE)
    f128 ret;
    ret.vec.ps[0] = __PS_MADD(a.vec.ps[0], b.vec.ps[0], c.vec.ps[0]);
    ret.vec.ps[1] = __PS_MADD(a.vec.ps[1], b.vec.ps[1], c.vec.ps[1]);
    return ret;
#else
    return F128::Add(c, F128::Mult(a, b));
#endif
}

// r[i] = c[i] + a * b[i]
NLIB_M(f128) F128::MultAdd(float a, f128arg b, f128arg c) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
# if __aarch64__
    return vfmaq_n_f32(c, b, a);
# else
    return vmlaq_n_f32(c, b, a);
# endif
#else
    return F128::MultAdd(F128::SetValue(a, each_float), b, c);
#endif
}

template <size_t N>
// r[i] = c[i] + a[N] * b[i]
NLIB_M(f128) F128::MultAdd(f128arg a, f128arg b, f128arg c,
                            each_select32_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
# if __aarch64__
    return vfmaq_laneq_f32(c, b, a, N);
# else
    return vmlaq_lane_f32(c, b, N < 2 ? vget_low_f32(a) : vget_high_f32(a), (N & 1));
# endif
#else
    return F128::MultAdd(F128::SetValue<N>(a, each_select32), b, c);
#endif
}

// r[i] = c[i] - a[i] * b[i]
NLIB_M(f128) F128::MultSub(f128arg a, f128arg b, f128arg c) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
# if __aarch64__
    return vfmsq_f32(c, a, b);
# else
    return vmlsq_f32(c, a, b);
# endif
#elif defined(CAFE) && !defined(NLIB_F128_SIMD_NOUSE)
    f128 ret;
    ret.vec.ps[0] = __PS_NMSUB(a.vec.ps[0], b.vec.ps[0], c.vec.ps[0]);
    ret.vec.ps[1] = __PS_NMSUB(a.vec.ps[1], b.vec.ps[1], c.vec.ps[1]);
    return ret;
#else
    return F128::Sub(c, F128::Mult(a, b));
#endif
}

// r[i] = c[i] - a * b[i]
NLIB_M(f128) F128::MultSub(float a, f128arg b, f128arg c) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
# if __aarch64__
    return vfmsq_n_f32(c, b, a);
# else
    return vmlsq_n_f32(c, b, a);
# endif
#else
    return F128::MultSub(F128::SetValue(a, each_float), b, c);
#endif
}

template <size_t N>
// r[i] = c[i] - a[N] * b[i]
NLIB_M(f128) F128::MultSub(f128arg a, f128arg b, f128arg c,
                            each_select32_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
# ifdef __arch64__
    return vfmsq_laneq_f32(c, b, a, N);
# else
    return vmlsq_lane_f32(c, b, N < 2 ? vget_low_f32(a) : vget_high_f32(a), (N & 1));
# endif
#else
    return F128::MultSub(F128::SetValue<N>(a, each_select32), b, c);
#endif
}

// r[i] = a[i] + t[i] * (b[i] - a[i])
NLIB_M(f128) F128::Lerp(f128arg a, f128arg b, f128arg t) NLIB_NOEXCEPT {
    // a + t * (b - a)
    return F128::MultAdd(t, F128::Sub(b, a), a);
}

// r[i] = a[i] & b[i]
NLIB_M(f128) F128::And(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = a.vec.u[0] & b.vec.u[0];
    ret.vec.u[1] = a.vec.u[1] & b.vec.u[1];
    ret.vec.u[2] = a.vec.u[2] & b.vec.u[2];
    ret.vec.u[3] = a.vec.u[3] & b.vec.u[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_and_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vandq_u32(vreinterpretq_u32_f32(a), vreinterpretq_u32_f32(b));
    return vreinterpretq_f32_u32(tmp);
#endif
}

// -pi <= angle1, angle2, r < pi
NLIB_M2(f128) F128::AddAngle(f128arg angle1, f128arg angle2) NLIB_NOEXCEPT {
    // -pi <= angle1 < pi, -2pi <= angle2 <= 2pi
    // -pi <= ret < pi
    f128 pi_pi2 = F128::LoadA16(F128::pi_values_);
    f128 pi_dbl = F128::SetValue<2>(pi_pi2, each_select32);

    f128 sum = F128::Add(angle1, angle2);
    f128 cond = F128::CmpLt(sum, F128::SetValue<1>(pi_pi2, each_select32));
    f128 ofs = F128::And(cond, pi_dbl);
    f128 result = F128::Add(sum, ofs);
    cond = F128::CmpGe(sum, F128::SetValue<0>(pi_pi2, each_select32));
    ofs = F128::And(cond, pi_dbl);
    return F128::Sub(result, ofs);
}

// -pi <= angle1, angle2, r < pi
NLIB_M2(f128) F128::SubAngle(f128arg angle1, f128arg angle2) NLIB_NOEXCEPT {
    // -pi <= angle1 < pi, -2pi <= angle2 <= 2pi
    // -pi <= ret < pi
    f128 pi_pi2 = F128::LoadA16(F128::pi_values_);
    f128 pi_dbl = F128::SetValue<2>(pi_pi2, each_select32);

    f128 sum = F128::Sub(angle1, angle2);
    f128 cond = F128::CmpLt(sum, F128::SetValue<1>(pi_pi2, each_select32));
    f128 ofs = F128::And(cond, pi_dbl);
    f128 result = F128::Add(sum, ofs);
    cond = F128::CmpGe(sum, F128::SetValue<0>(pi_pi2, each_select32));
    ofs = F128::And(cond, pi_dbl);
    return F128::Sub(result, ofs);
}

//               ( 2  1 -2  1) (p0)
// (t^3 t^2 t 1) (-3 -2  3 -1) (v0)
//               ( 0  1  0  0) (p1)
//               ( 1  0  0  0) (v1)
NLIB_M2(f128) F128::Hermite(f128arg p0, f128arg v0, f128arg p1, f128arg_ex v1,
                            f128arg_ex t) NLIB_NOEXCEPT {
    // (2 * p0 + v0 - 2 * p1 + v1) * t^3 + (-3 * p0 - 2 * v0 + 3 * p1 - v1) * t^2
    //   + v0 * t + p0
    // ==
    // (2 * t^3 - 3 * t^2 + 1) * p0 + (t^3 - 2 * t^2 + t) * v0
    //   + (-2 * t^3 + 3 * t^2) * p1 + (t^3 - t^2) * v1
    f128 tt = F128::Mult(t, t);
    f128 ttt = F128::Mult(tt, t);
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
    f128 hermite_R0 = vcombine_f32(vcreate_f32(0x3F80000040000000ULL),
                                   vcreate_f32(0x3F800000C0000000ULL));
    f128 hermite_R1 = vcombine_f32(vcreate_f32(0xC0000000C0400000ULL),
                                   vcreate_f32(0xBF80000040400000ULL));
#else
    f128 hermite_R0 = F128::LoadA16(hermite_R0_);
    f128 hermite_R1 = F128::LoadA16(hermite_R1_);
#endif

    ttt = F128::Mult(ttt, hermite_R0);
    ttt = F128::MultAdd(tt, hermite_R1, ttt);
    ttt = F128::MultAdd(t, F128::Set0100(), ttt);
    ttt = F128::Add(ttt, F128::Set1000());

    // vec(ttt) * mtx(p0, v0, p1, v1)
    f128 result = F128::Mult<0>(ttt, p0, each_select32);
    result = F128::MultAdd<1>(ttt, v0, result, each_select32);
    result = F128::MultAdd<2>(ttt, p1, result, each_select32);
    result = F128::MultAdd<3>(ttt, v1, result, each_select32);
    return result;
}

//                     (-1  3 -3  1) (p0)
// 0.5 * (t^3 t^2 t 1) ( 2 -5  4 -1) (p1)
//                     (-1  0  1  0) (p2)
//                     ( 0  2  0  0) (p3)
NLIB_M2(f128) F128::CatmullRom(f128arg p0, f128arg p1, f128arg p2, f128arg_ex p3,
                               f128arg_ex t) NLIB_NOEXCEPT {
    f128 tt = F128::Mult(t, t);
    f128 ttt = F128::Mult(tt, t);
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
    f128 catmull_R0 = vcombine_f32(vcreate_f32(0x40400000BF800000ULL),
                                   vcreate_f32(0x3F800000C0400000ULL));
    f128 catmull_R1 = vcombine_f32(vcreate_f32(0xC0A0000040000000ULL),
                                   vcreate_f32(0xBF80000040800000ULL));
    f128 catmull_R2 = vcombine_f32(vcreate_f32(0x00000000BF800000ULL),
                                   vcreate_f32(0x000000003F800000ULL));
#else
    f128 catmull_R0 = F128::LoadA16(catmull_R0_);
    f128 catmull_R1 = F128::LoadA16(catmull_R1_);
    f128 catmull_R2 = F128::LoadA16(catmull_R2_);
#endif
    ttt = F128::Mult(ttt, catmull_R0);
    ttt = F128::MultAdd(tt, catmull_R1, ttt);
    ttt = F128::MultAdd(t, catmull_R2, ttt);
    ttt = F128::Add(ttt, F128::Set0100());

    // vec(ttt) * mtx(p0, p1, p2, p3)
    f128 result = F128::Mult<0>(ttt, p0, each_select32);
    result = F128::MultAdd<1>(ttt, p1, result, each_select32);
    result = F128::MultAdd<2>(ttt, p2, result, each_select32);
    result = F128::MultAdd<3>(ttt, p3, result, each_select32);

    return result;
}

// p0 + f * (p1 - p0) + g * (p2 - p0)
NLIB_M(f128) F128::BaryCentric(f128arg p0, f128arg p1, f128arg p2, f128arg_ex f,
                               f128arg_ex g) NLIB_NOEXCEPT {
    f128 p1p0 = F128::Sub(p1, p0);
    f128 p2p0 = F128::Sub(p2, p0);
    f128 tmp = F128::MultAdd(f, p1p0, p0);
    return F128::MultAdd(g, p2p0, tmp);
}

// r[i] = a[i] | b[i]
NLIB_M(f128) F128::Or(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = a.vec.u[0] | b.vec.u[0];
    ret.vec.u[1] = a.vec.u[1] | b.vec.u[1];
    ret.vec.u[2] = a.vec.u[2] | b.vec.u[2];
    ret.vec.u[3] = a.vec.u[3] | b.vec.u[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_or_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vorrq_u32(vreinterpretq_u32_f32(a), vreinterpretq_u32_f32(b));
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = a[i] ^ b[i]
NLIB_M(f128) F128::Xor(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = a.vec.u[0] ^ b.vec.u[0];
    ret.vec.u[1] = a.vec.u[1] ^ b.vec.u[1];
    ret.vec.u[2] = a.vec.u[2] ^ b.vec.u[2];
    ret.vec.u[3] = a.vec.u[3] ^ b.vec.u[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_xor_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = veorq_u32(vreinterpretq_u32_f32(a), vreinterpretq_u32_f32(b));
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = ~a[i]
NLIB_M(f128) F128::Not(f128arg a) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = ~a.vec.u[0];
    ret.vec.u[1] = ~a.vec.u[1];
    ret.vec.u[2] = ~a.vec.u[2];
    ret.vec.u[3] = ~a.vec.u[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_andnot_ps(a, F128::CmpEq(a, a));
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vmvnq_u32(vreinterpretq_u32_f32(a));
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = ~a[i] & b[i]
NLIB_M(f128) F128::AndNot(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = ~a.vec.u[0] & b.vec.u[0];
    ret.vec.u[1] = ~a.vec.u[1] & b.vec.u[1];
    ret.vec.u[2] = ~a.vec.u[2] & b.vec.u[2];
    ret.vec.u[3] = ~a.vec.u[3] & b.vec.u[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_andnot_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vbicq_u32(vreinterpretq_u32_f32(b), vreinterpretq_u32_f32(a));
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = ~a[i] | b[i]
NLIB_M(f128) F128::OrNot(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = ~a.vec.u[0] | b.vec.u[0];
    ret.vec.u[1] = ~a.vec.u[1] | b.vec.u[1];
    ret.vec.u[2] = ~a.vec.u[2] | b.vec.u[2];
    ret.vec.u[3] = ~a.vec.u[3] | b.vec.u[3];
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_or_ps(F128::Not(a), b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vornq_u32(vreinterpretq_u32_f32(b), vreinterpretq_u32_f32(a));
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = (a[i] == b[i]) ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::CmpEq(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.u[0] = (a.vec.v[0] == b.vec.v[0]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[1] = (a.vec.v[1] == b.vec.v[1]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[2] = (a.vec.v[2] == b.vec.v[2]) ? 0xFFFFFFFFUL : 0;
    ret.vec.u[3] = (a.vec.v[3] == b.vec.v[3]) ? 0xFFFFFFFFUL : 0;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_cmpeq_ps(a, b);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vceqq_f32(a, b);
    return vreinterpretq_f32_u32(tmp);
#endif
}

// r[i] = (absf(a[i] - b[i]) <= eps[i]) ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::CmpNearEq(f128arg a, f128arg b, f128arg eps) NLIB_NOEXCEPT {
    f128 tmp = F128::AbsDiff(a, b);
    return F128::CmpLe(tmp, eps);
}

// r[i] = clamp(value[i], min[i], max[i])
NLIB_M(f128) F128::Clamp(f128arg value, f128arg min, f128arg max) NLIB_NOEXCEPT {
    return F128::Min(max, F128::Max(min, value));
}

// r[i] = absf(value[i]) <= bounds[i] ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::InBound(f128arg value, f128arg bounds) NLIB_NOEXCEPT {
#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
    uint32x4_t tmp = vcaleq_f32(value, bounds);
    return vreinterpretq_f32_u32(tmp);
#else
    return F128::CmpLe(F128::Abs(value), bounds);
#endif
}

NLIB_M(f128) F128::CmpEqZero(f128arg value) NLIB_NOEXCEPT {
#if defined(__aarch64__) && !defined(NLIB_F128_SIMD_NOUSE)
    return vreinterpretq_f32_u32(vceqzq_f32(value));
#else
    return F128::CmpEq(value, F128::SetZero());
#endif
}

NLIB_M(f128) F128::CmpLtZero(f128arg value) NLIB_NOEXCEPT {
#if defined(__aarch64__) && !defined(NLIB_F128_SIMD_NOUSE)
    return vreinterpretq_f32_u32(vcltzq_f32(value));
#else
    return F128::CmpLt(value, F128::SetZero());
#endif
}

NLIB_M(f128) F128::CmpLeZero(f128arg value) NLIB_NOEXCEPT {
#if defined(__aarch64__) && !defined(NLIB_F128_SIMD_NOUSE)
    return vreinterpretq_f32_u32(vclezq_f32(value));
#else
    return F128::CmpLe(value, F128::SetZero());
#endif
}

NLIB_M(f128) F128::CmpGtZero(f128arg value) NLIB_NOEXCEPT {
#if defined(__aarch64__) && !defined(NLIB_F128_SIMD_NOUSE)
    return vreinterpretq_f32_u32(vcgtzq_f32(value));
#else
    return F128::CmpGt(value, F128::SetZero());
#endif
}

NLIB_M(f128) F128::CmpGeZero(f128arg value) NLIB_NOEXCEPT {
#if defined(__aarch64__) && !defined(NLIB_F128_SIMD_NOUSE)
    return vreinterpretq_f32_u32(vcgezq_f32(value));
#else
    return F128::CmpGe(value, F128::SetZero());
#endif
}

NLIB_M(f128) F128::CmpNeZero(f128arg value) NLIB_NOEXCEPT {
#if defined(__aarch64__) && !defined(NLIB_F128_SIMD_NOUSE)
    return vreinterpretq_f32_u32(vmvnq_u32(vceqzq_f32(value)));
#else
    return F128::CmpNe(value, F128::SetZero());
#endif
}

// r[i] = (absf(value[i]) <= eps[i]) ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::CmpNearEqZero(f128arg value, f128arg eps) NLIB_NOEXCEPT {
    f128 tmp = F128::Abs(value);
    return F128::CmpLe(tmp, eps);
}

// r[i] = 1.f / value[i] with higher precision, set infinity if value[i] == 0
NLIB_M2(f128) F128::Recp(f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = (value.vec.v[0] != 0.f) ? 1.f / value.vec.v[0] : INFINITY;
    ret.vec.v[1] = (value.vec.v[1] != 0.f) ? 1.f / value.vec.v[1] : INFINITY;
    ret.vec.v[2] = (value.vec.v[2] != 0.f) ? 1.f / value.vec.v[2] : INFINITY;
    ret.vec.v[3] = (value.vec.v[3] != 0.f) ? 1.f / value.vec.v[3] : INFINITY;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_div_ps(F128::SetOne(), value);
#elif defined(NLIB_NEON)
#ifdef __aarch64__
    return vdivq_f32(vdupq_n_f32(1.f), value);
#else
    float32x4_t x;
    x = vrecpeq_f32(value);
    x = vmulq_f32(x, vrecpsq_f32(x, value));  // x1 = x0 * (2 - x0 * value)
    x = vmulq_f32(x, vrecpsq_f32(x, value));  // x2 = x1 * (2 - x1 * value)
    uint32x4_t zeromask = vceqq_f32(value, vdupq_n_f32(0));
    float32x4_t result = vbslq_f32(zeromask, F128::SetInfinity(), x);
    return result;
#endif
#elif defined(CAFE)
    return F128::Div(F128::SetOne(), value);
#endif
}

// r[i] = 1.f / value[i] with lower precision
NLIB_M(f128) F128::RecpEst(f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = (value.vec.v[0] != 0.f) ? 1.f / value.vec.v[0] : INFINITY;
    ret.vec.v[1] = (value.vec.v[1] != 0.f) ? 1.f / value.vec.v[1] : INFINITY;
    ret.vec.v[2] = (value.vec.v[2] != 0.f) ? 1.f / value.vec.v[2] : INFINITY;
    ret.vec.v[3] = (value.vec.v[3] != 0.f) ? 1.f / value.vec.v[3] : INFINITY;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_rcp_ps(value);
#elif defined(NLIB_NEON)
    return vrecpeq_f32(value);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = __PS_RES(value.vec.ps[0]);
    ret.vec.ps[1] = __PS_RES(value.vec.ps[1]);
    return ret;
#endif
}

// r[i] = sqrtf(value[i]) with higher precision
NLIB_M2(f128) F128::Sqrt(f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = sqrtf(value.vec.v[0]);
    ret.vec.v[1] = sqrtf(value.vec.v[1]);
    ret.vec.v[2] = sqrtf(value.vec.v[2]);
    ret.vec.v[3] = sqrtf(value.vec.v[3]);
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_sqrt_ps(value);
#elif defined(NLIB_NEON)
    f128 iszero = F128::CmpEqZero(value);
    f128 result = F128::Mult(value, F128::RecpSqrt(value));
    return F128::AndNot(iszero, result);
#elif defined(CAFE)
    f128 zero = F128::SetZero();
    f128 iszero = F128::CmpEq(zero, value);
    f128 result = F128::Mult(value, F128::RecpSqrt(value));
    return F128::Select(iszero, zero, result);
#endif
}

// r[i] = sqrtf(value[i]) with lower precision
NLIB_M(f128) F128::SqrtEst(f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = sqrtf(value.vec.v[0]);
    ret.vec.v[1] = sqrtf(value.vec.v[1]);
    ret.vec.v[2] = sqrtf(value.vec.v[2]);
    ret.vec.v[3] = sqrtf(value.vec.v[3]);
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_sqrt_ps(value);
#elif defined(NLIB_NEON)
    return vrecpeq_f32(vrsqrteq_f32(value));
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = __PS_RES(__PS_RSQRTE(value.vec.ps[0]));
    ret.vec.ps[1] = __PS_RES(__PS_RSQRTE(value.vec.ps[1]));
    return ret;
#endif
}

// r[i] = sqrtf(1.f / value[i]) with higher precision
NLIB_M2(f128) F128::RecpSqrt(f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = (value.vec.v[0] != 0.f) ? 1.f / sqrtf(value.vec.v[0]) : INFINITY;
    ret.vec.v[1] = (value.vec.v[1] != 0.f) ? 1.f / sqrtf(value.vec.v[1]) : INFINITY;
    ret.vec.v[2] = (value.vec.v[2] != 0.f) ? 1.f / sqrtf(value.vec.v[2]) : INFINITY;
    ret.vec.v[3] = (value.vec.v[3] != 0.f) ? 1.f / sqrtf(value.vec.v[3]) : INFINITY;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_div_ps(F128::SetOne(), F128::Sqrt(value));
#elif defined(NLIB_NEON)
    float32x4_t x;
    x = vrsqrteq_f32(value);
    x = vmulq_f32(x, vrsqrtsq_f32(value, vmulq_f32(x, x)));
    x = vmulq_f32(x, vrsqrtsq_f32(value, vmulq_f32(x, x)));
    f128 zeromask = F128::CmpEqZero(value);
    return F128::Select(zeromask, F128::SetInfinity(), x);
#elif defined(CAFE)
    f32x2 three = __PS_FDUP(3.f);
    f32x2 half = __PS_FDUP(0.5f);
    f32x2 x;
    f32x2 xx;
    f32x2 v;
    f128 ret;

    v = value.vec.ps[0];
    x = __PS_RSQRTE(v);

    xx = __PS_MUL(x, x);
    xx = __PS_NMSUB(v, xx, three);
    xx = __PS_MUL(x, xx);
    x = __PS_MUL(half, xx);

    xx = __PS_MUL(x, x);
    xx = __PS_NMSUB(v, xx, three);
    xx = __PS_MUL(x, xx);
    ret.vec.ps[0] = __PS_MUL(half, xx);

    v = value.vec.ps[1];
    x = __PS_RSQRTE(v);

    xx = __PS_MUL(x, x);
    xx = __PS_NMSUB(v, xx, three);
    xx = __PS_MUL(x, xx);
    x = __PS_MUL(half, xx);

    xx = __PS_MUL(x, x);
    xx = __PS_NMSUB(v, xx, three);
    xx = __PS_MUL(x, xx);
    ret.vec.ps[1] = __PS_MUL(half, xx);

    f128 iszero = F128::CmpEq(F128::SetZero(), value);
    f128 inf = F128::SetInfinity();
    return F128::Select(iszero, inf, ret);
#endif
}

// r[i] = sqrtf(1.f / value[i]) with lower precision
NLIB_M(f128) F128::RecpSqrtEst(f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = (value.vec.v[0] != 0.f) ? 1.f / sqrtf(value.vec.v[0]) : INFINITY;
    ret.vec.v[1] = (value.vec.v[1] != 0.f) ? 1.f / sqrtf(value.vec.v[1]) : INFINITY;
    ret.vec.v[2] = (value.vec.v[2] != 0.f) ? 1.f / sqrtf(value.vec.v[2]) : INFINITY;
    ret.vec.v[3] = (value.vec.v[3] != 0.f) ? 1.f / sqrtf(value.vec.v[3]) : INFINITY;
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_rsqrt_ps(value);
#elif defined(NLIB_NEON)
    return vrsqrteq_f32(value);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = __PS_RSQRTE(value.vec.ps[0]);
    ret.vec.ps[1] = __PS_RSQRTE(value.vec.ps[1]);
    return ret;
#endif
}

template <bool NegateLane0, bool NegateLane1, bool NegateLane2, bool NegateLane3>
NLIB_M(f128) F128::NegateEx(f128arg value) NLIB_NOEXCEPT {
    const size_t lane0 = NegateLane0 ? 4 : 0;
    const size_t lane1 = NegateLane1 ? 5 : 1;
    const size_t lane2 = NegateLane2 ? 6 : 2;
    const size_t lane3 = NegateLane3 ? 7 : 3;
    return F128::Permute<lane0, lane1, lane2, lane3>(value, F128::Negate(value));
}

template <>
NLIB_M(f128) F128::NegateEx<false, false, false, false>(f128arg value) NLIB_NOEXCEPT {
    return value;
}

template <>
NLIB_M(f128) F128::NegateEx<true, true, true, true>(f128arg value) NLIB_NOEXCEPT {
    return F128::Negate(value);
}

#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
#define NLIB_ISNAN(vec, idx) \
    ((vec.u[idx] & 0x7F800000U) == 0x7F800000U && (vec.u[idx] & 0x7FFFFFU) != 0)
#define NLIB_ISINF(vec, idx) ((vec.u[idx] & 0x7FFFFFFFU) == 0x7F800000U)
#endif

// r[i] = isnan(value[i]) ? 0xFFFFFFFF : 0
NLIB_M2(f128) F128::IsNaN(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE)
    f128 ret;
    ret.vec.u[0] = NLIB_ISNAN(value.vec, 0) ? 0xFFFFFFFFU : 0;
    ret.vec.u[1] = NLIB_ISNAN(value.vec, 1) ? 0xFFFFFFFFU : 0;
    ret.vec.u[2] = NLIB_ISNAN(value.vec, 2) ? 0xFFFFFFFFU : 0;
    ret.vec.u[3] = NLIB_ISNAN(value.vec, 3) ? 0xFFFFFFFFU : 0;
    return ret;
#elif defined(CAFE)
    // on CAFE, value is NaN if value < 0 && -value < 0
    f32x2 one = __PS_FDUP(1.f);
    f32x2 minus_one = __PS_NEG(one);
    f32x2 v0 = value.vec.ps[0];
    f32x2 v1 = value.vec.ps[1];
    f32x2 t0 = __PS_SEL(v0, one, minus_one);
    f32x2 t1 = __PS_SEL(v1, one, minus_one);
    f128 ret;
    f32x2 v0neg = __PS_NEG(v0);
    f32x2 v1neg = __PS_NEG(v1);
    ret.vec.ps[0] = __PS_SEL(v0neg, one, t0);
    ret.vec.ps[1] = __PS_SEL(v1neg, one, t0);
    return ret;
#else
    return F128::CmpNe(value, value);
#endif
}

// r[i] = isinf(value[i]) ? 0xFFFFFFFF : 0
NLIB_M(f128) F128::IsInfinite(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE)
    f128 ret;
    ret.vec.u[0] = NLIB_ISINF(value.vec, 0) ? 0xFFFFFFFFU : 0;
    ret.vec.u[1] = NLIB_ISINF(value.vec, 1) ? 0xFFFFFFFFU : 0;
    ret.vec.u[2] = NLIB_ISINF(value.vec, 2) ? 0xFFFFFFFFU : 0;
    ret.vec.u[3] = NLIB_ISINF(value.vec, 3) ? 0xFFFFFFFFU : 0;
    return ret;
#elif defined(CAFE)
    f128 ret;
    f32x2 big_value = __PS_FDUP(FLT_MAX);
    ret.vec.ps[0] = __PS_SUB(big_value, __PS_ABS(value.vec.ps[0]));
    ret.vec.ps[1] = __PS_SUB(big_value, __PS_ABS(value.vec.ps[1]));
    return ret;
#else
    f128 inf_value = F128::SetInfinity();
    f128 abs_value = F128::Abs(value);
    return F128::CmpEq(inf_value, abs_value);
#endif
}

// for example, 6.54321 -> 7.0, -6.54321 -> -7.0
NLIB_M(f128) F128::Round(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41) && !defined(NLIB_F128_SIMD_NOUSE)
    return _mm_round_ps(value, _MM_FROUND_TO_NEAREST_INT | _MM_FROUND_NO_EXC);
#elif defined(NLIB_NEON) && __ARM_ARCH >= 8 && !defined(NLIB_F128_SIMD_NOUSE)
    return vrndaq_f32(value);
#else
    // Add and Sub a big value to round the number after the decimal point
    f128 sgn = F128::And(value, F128::SetSignMask());
    f128 sm = F128::Or(F128::SetValue(0x4B000000U, each_uint32), sgn);
    f128 result = F128::Sub(F128::Add(value, sm), sm);
    return result;
#endif
}

// for example, 6.54321 -> 6.0, -6.54321 -> -6.0
NLIB_M2(f128) F128::Truncate(f128arg value) NLIB_NOEXCEPT {
// Note that there is no fraction if |value| > 2^23
// 2^23 = 8388608
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    for (size_t i = 0; i < 4; ++i) {
        if (NLIB_ISNAN(value.vec, i)) {
            ret.vec.u[i] = 0x7FC00000U;
        } else {
            ret.vec.v[i] = (fabsf(value.vec.v[i]) < 8388608.f)
                               ? static_cast<float>(static_cast<int>(value.vec.v[i]))
                               : value.vec.v[i];
        }
    }
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_round_ps(value, _MM_FROUND_TO_ZERO | _MM_FROUND_NO_EXC);
#elif defined(NLIB_NEON)
# if __ARM_ARCH < 8
    f128 x = F128::Abs(value);
    f128 c_2_23 = F128::SetValue(8388608.f, each_float);
    f128 cond = F128::CmpLt(x, c_2_23);
    f128 casted = F128::ConvertFromI128(F128::ConvertToI128Truncate(value));
    return F128::Select(cond, casted, value);
# else
    return vrndq_f32(value);
# endif
#endif
}

// for example, 6.54321 -> 6.0, -6.54321 -> -7.0
NLIB_M2(f128) F128::Floor(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.v[0] = floorf(value.vec.v[0]);
    ret.vec.v[1] = floorf(value.vec.v[1]);
    ret.vec.v[2] = floorf(value.vec.v[2]);
    ret.vec.v[3] = floorf(value.vec.v[3]);
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_floor_ps(value);
#elif defined(NLIB_NEON)
# if __ARM_ARCH < 8
    // Note that there is no fraction if |value| > 2^23
    // 2^23 = 8388608
    f128 x = F128::Abs(value);
    f128 c_2_23 = F128::SetValue(8388608.f, each_float);
    f128 cond = F128::CmpLt(x, c_2_23);
    f128 casted = F128::ConvertFromI128(F128::ConvertToI128Truncate(value));

    // -1 if result is larger
    f128 large_mask = F128::CmpGt(casted, value);
    // 0xFFFFFFFF -> -1 -> -1.f, 0 -> 0 -> 0.f
    casted = F128::Add(casted, F128::ConvertFromI128(F128::CastToI128(large_mask)));
    return F128::Select(cond, casted, value);
# else
    return vrndmq_f32(value);
# endif
#endif
}

// for example, 6.54321 -> 7.0, -6.54321 -> -6.0
NLIB_M2(f128) F128::Ceil(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.v[0] = ceilf(value.vec.v[0]);
    ret.vec.v[1] = ceilf(value.vec.v[1]);
    ret.vec.v[2] = ceilf(value.vec.v[2]);
    ret.vec.v[3] = ceilf(value.vec.v[3]);
    return ret;
#elif defined(NLIB_SSE41)
    return _mm_ceil_ps(value);
#elif defined(NLIB_NEON)
# if __ARM_ARCH < 8
    // Note that there is no fraction if |value| > 2^23
    // 2^23 = 8388608
    f128 x = F128::Abs(value);
    f128 c_2_23 = F128::SetValue(8388608.f, each_float);
    f128 cond = F128::CmpLt(x, c_2_23);
    f128 casted = F128::ConvertFromI128(F128::ConvertToI128Truncate(value));

    // +1 if result is smaller
    f128 small_mask = F128::CmpLt(casted, value);
    // 0xFFFFFFFF -> -1 -> -1.f, 0 -> 0 -> 0.f
    casted = F128::Sub(casted, F128::ConvertFromI128(F128::CastToI128(small_mask)));
    return F128::Select(cond, casted, value);
# else
    return vrndpq_f32(value);
# endif
#endif
}

#ifdef NLIB_F128_SIMD_NOUSE
#undef NLIB_ISNAN
#undef NLIB_ISINF
#endif

// r[i] = clamp(value[i], { 0, 0, 0, 0 }, { 1, 1, 1, 1 })
NLIB_M(f128) F128::Saturate(f128arg value) NLIB_NOEXCEPT {
    return F128::Clamp(value, F128::SetZero(), F128::SetOne());
}

NLIB_M2(f128) F128::ModAngle(f128arg value) NLIB_NOEXCEPT {
    static const float v_1_2pi = 0.15915494309189535f;
    static const float v_2pi = 6.283185307179586f;
    // value - 2pi * round(value * (1/2pi)) to be [-pi, pi)
    const f128 recp_two_pi = F128::SetValue(v_1_2pi, each_float);
    f128 round = F128::Round(F128::Mult(value, recp_two_pi));
    const f128 two_pi = F128::SetValue(v_2pi, each_float);
    return F128::MultSub(two_pi, round, value);
}

NLIB_M2(f128) F128::Sin(f128arg value) NLIB_NOEXCEPT {
    // within [-pi, pi)
    f128 x = F128::ModAngle(value);

    // within [-pi/2, pi/2]
    // use sin(x) == sin(pi - x), sin(x) == sin(-pi - x)
    // |x| <= pi/2  -> x = x
    // x > pi/2   -> x = pi - x
    // x < -pi/2  -> x = -pi - x
    f128 sin_cvalue = F128::LoadA16(F128::sin_cvalue_);
    f128 pi = F128::SetValue<0>(sin_cvalue, each_select32);
    f128 pidiv2 = F128::SetValue<1>(sin_cvalue, each_select32);

    f128 xabs = F128::Abs(value);
    f128 xsign = F128::And(F128::SetSignMask(), x);
    f128 mypi = F128::Or(xsign, pi);
    f128 pi_x = F128::Sub(mypi, x);
    f128 cond = F128::CmpLe(xabs, pidiv2);
    x = F128::Select(cond, x, pi_x);

    f128 xx = F128::Mult(x, x);
    f128 coeff = F128::LoadA16(sin_coeff_);
    f128 result;
    result = F128::MultSub<0>(coeff, xx, F128::SetValue<1>(coeff, each_select32), each_select32);

    result = F128::MultSub(xx, result, F128::SetValue<2>(coeff, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<3>(coeff, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<2>(sin_cvalue, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<3>(sin_cvalue, each_select32));
    result = F128::Mult(xx, result);
    result = F128::MultSub(result, x, x);
    return result;
}

NLIB_M2(f128) F128::Cos(f128arg value) NLIB_NOEXCEPT {
    // within [-pi, pi)
    f128 x = F128::ModAngle(value);

    // within [-pi/2, pi/2]
    // use cos(x) = -cos(pi - x), cos(x) = -cos(-pi - x)
    // |x| <= pi/2  -> x = x
    // x > pi/2   -> x = pi - x
    // x < -pi/2  -> x = -pi - x
    f128 cvalue = F128::LoadA16(cos_cvalue_);

    f128 xabs = F128::Abs(value);
    f128 xsign = F128::And(F128::SetSignMask(), x);
    f128 mypi = F128::Or(xsign, F128::SetValue<0>(cvalue, each_select32));  // pi
    f128 pi_x = F128::Sub(mypi, x);
    f128 cond = F128::CmpLe(xabs, F128::SetValue<1>(cvalue, each_select32));  // pi/2
    x = F128::Select(cond, x, pi_x);

    // +1 if [-pi/2, pi/2], -1 otherwise
    f128 sign = F128::AndNot(cond, F128::SetSignMask());

    // xx = x^2
    // 1 - xx * (1/2 - xx * (1/24 - xx * (1/720 - xx * (1/40320 - xx/3628800))))
    f128 xx = F128::Mult(x, x);
    f128 coeff = F128::LoadA16(cos_coeff_);
    f128 result;
    result = F128::MultSub<0>(coeff, xx, F128::SetValue<1>(coeff, each_select32), each_select32);

    result = F128::MultSub(xx, result, F128::SetValue<2>(coeff, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<3>(coeff, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<2>(cvalue, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<3>(cvalue, each_select32));
    result = F128::MultSub(xx, result, F128::SetOne());
    result = F128::Xor(sign, result);
    return result;
}

NLIB_M2(f128x2) F128::SinCos(f128arg value) NLIB_NOEXCEPT {
    // within [-pi, pi)
    const f128 signmask = F128::SetSignMask();
    f128 x = F128::ModAngle(value);

    // within [-pi/2, pi/2]
    // use cos(x) = -cos(pi - x), cos(x) = -cos(-pi - x)
    // |x| <= pi/2  -> x = x
    // x > pi/2   -> x = pi - x
    // x < -pi/2  -> x = -pi - x
    f128 cvalue = F128::LoadA16(cos_cvalue_);

    f128 xabs = F128::Abs(value);
    f128 xsign = F128::And(signmask, x);
    f128 mypi = F128::Or(xsign, F128::SetValue<0>(cvalue, each_select32));  // pi
    f128 pi_x = F128::Sub(mypi, x);
    f128 cond = F128::CmpLe(xabs, F128::SetValue<1>(cvalue, each_select32));  // pi/2
    x = F128::Select(cond, x, pi_x);

    // +1 if [-pi/2, pi/2], -1 otherwise
    f128 sign = F128::AndNot(cond, signmask);

    // xx = x^2
    // 1 - xx * (1/2 - xx * (1/24 - xx * (1/720 - xx * (1/40320 - xx/3628800))))
    f128 xx = F128::Mult(x, x);
    f128x2 ret;

    // cos
    {
        f128 coeff = F128::LoadA16(cos_coeff_);
        f128 result;
        result =
            F128::MultSub<0>(coeff, xx, F128::SetValue<1>(coeff, each_select32), each_select32);

        result = F128::MultSub(xx, result, F128::SetValue<2>(coeff, each_select32));
        result = F128::MultSub(xx, result, F128::SetValue<3>(coeff, each_select32));
        result = F128::MultSub(xx, result, F128::SetValue<2>(cvalue, each_select32));
        result = F128::MultSub(xx, result, F128::SetValue<3>(cvalue, each_select32));
        result = F128::MultSub(xx, result, F128::SetOne());

        ret.val[1] = F128::Xor(sign, result);  // cos
    }

    // sin
    {
        f128 coeff = F128::LoadA16(sin_coeff_);
        f128 result;
        result =
            F128::MultSub<0>(coeff, xx, F128::SetValue<1>(coeff, each_select32), each_select32);

        result = F128::MultSub(xx, result, F128::SetValue<2>(coeff, each_select32));
        result = F128::MultSub(xx, result, F128::SetValue<3>(coeff, each_select32));
        result = F128::MultSub(xx, result, F128::SetValue(sin_cvalue_[2], each_float));
        result = F128::MultSub(xx, result, F128::SetValue(sin_cvalue_[3], each_float));
        result = F128::Mult(xx, result);
        ret.val[0] = F128::MultSub(result, x, x);  // sin
    }
    return ret;
}

NLIB_M2(f128) F128::ArcTan(f128arg value) NLIB_NOEXCEPT {
    // |value| <= 1 -> atan(value)
    // value >  1 -> pi/2 - atan(1/value)
    // value < -1 -> -pi/2 - atan(1/value)
    f128 cmp, value_sign;
    {
        f128 one = F128::SetOne();

        // value_sign:
        //   1 if value > 1,
        //  -1 if value < -1
        value_sign = F128::AndNot(F128::CmpGt(value, one), F128::SetSignMask());
        cmp = F128::CmpLe(F128::Abs(value), one);
    }
    f128 x = F128::Select(cmp, value, F128::Recp(value));

    // atan(x) = x - 1/3 * x^3 + ... + (-1)^n/(2n+1) * x^(2n+1)
    //  = x(1 - xx(1/3 - xx(1/5 - xx(1/7 - xx(1/9 - xx(1/11 - xx(1/13 - xx(1/15 - xx(1/17)...)
    // NOTE:
    // DO NOT USE TAYLOR SERIES
    // minmax approximation(the output of Remez algorithm)
    f128 coeff0 = F128::LoadA16(&atan_coeff_[0]);
    f128 coeff1 = F128::LoadA16(&atan_coeff_[4]);
    f128 xx = F128::Mult(x, x);
    f128 result;
    result = F128::MultSub<3>(coeff1, xx, F128::SetValue<2>(coeff1, each_select32), each_select32);
    result = F128::MultSub(xx, result, F128::SetValue<1>(coeff1, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<0>(coeff1, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<3>(coeff0, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<2>(coeff0, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<1>(coeff0, each_select32));
    result = F128::MultSub(xx, result, F128::SetValue<0>(coeff0, each_select32));

    result = F128::Mult(result, x);
    result = F128::MultSub(xx, result, x);

    f128 pi_2 = F128::SetValue(1.5707963267948966f, each_float);
    f128 result_another = F128::Sub(F128::Xor(value_sign, pi_2), result);
    result = F128::Select(cmp, result, result_another);
    return result;
}

NLIB_M2(f128) F128::ArcTan2(f128arg y, f128arg x) NLIB_NOEXCEPT {
    //  y / x -> value
    // 0 / - -> pi | sign(y)
    // 0 / + -> 0 | sign(y)
    //    y / Inf -> 0 | sign(y)
    //    y / -Inf -> pi | sign(y)
    // y!=0 / 0 -> pi/2 | sign(y)
    //    y!=0 / - -> atan(y/x) + pi | sign(y)
    // Inf / x -> pi/2 | sign(y)
    // +-Inf / Inf  -> pi/4 | sign(y)
    // +-Inf / -Inf -> 3pi/4 | sign(y)
    //    otherwise -> atan(y/x)

    // sx = sign(x), sy = sign(y)
    // infx = isinf(x), infy = isinf(y)
    // zerox = iszero(x), zeroy = iszero(y)
    // posx = x > 0
    const f128 signmask = F128::SetSignMask();
    // const f128 sx = F128::And(x, signmask);
    const f128 sy = F128::And(y, signmask);
    const f128 infx = F128::IsInfinite(x);
    const f128 infy = F128::IsInfinite(y);
    const f128 zerox = F128::CmpEqZero(x);
    const f128 zeroy = F128::CmpEqZero(y);
    const f128 posx = F128::CmpGtZero(x);

    // v =
    // infy ?
    //    infx ?
    //       posx ? (pi/4 | sy) : (3pi/4 | sy)
    //    :  (pi/2 | sy)
    // :  zeroy ?
    //       posx ? (0 | sy) : (pi | sy)
    //    :  zerox ? (pi/2 | sy) : TrueMask;
    const f128 cval = F128::LoadA16(atan2_cvalue_);
    const f128 pi = F128::Or(sy, F128::SetValue<0>(cval, each_select32));
    const f128 pi_34 = F128::Or(sy, F128::SetValue<1>(cval, each_select32));
    const f128 pi_2 = F128::Or(sy, F128::SetValue<2>(cval, each_select32));
    const f128 pi_4 = F128::Or(sy, F128::SetValue<3>(cval, each_select32));

    f128 v = F128::Select(
        infy, F128::Select(infx, F128::Select(posx, pi_4, pi_34), pi_2),
        F128::Select(zeroy, F128::AndNot(posx, pi), F128::OrNot(zerox, pi_2)));

// mask = EqInt(v, full);
// result = Atan(y/x) + (posx ? (0 | sy) : (pi | sy))
// return mask ? result : v;
#if defined(NLIB_F128_SIMD_NOUSE)
    f128 mask;
    mask.vec.u[0] = v.vec.u[0] == 0xFFFFFFFFU ? v.vec.u[0] : 0;
    mask.vec.u[1] = v.vec.u[1] == 0xFFFFFFFFU ? v.vec.u[1] : 0;
    mask.vec.u[2] = v.vec.u[2] == 0xFFFFFFFFU ? v.vec.u[2] : 0;
    mask.vec.u[3] = v.vec.u[3] == 0xFFFFFFFFU ? v.vec.u[3] : 0;
#elif defined(CAFE)
    // select makes 0xFFFFFFFFUL -> 0xFF7FFFFFUL
    f128 mask;
    mask.vec.ps[0][0] = v.vec.u[0] == 0xFF7FFFFFUL ? -1.f : 1.f;
    mask.vec.ps[0][1] = v.vec.u[1] == 0xFF7FFFFFUL ? -1.f : 1.f;
    mask.vec.ps[1][0] = v.vec.u[2] == 0xFF7FFFFFUL ? -1.f : 1.f;
    mask.vec.ps[1][1] = v.vec.u[3] == 0xFF7FFFFFUL ? -1.f : 1.f;
#else
    f128 mask = F128::CastFromI128(I128::CmpEq32(F128::CastToI128(v),
                                                 I128::SetValue(-1, each_int8)));
#endif
    f128 result = F128::Add(F128::ArcTan(F128::Div(y, x)), F128::AndNot(posx, pi));
    return F128::Select(mask, result, v);
}

NLIB_M2(f128) F128::ArcSin(f128arg value) NLIB_NOEXCEPT {
    // asin(x) = atan2 (x, sqrt ((1.0 + x) * (1.0 - x)))
    f128 one = F128::SetOne();
    f128 tmp = F128::MultSub(value, value, one);
    f128 argx = F128::Sqrt(F128::AndNot(F128::CmpLtZero(tmp), tmp));
    return F128::ArcTan2(value, argx);
}

NLIB_M2(f128) F128::ArcCos(f128arg value) NLIB_NOEXCEPT {
    // acos(x) = atan2 (sqrt ((1.0 + x) * (1.0 - x)), x)
    f128 one = F128::SetOne();
    f128 tmp = F128::MultSub(value, value, one);
    f128 argx = F128::Sqrt(F128::AndNot(F128::CmpLtZero(tmp), tmp));
    return F128::ArcTan2(argx, value);
}

// see _mm_movemask_ps of SSE
NLIB_M2(int) F128::MoveMask(f128arg value) NLIB_NOEXCEPT {  // NOLINT
#ifdef NLIB_F128_SIMD_NOUSE
    uint8_t ret = 0;
    ret |= value.vec.u[0] == 0xFFFFFFFFU ? 1 : 0;
    ret |= value.vec.u[1] == 0xFFFFFFFFU ? 2 : 0;
    ret |= value.vec.u[2] == 0xFFFFFFFFU ? 4 : 0;
    ret |= value.vec.u[3] == 0xFFFFFFFFU ? 8 : 0;
    return ret;
#elif defined(NLIB_SSE41)
    return static_cast<uint8_t>(_mm_movemask_ps(value));
#elif defined(NLIB_NEON)
    uint32x2_t powers_lo = vcreate_u32(0x0000000200000001ULL);
    uint32x2_t powers_hi = vshl_n_u32(powers_lo, 2);
    uint32x4_t powers = vcombine_u32(powers_lo, powers_hi);
    uint32x4_t a = vandq_u32(vreinterpretq_u32_f32(value), powers);
# ifdef __aarch64__
    return vaddvq_u32(a);
# else
    uint16x4_t tmp = vmovn_u32(a);
    tmp = vpadd_u16(tmp, tmp);
    tmp = vpadd_u16(tmp, tmp);
    return vget_lane_u8(vreinterpret_u8_u16(tmp), 0);
# endif
#elif defined(CAFE)
    int tmp = (value.vec.u[0] >> 31);
    tmp |= (value.vec.u[1] >> 30) & 2;
    tmp |= (value.vec.u[2] >> 29) & 4;
    tmp |= (value.vec.u[3] >> 28) & 8;
    return tmp;
#endif
}

// true if value[i] == 0 for all i
NLIB_M2(bool) F128::IsAllMaskFalse(f128arg value) NLIB_NOEXCEPT {  // NOLINT
#ifdef NLIB_F128_SIMD_NOUSE
    return value.vec.u[0] == 0 && value.vec.u[1] == 0 && value.vec.u[2] == 0 && value.vec.u[3] == 0;
#elif defined(NLIB_SSE41)
    i128 casted = F128::CastToI128(value);
    return _mm_testz_si128(casted, casted) != 0;
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    uint32x4_t mask = vceqzq_u32(vreinterpretq_u32_f32(value));
    return vaddvq_s32(vreinterpretq_s32_u32(mask)) == -4;
# else
    int32x4_t casted = vreinterpretq_s32_f32(value);
    int32x2_t tmp = vorr_s32(vget_low_s32(casted), vget_high_s32(casted));
    return vget_lane_u64(vreinterpret_u64_s32(tmp), 0) == 0;
# endif
#elif defined(CAFE)
    uint32_t tmp = value.vec.u[0] | value.vec.u[1] | value.vec.u[2] | value.vec.u[3];
    return (tmp & 0x80000000U) == 0;
#endif
}

// true if value[i] == 0xFFFFFFFF for all i
NLIB_M2(bool) F128::IsAllMaskTrue(f128arg value) NLIB_NOEXCEPT {  // NOLINT
#ifdef NLIB_F128_SIMD_NOUSE
    return value.vec.u[0] == 0xFFFFFFFFU && value.vec.u[1] == 0xFFFFFFFFU &&
           value.vec.u[2] == 0xFFFFFFFFU && value.vec.u[3] == 0xFFFFFFFFU;
#elif defined(NLIB_SSE41)
    i128 casted = F128::CastToI128(value);
    return _mm_testc_si128(casted, _mm_cmpeq_epi8(casted, casted)) != 0;
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    uint32x4_t mask = vceqzq_u32(vmvnq_u32(vreinterpretq_u32_f32(value)));
    return vaddvq_s32(vreinterpretq_s32_u32(mask)) == -4;
# else
    int32x4_t casted = vreinterpretq_s32_f32(value);
    int32x2_t tmp = vand_s32(vget_low_s32(casted), vget_high_s32(casted));
    return vget_lane_s64(vreinterpret_s64_s32(tmp), 0) == -1;
# endif
#elif defined(CAFE)
    uint32_t tmp = value.vec.u[0] & value.vec.u[1] & value.vec.u[2] & value.vec.u[3];
    return (tmp & 0x80000000U) != 0;
#endif
}

template <size_t N>
// r = value[N]
NLIB_M(float) F128::GetFloatFromLane(f128arg value) NLIB_NOEXCEPT {  // NOLINT
    NLIB_STATIC_ASSERT(N < 4);
#ifdef NLIB_F128_SIMD_NOUSE
    return value.vec.v[N];
#elif defined(NLIB_SSE41)
    float dest;
    _MM_EXTRACT_FLOAT(dest, value, N);
    return dest;
#elif defined(NLIB_NEON)
    return vgetq_lane_f32(value, N);
#elif defined(CAFE)
    return value.vec.ps[N / 2][N % 2];
#endif
}

template <size_t N>
// r = *reinterpret_cast<uint32_t*>(&value[N])
NLIB_M(uint32_t) F128::GetUint32FromLane(f128arg value) NLIB_NOEXCEPT {  // NOLINT
    NLIB_STATIC_ASSERT(N < 4);
#ifdef NLIB_F128_SIMD_NOUSE
    return value.vec.u[N];
#elif defined(NLIB_SSE41)
    return _mm_extract_ps(value, N);
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vreinterpretq_u32_f32(value);
    return vgetq_lane_u32(tmp, N);
#elif defined(CAFE)
    return value.vec.u[N];
#endif
}

// r = value[idx]
NLIB_M2(float) F128::GetFloatByIndex(f128arg value, size_t idx) NLIB_NOEXCEPT {  // NOLINT
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    return value.vec.v[idx];
#elif defined(NLIB_SSE41)
    float dest;
    switch (idx) {
    case 0:
        _MM_EXTRACT_FLOAT(dest, value, 0);
        break;
    case 1:
        _MM_EXTRACT_FLOAT(dest, value, 1);
        break;
    case 2:
        _MM_EXTRACT_FLOAT(dest, value, 2);
        break;
    case 3:
        _MM_EXTRACT_FLOAT(dest, value, 3);
        break;
    default:
        NLIB_ASSUME(0);
        break;
    }
    return dest;
#elif defined(NLIB_NEON)
    switch (idx) {
    case 0:
        return vgetq_lane_f32(value, 0);
    case 1:
        return vgetq_lane_f32(value, 1);
    case 2:
        return vgetq_lane_f32(value, 2);
    case 3:
        return vgetq_lane_f32(value, 3);
    default:
        NLIB_ASSUME(0);
        break;
    }
#endif
}

// r = *reinterpret_cast<uint32_t*>(&value[idx])
NLIB_M2(uint32_t) F128::GetUint32ByIndex(f128arg value, size_t idx) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    return value.vec.u[idx];
#elif defined(NLIB_SSE41)
    switch (idx) {
    case 0:
        return static_cast<uint32_t>(_mm_extract_ps(value, 0));
    case 1:
        return static_cast<uint32_t>(_mm_extract_ps(value, 1));
    case 2:
        return static_cast<uint32_t>(_mm_extract_ps(value, 2));
    case 3:
        return static_cast<uint32_t>(_mm_extract_ps(value, 3));
    default:
        NLIB_ASSUME(0);
        break;
    }
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vreinterpretq_u32_f32(value);
    switch (idx) {
    case 0:
        return vgetq_lane_u32(tmp, 0);
    case 1:
        return vgetq_lane_u32(tmp, 1);
    case 2:
        return vgetq_lane_u32(tmp, 2);
    case 3:
        return vgetq_lane_u32(tmp, 3);
    default:
        NLIB_ASSUME(0);
        break;
    }
#endif
}

template <size_t N>
// r = value, r[N] = v
NLIB_M(f128) F128::SetFloatToLane(f128arg value, float v) NLIB_NOEXCEPT {  // NOLINT
    NLIB_STATIC_ASSERT(N < 4);
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret = value;
    ret.vec.v[N] = v;
    return ret;
#elif defined(NLIB_SSE41)
    f128 tmp = _mm_set_ss(v);
    return _mm_insert_ps(value, tmp, N << 4);
#elif defined(NLIB_NEON)
    return __builtin_constant_p(v) ?
        F128::Permute<N == 0 ? 4 : 0,
                      N == 1 ? 5 : 1,
                      N == 2 ? 6 : 2,
                      N == 3 ? 7 : 3>(value, vdupq_n_f32(v)) :
        vsetq_lane_f32(v, value, N);
#elif defined(CAFE)
    f128 ret = value;
    ret.vec.ps[N / 2][N % 2] = v;
    return ret;
#endif
}

// r = value, r[i] = v
NLIB_M2(f128) F128::SetFloatByIndex(f128arg value, float v, size_t i) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret = value;
    ret.vec.v[i] = v;
    return ret;
#elif defined(NLIB_SSE41)
    f128 tmp = _mm_set_ss(v);
    switch (i) {
    case 0:
        return _mm_insert_ps(value, tmp, 0x00);
    case 1:
        return _mm_insert_ps(value, tmp, 0x10);
    case 2:
        return _mm_insert_ps(value, tmp, 0x20);
    case 3:
        return _mm_insert_ps(value, tmp, 0x30);
    default:
        NLIB_ASSUME(0);
        break;
    }
#elif defined(NLIB_NEON)
    switch (i) {
    case 0:
        return F128::SetFloatToLane<0>(value, v);
    case 1:
        return F128::SetFloatToLane<1>(value, v);
    case 2:
        return F128::SetFloatToLane<2>(value, v);
    case 3:
        return F128::SetFloatToLane<3>(value, v);
    default:
        NLIB_ASSUME(0);
        break;
    }
#elif defined(CAFE)
    f128 ret = value;
    switch (i) {
    case 0:
        ret.vec.ps[0][0] = v;
        break;
    case 1:
        ret.vec.ps[0][1] = v;
        break;
    case 2:
        ret.vec.ps[1][0] = v;
        break;
    default:
        ret.vec.ps[1][1] = v;
        break;
    }
    return ret;
#endif
}

#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
namespace detail {

template <bool IsHighA, bool IsHighB>
float32x2_t F64Merge(float32x2_t a, float32x2_t b) NLIB_NOEXCEPT;

template <>
NLIB_ALWAYS_INLINE float32x2_t F64Merge<false, false>(float32x2_t a, float32x2_t b) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vtrn1_f32(a, b);
#else
    return vtrn_f32(a, b).val[0];
#endif
};

template <>
NLIB_ALWAYS_INLINE float32x2_t F64Merge<true, false>(float32x2_t a, float32x2_t b) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vtrn1_f32(vrev64_f32(a), b);
#else
    return vtrn_f32(vrev64_f32(a), b).val[0];
#endif
};

template <>
NLIB_ALWAYS_INLINE float32x2_t F64Merge<false, true>(float32x2_t a, float32x2_t b) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vtrn1_f32(a, vrev64_f32(b));
#else
    return vtrn_f32(a, vrev64_f32(b)).val[0];
#endif
};

template <>
NLIB_ALWAYS_INLINE float32x2_t F64Merge<true, true>(float32x2_t a, float32x2_t b) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vtrn2_f32(a, b);
#else
    return vtrn_f32(a, b).val[1];
#endif
};

template <int Z>
float32x2_t F128SwizzleGet64(f128arg value) NLIB_NOEXCEPT;

template <>
NLIB_ALWAYS_INLINE float32x2_t F128SwizzleGet64<0>(f128arg value) NLIB_NOEXCEPT {
    return vget_low_f32(value);
}

template <>
NLIB_ALWAYS_INLINE float32x2_t F128SwizzleGet64<1>(f128arg value) NLIB_NOEXCEPT {
    return vget_high_f32(value);
}

template <int X0, int X1>
struct F128SwizzleHelper2 {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        float32x2_t x0 = F128SwizzleGet64<X0 / 2>(value);
        float32x2_t x1 = F128SwizzleGet64<X1 / 2>(value);
        return F64Merge<(X0 & 1), (X1 & 1)>(x0, x1);
    }
};

template <int X>
struct F128SwizzleHelper2<X, X> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        float32x2_t x = F128SwizzleGet64<X / 2>(value);
        return vdup_lane_f32(x, (X & 1));
    }
};

template <>
struct F128SwizzleHelper2<0, 1> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        return vget_low_f32(value);
    }
};

template <>
struct F128SwizzleHelper2<0, 2> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
        return vget_low_f32(vuzp1q_f32(value, value));
#else
        float32x2_t lo = vget_low_f32(value);
        float32x2_t hi = vget_high_f32(value);
        return vzip_f32(lo, hi).val[0];
#endif
    }
};

template <>
struct F128SwizzleHelper2<0, 3> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        float32x2_t lo = vget_low_f32(value);
        float32x2_t hi = vrev64_f32(vget_high_f32(value));
#ifdef __aarch64__
        return vzip1_f32(lo, hi);
#else
        return vzip_f32(lo, hi).val[0];
#endif
    }
};

template <>
struct F128SwizzleHelper2<1, 0> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        return vrev64_f32(vget_low_f32(value));
    }
};

template <>
struct F128SwizzleHelper2<1, 2> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        float32x2_t lo = vget_low_f32(value);
        float32x2_t hi = vrev64_f32(vget_high_f32(value));
#ifdef __aarch64__
        return vzip2_f32(lo, hi);
#else
        return vzip_f32(lo, hi).val[1];
#endif
    }
};

template <>
struct F128SwizzleHelper2<1, 3> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
        return vget_low_f32(vuzp2q_f32(value, value));
#else
        float32x2_t lo = vget_low_f32(value);
        float32x2_t hi = vget_high_f32(value);
        return vzip_f32(lo, hi).val[1];
#endif
    }
};

template <>
struct F128SwizzleHelper2<2, 0> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
        return vget_high_f32(vcopyq_laneq_f32(value, 3, value, 0));
#else
        float32x2_t lo = vget_low_f32(value);
        float32x2_t hi = vget_high_f32(value);
        return vzip_f32(hi, lo).val[0];
#endif
    }
};

template <>
struct F128SwizzleHelper2<2, 1> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
        return vget_high_f32(vcopyq_laneq_f32(value, 3, value, 1));
#else
        float32x2_t lo = vget_low_f32(value);
        float32x2_t hi = vrev64_f32(vget_high_f32(value));
        return vzip_f32(hi, lo).val[1];
#endif
    }
};

template <>
struct F128SwizzleHelper2<2, 3> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        return vget_high_f32(value);
    }
};

template <>
struct F128SwizzleHelper2<3, 0> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        float32x2_t lo = vget_low_f32(value);
        float32x2_t hi = vrev64_f32(vget_high_f32(value));
#ifdef __aarch64__
        return vzip1_f32(hi, lo);
#else
        return vzip_f32(hi, lo).val[0];
#endif
    }
};

template <>
struct F128SwizzleHelper2<3, 1> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        float32x2_t lo = vget_low_f32(value);
        float32x2_t hi = vget_high_f32(value);
#ifdef __aarch64__
        return vzip2_f32(hi, lo);
#else
        return vzip_f32(hi, lo).val[1];
#endif
    }
};

template <>
struct F128SwizzleHelper2<3, 2> {
    static NLIB_ALWAYS_INLINE float32x2_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        return vrev64_f32(vget_high_f32(value));
    }
};

template <int V0, int V1, int V2, int V3>
struct F128SwizzleHelper {
    static NLIB_ALWAYS_INLINE float32x4_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        return vcombine_f32(detail::F128SwizzleHelper2<V0, V1>::Swizzle(value),
                            detail::F128SwizzleHelper2<V2, V3>::Swizzle(value));
    }
};

template <int Vx, int Vy>
struct F128SwizzleHelper<Vx, Vy, Vx, Vy> {
    static NLIB_ALWAYS_INLINE float32x4_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        float32x2_t tmp = detail::F128SwizzleHelper2<Vx, Vy>::Swizzle(value);
        return vcombine_f32(tmp, tmp);
    }
};

template <int V>
struct F128SwizzleHelper<V, V, V, V> {
    static NLIB_ALWAYS_INLINE float32x4_t Swizzle(f128arg value) NLIB_NOEXCEPT {
        return F128::SetValue<V>(value, each_select32);
    }
};

}  // namespace detail
#elif defined(CAFE) && !defined(NLIB_F128_SIMD_NOUSE)
namespace detail {

template <int X0, int X1>
struct F128SwizzleHelper {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT;
};

template<>
struct F128SwizzleHelper<0, 0> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        (void)v1;
        return __PS_MERGE00(v0, v0);
    }
};

template<>
struct F128SwizzleHelper<0, 1> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        (void)v1;
        return v0;
    }
};

template<>
struct F128SwizzleHelper<0, 2> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        return __PS_MERGE00(v0, v1);
    }
};

template<>
struct F128SwizzleHelper<0, 3> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        return __PS_MERGE01(v0, v1);
    }
};

template<>
struct F128SwizzleHelper<1, 0> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        (void)v1;
        return __PS_MERGE10(v0, v0);
    }
};

template<>
struct F128SwizzleHelper<1, 1> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        (void)v1;
        return __PS_MERGE11(v0, v0);
    }
};

template<>
struct F128SwizzleHelper<1, 2> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        return __PS_MERGE10(v0, v1);
    }
};

template<>
struct F128SwizzleHelper<1, 3> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        return __PS_MERGE11(v0, v1);
    }
};

template<>
struct F128SwizzleHelper<2, 0> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        return __PS_MERGE00(v1, v0);
    }
};

template<>
struct F128SwizzleHelper<2, 1> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        return __PS_MERGE01(v1, v0);
    }
};

template<>
struct F128SwizzleHelper<2, 2> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        (void)v0;
        return __PS_MERGE00(v1, v1);
    }
};

template<>
struct F128SwizzleHelper<2, 3> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        (void)v0;
        return v1;
    }
};

template<>
struct F128SwizzleHelper<3, 0> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        return __PS_MERGE10(v1, v0);
    }
};

template<>
struct F128SwizzleHelper<3, 1> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        return __PS_MERGE11(v1, v0);
    }
};

template<>
struct F128SwizzleHelper<3, 2> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        (void)v0;
        return __PS_MERGE10(v1, v1);
    }
};

template<>
struct F128SwizzleHelper<3, 3> {
    static NLIB_ALWAYS_INLINE f32x2 Swizzle(f32x2 v0, f32x2 v1) NLIB_NOEXCEPT {
        (void)v0;
        return __PS_MERGE11(v1, v1);
    }
};

}  // namespace detail
#endif

template <int V0, int V1, int V2, int V3>
// r[0] = value[V0], r[1] = value[V1], r[2] = value[V2], r[3] = value[V3]
NLIB_M(f128) F128::Swizzle(f128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(V0 < 4);
    NLIB_STATIC_ASSERT(V1 < 4);
    NLIB_STATIC_ASSERT(V2 < 4);
    NLIB_STATIC_ASSERT(V3 < 4);
#if defined(NLIB_F128_SIMD_NOUSE)
    f128 ret;
    ret.vec.v[0] = value.vec.v[V0 != -1 ? V0 : 0];
    ret.vec.v[1] = value.vec.v[V1 != -1 ? V1 : 1];
    ret.vec.v[2] = value.vec.v[V2 != -1 ? V2 : 2];
    ret.vec.v[3] = value.vec.v[V3 != -1 ? V3 : 3];
    return ret;
#elif __has_builtin(__builtin_shufflevector)
    return __builtin_shufflevector(value, value, V0, V1, V2, V3);
#elif defined(NLIB_SSE41)
    return _mm_shuffle_ps(value, value,
                          _MM_SHUFFLE(V3 != -1 ? V3 : 3,
                                      V2 != -1 ? V2 : 2,
                                      V1 != -1 ? V1 : 1,
                                      V0 != -1 ? V0 : 0));
#elif defined(NLIB_NEON)
    return detail::F128SwizzleHelper<
        V0 != -1 ? V0 : 0,
        V1 != -1 ? V1 : 1,
        V2 != -1 ? V2 : 2,
        V3 != -1 ? V3 : 3>::Swizzle(value);
#elif defined(CAFE)
    f128 ret;
    ret.vec.ps[0] = detail::F128SwizzleHelper<
        (V0 != -1 ? V0 : 0), (V1 != -1 ? V1 : 1)>::Swizzle(value.vec.ps[0], value.vec.ps[1]);
    ret.vec.ps[1] = detail::F128SwizzleHelper<
        (V2 != -1 ? V2 : 2), (V3 != -1 ? V3 : 3)>::Swizzle(value.vec.ps[0], value.vec.ps[1]);
    return ret;
#endif
}

#if defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)
// Swizzle specialization for NEON
template <>
NLIB_M(f128) F128::Swizzle<0, 0, 1, 1>(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vzip1q_f32(value, value);
#else
    return vzipq_f32(value, value).val[0];
#endif
}
template <>
NLIB_M(f128) F128::Swizzle<0, 0, 2, 2>(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vtrn1q_f32(value, value);
#else
    return vtrnq_f32(value, value).val[0];
#endif
}
template <>
NLIB_M(f128) F128::Swizzle<0, 1, 2, 3>(f128arg value) NLIB_NOEXCEPT {
    return value;
}
template <>
NLIB_M(f128) F128::Swizzle<0, 2, 0, 2>(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vuzp1q_f32(value, value);
#else
    return vuzpq_f32(value, value).val[0];
#endif
}
template <>
NLIB_M(f128) F128::Swizzle<1, 0, 3, 2>(f128arg value) NLIB_NOEXCEPT {
    return vrev64q_f32(value);
}
template <>
NLIB_M(f128) F128::Swizzle<1, 1, 3, 3>(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vtrn2q_f32(value, value);
#else
    return vtrnq_f32(value, value).val[1];
#endif
}
template <>
NLIB_M(f128) F128::Swizzle<1, 2, 3, 0>(f128arg value) NLIB_NOEXCEPT {
    uint32x4_t ival = vreinterpretq_u32_f32(value);
    uint32x4_t rotated = vextq_u32(ival, ival, 1);
    return vreinterpretq_f32_u32(rotated);
}
template <>
NLIB_M(f128) F128::Swizzle<1, 3, 1, 3>(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vuzp2q_f32(value, value);
#else
    return vuzpq_f32(value, value).val[1];
#endif
}
template <>
NLIB_M(f128) F128::Swizzle<2, 2, 3, 3>(f128arg value) NLIB_NOEXCEPT {
#ifdef __aarch64__
    return vzip2q_f32(value, value);
#else
    return vzipq_f32(value, value).val[1];
#endif
}
template <>
NLIB_M(f128) F128::Swizzle<2, 3, 0, 1>(f128arg value) NLIB_NOEXCEPT {
    uint32x4_t ival = vreinterpretq_u32_f32(value);
    uint32x4_t rotated = vextq_u32(ival, ival, 2);
    return vreinterpretq_f32_u32(rotated);
}
template <>
NLIB_M(f128) F128::Swizzle<3, 0, 1, 2>(f128arg value) NLIB_NOEXCEPT {
    uint32x4_t ival = vreinterpretq_u32_f32(value);
    uint32x4_t rotated = vextq_u32(ival, ival, 3);
    return vreinterpretq_f32_u32(rotated);
}
#endif

namespace detail {

#if defined(NLIB_SSE41) && !defined(NLIB_F128_SIMD_NOUSE)
template <bool UseBlend, bool UseShuffle, int V0, int V1, int V2, int V3>
struct F128PermuteHelper2 {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        f128 as = F128::Swizzle<V0 & 3, V1 & 3, V2 & 3, V3 & 3>(a);
        f128 bs = F128::Swizzle<V0 & 3, V1 & 3, V2 & 3, V3 & 3>(b);
        return _mm_blend_ps(as, bs, (((V0 & 4) ? 1 : 0) | ((V1 & 4) ? 2 : 0) |
                                     ((V2 & 4) ? 4 : 0) | ((V3 & 4) ? 8 : 0)));
    }
};

template <bool UseShuffle, int V0, int V1, int V2, int V3>
struct F128PermuteHelper2<true, UseShuffle, V0, V1, V2, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        return _mm_blend_ps(a, b, (((V0 & 4) ? 1 : 0) | ((V1 & 4) ? 2 : 0) |
                                    ((V2 & 4) ? 4 : 0) | ((V3 & 4) ? 8 : 0)));
    }
};

template <int V0, int V1, int V2, int V3>
struct F128PermuteHelper2<false, true, V0, V1, V2, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        return _mm_shuffle_ps(V0 < 4 ? a : b, V0 < 4 ? b : a,
                              _MM_SHUFFLE((V3 & 3), (V2 & 3), (V1 & 3), (V0 & 3)));
    }
};

template <>
struct F128PermuteHelper2<false, false, 1, 2, 3, 4> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        __m128i tmp = _mm_alignr_epi8(_mm_castps_si128(b), _mm_castps_si128(a), 4);
        return _mm_castsi128_ps(tmp);
    }
};

template <>
struct F128PermuteHelper2<false, false, 3, 4, 5, 6> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        __m128i tmp = _mm_alignr_epi8(_mm_castps_si128(b), _mm_castps_si128(a), 12);
        return _mm_castsi128_ps(tmp);
    }
};

template <>
struct F128PermuteHelper2<false, false, 5, 6, 7, 0> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        __m128i tmp = _mm_alignr_epi8(_mm_castps_si128(b), _mm_castps_si128(a), 20);
        return _mm_castsi128_ps(tmp);
    }
};

template <int V>
struct F128PermuteHelper2<false, false, V, 1, 2, 3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V > 3);
        return _mm_insert_ps(a, b, ((V - 4) << 6) | (0 << 4));
    }
};

template <int V>
struct F128PermuteHelper2<false, false, 0, V, 2, 3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V > 3);
        return _mm_insert_ps(a, b, ((V - 4) << 6) | (1 << 4));
    }
};

template <int V>
struct F128PermuteHelper2<false, false, 0, 1, V, 3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V > 3);
        return _mm_insert_ps(a, b, ((V - 4) << 6) | (2 << 4));
    }
};

template <int V>
struct F128PermuteHelper2<false, false, 0, 1, 2, V> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V > 3);
        return _mm_insert_ps(a, b, ((V - 4) << 6) | (3 << 4));
    }
};

template <int V>
struct F128PermuteHelper2<false, false, V, 5, 6, 7> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V < 4);
        return _mm_insert_ps(b, a, (V << 6) | (0 << 4));
    }
};

template <int V>
struct F128PermuteHelper2<false, false, 4, V, 6, 7> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V < 4);
        return _mm_insert_ps(b, a, (V << 6) | (1 << 4));
    }
};

template <int V>
struct F128PermuteHelper2<false, false, 4, 5, V, 7> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V < 4);
        return _mm_insert_ps(b, a, (V << 6) | (2 << 4));
    }
};

template <int V>
struct F128PermuteHelper2<false, false, 4, 5, 6, V> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V < 4);
        return _mm_insert_ps(b, a, (V << 6) | (3 << 4));
    }
};

template <bool IsAllA, bool IsAllB, int V0, int V1, int V2, int V3>
struct F128PermuteHelper {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        return F128PermuteHelper2<
            ((V0 % 4 == 0) && (V1 % 4 == 1) && (V2 % 4 == 2) && (V3 % 4 == 3)),
            ((V0 < 4 && V1 < 4 && V2 >= 4 && V3 >= 4) || (V0 >= 4 && V1 >= 4 && V2 < 4 && V3 < 4)),
            V0, V1, V2, V3>::Permute(a, b);
    }
};

#elif defined(NLIB_NEON) && !defined(NLIB_F128_SIMD_NOUSE)

template <int Z>
float32x2_t F128PermuteGet64(f128arg a, f128arg b) NLIB_NOEXCEPT;

template <>
NLIB_ALWAYS_INLINE float32x2_t F128PermuteGet64<0>(f128arg a, f128arg b) NLIB_NOEXCEPT {
    NLIB_UNUSED(b);
    return vget_low_f32(a);
}
template <>
NLIB_ALWAYS_INLINE float32x2_t F128PermuteGet64<1>(f128arg a, f128arg b) NLIB_NOEXCEPT {
    NLIB_UNUSED(b);
    return vget_high_f32(a);
}
template <>
NLIB_ALWAYS_INLINE float32x2_t F128PermuteGet64<2>(f128arg a, f128arg b) NLIB_NOEXCEPT {
    NLIB_UNUSED(a);
    return vget_low_f32(b);
}
template <>
NLIB_ALWAYS_INLINE float32x2_t F128PermuteGet64<3>(f128arg a, f128arg b) NLIB_NOEXCEPT {
    NLIB_UNUSED(a);
    return vget_high_f32(b);
}

template <int X0, int X1>
struct F128PermuteHelper2 {
    static NLIB_ALWAYS_INLINE float32x2_t Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        float32x2_t x0 = F128PermuteGet64<X0 / 2>(a, b);
        float32x2_t x1 = F128PermuteGet64<X1 / 2>(a, b);
        return F64Merge<(X0 & 1), (X1 & 1)>(x0, x1);
    }
};

template <int X>
struct F128PermuteHelper2<X, X> {
    static NLIB_ALWAYS_INLINE float32x2_t Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        float32x2_t x = F128PermuteGet64<X / 2>(a, b);
        return vdup_lane_f32(x, (X & 1));
    }
};

template <bool IsAllA, bool IsAllB, int V0, int V1, int V2, int V3>
struct F128PermuteHelper {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        return vcombine_f32(F128PermuteHelper2<V0, V1>::Permute(a, b),
                            F128PermuteHelper2<V2, V3>::Permute(a, b));
    }
};

template <>
struct F128PermuteHelper<false, false, 1, 2, 3, 4> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        int32x4_t tmp = vextq_s32(vreinterpretq_s32_f32(a), vreinterpretq_s32_f32(b), 1);
        return vreinterpretq_f32_s32(tmp);
    }
};

template <>
struct F128PermuteHelper<false, false, 3, 4, 5, 6> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        int32x4_t tmp = vextq_s32(vreinterpretq_s32_f32(a), vreinterpretq_s32_f32(b), 3);
        return vreinterpretq_f32_s32(tmp);
    }
};

template <>
struct F128PermuteHelper<false, false, 5, 6, 7, 0> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        int32x4_t tmp = vextq_s32(vreinterpretq_s32_f32(b), vreinterpretq_s32_f32(a), 1);
        return vreinterpretq_f32_s32(tmp);
    }
};
#elif defined(CAFE) && !defined(NLIB_F128_SIMD_NOUSE)
template<int R0, int R1, int VAR0, int VAR1>
struct F128PermuteHelper2 {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT;
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 0, 0> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v0, v0);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 0, 1> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v0, v1);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 0, 2> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v0, v2);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 0, 3> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v0, v3);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 1, 0> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v1, v0);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 1, 1> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v1, v1);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 1, 2> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v1, v2);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 1, 3> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v1, v3);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 2, 0> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v2, v0);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 2, 1> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v2, v1);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 2, 2> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v2, v2);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 2, 3> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v2, v3);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 3, 0> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v3, v0);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 3, 1> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v3, v1);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 3, 2> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v3, v2);
    }
};

template<int R0, int R1>
struct F128PermuteHelper2<R0, R1, 3, 3> {
    static f32x2 Permute(f32x2 v0, f32x2 v1, f32x2 v2, f32x2 v3) NLIB_NOEXCEPT {
        return detail::F128SwizzleHelper<R0, (2 + R1)>::Swizzle(v3, v3);
    }
};

template <bool IsAllA, bool IsAllB, int V0, int V1, int V2, int V3>
struct F128PermuteHelper {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        f128 ret;
        f32x2 x0 = a.vec.ps[0];
        f32x2 x1 = a.vec.ps[1];
        f32x2 x2 = b.vec.ps[0];
        f32x2 x3 = b.vec.ps[1];
        ret.vec.ps[0] = F128PermuteHelper2<(V0 & 1), (V1 & 1), (V0 / 2), (V1 / 2)>
            ::Permute(x0, x1, x2, x3);
        ret.vec.ps[1] = F128PermuteHelper2<(V2 & 1), (V3 & 1), (V2 / 2), (V3 / 2)>
            ::Permute(x0, x1, x2, x3);
        return ret;
    }
};
#else
template <bool IsAllA, bool IsAllB, int V0, int V1, int V2, int V3>
struct F128PermuteHelper {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        f128 ret = F128::SetValue(F128::GetFloatFromLane<V0 & 3>(V0 < 4 ? a : b),
                                  F128::GetFloatFromLane<V1 & 3>(V1 < 4 ? a : b),
                                  F128::GetFloatFromLane<V2 & 3>(V2 < 4 ? a : b),
                                  F128::GetFloatFromLane<V3 & 3>(V3 < 4 ? a : b));
        return ret;
    }
};
#endif

template <int V0, int V1, int V2, int V3>
struct F128PermuteHelper<true, false, V0, V1, V2, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_UNUSED(b);
        return F128::Swizzle<V0, V1, V2, V3>(a);
    }
};

template <int V0, int V1, int V2, int V3>
struct F128PermuteHelper<false, true, V0, V1, V2, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_UNUSED(a);
        return F128::Swizzle<(V0 - 4), (V1 - 4), (V2 - 4), (V3 - 4)>(b);
    }
};

#if defined(NLIB_SSE41) && !defined(NLIB_F128_SIMD_NOUSE)
// Permute specialization for SSE4.1
template <>
struct F128PermuteHelper<false, false, 0, 4, 1, 5> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        return _mm_unpacklo_ps(a, b);
    }
};
template <>
struct F128PermuteHelper<false, false, 4, 0, 5, 1> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        return _mm_unpacklo_ps(b, a);
    }
};
template <>
struct F128PermuteHelper<false, false, 2, 6, 3, 7> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        return _mm_unpackhi_ps(a, b);
    }
};
template <>
struct F128PermuteHelper<false, false, 6, 2, 7, 3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        return _mm_unpackhi_ps(b, a);
    }
};
#endif

template<int V0, int V1, int V2, int V3>
struct F128PermuteDontCareHelper {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V0 < 8);
        NLIB_STATIC_ASSERT(V1 < 8);
        NLIB_STATIC_ASSERT(V2 < 8);
        NLIB_STATIC_ASSERT(V3 < 8);
        static const bool arg1 = (V0 < 4 && V1 < 4 && V2 < 4 && V3 < 4);
        static const bool arg2 = (V0 > 3 && V1 > 3 && V2 > 3 && V3 > 3);
        return detail::F128PermuteHelper< arg1, arg2,
            V0, V1, V2, V3 >::Permute(a, b);
    }
};

template<int V1, int V2, int V3>
struct F128PermuteDontCareHelper<8, V1, V2, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V1 < 8);
        NLIB_STATIC_ASSERT(V2 < 8);
        NLIB_STATIC_ASSERT(V3 < 8);
        static const int V0 = (V1 & 1) ? V1 - 1 : V1;
        return F128PermuteDontCareHelper<V0, V1, V2, V3>::Permute(a, b);
    }
};

template<int V0, int V2, int V3>
struct F128PermuteDontCareHelper<V0, 8, V2, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V0 < 8);
        NLIB_STATIC_ASSERT(V2 < 8);
        NLIB_STATIC_ASSERT(V3 < 8);
        static const int V1 = (V0 & 1) ? V0 : (V0 + 1);
        return F128PermuteDontCareHelper<V0, V1, V2, V3>::Permute(a, b);
    }
};

template<int V0, int V1, int V3>
struct F128PermuteDontCareHelper<V0, V1, 8, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V0 < 8);
        NLIB_STATIC_ASSERT(V1 < 8);
        NLIB_STATIC_ASSERT(V3 < 8);
        static const int V2 = (V3 & 1) ? V3 - 1 : V3;
        return F128PermuteDontCareHelper<V0, V1, V2, V3>::Permute(a, b);
    }
};

template<int V0, int V1, int V2>
struct F128PermuteDontCareHelper<V0, V1, V2, 8> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V0 < 8);
        NLIB_STATIC_ASSERT(V1 < 8);
        NLIB_STATIC_ASSERT(V2 < 8);
        static const int V3 = (V2 & 1) ? V2 : (V2 + 1);
        return F128PermuteDontCareHelper<V0, V1, V2, V3>::Permute(a, b);
    }
};

template<int V2, int V3>
struct F128PermuteDontCareHelper<8, 8, V2, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V2 < 8);
        NLIB_STATIC_ASSERT(V3 < 8);
        static const int V0 = (V2 < 4) ? 0 : 4;
        return F128PermuteDontCareHelper<V0, V0 + 1, V2, V3>::Permute(a, b);
    }
};

template<int V1, int V2>
struct F128PermuteDontCareHelper<8, V1, V2, 8> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V1 < 8);
        NLIB_STATIC_ASSERT(V2 < 8);
        static const int V0 = (V1 & 1) ? V1 - 1: V1;
        static const int V3 = (V2 & 1) ? V2 : V2 + 1;
        return F128PermuteDontCareHelper<V0, V1, V2, V3>::Permute(a, b);
    }
};

template<int V0, int V1>
struct F128PermuteDontCareHelper<V0, V1, 8, 8> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V0 < 8);
        NLIB_STATIC_ASSERT(V1 < 8);
        static const int V2 = (V1 < 4) ? 2 : 6;
        return F128PermuteDontCareHelper<V0, V1, V2, V2 + 1>::Permute(a, b);
    }
};

template<int V0, int V3>
struct F128PermuteDontCareHelper<V0, 8, 8, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V0 < 8);
        NLIB_STATIC_ASSERT(V3 < 8);
        static const int V1 = (V0 & 1) ? V0 : V0 + 1;
        static const int V2 = (V3 & 1) ? V3 - 1 : V3;
        return F128PermuteDontCareHelper<V0, V1, V2, V3>::Permute(a, b);
    }
};

template<int V0, int V2>
struct F128PermuteDontCareHelper<V0, 8, V2, 8> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V0 < 8);
        NLIB_STATIC_ASSERT(V2 < 8);
        static const int V1 = (V0 & 1) ? V0 : V0 + 1;
        static const int V3 = (V2 & 1) ? V2 : V2 + 1;
        return F128PermuteDontCareHelper<V0, V1, V2, V3>::Permute(a, b);
    }
};

template<int V1, int V3>
struct F128PermuteDontCareHelper<8, V1, 8, V3> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V1 < 8);
        NLIB_STATIC_ASSERT(V3 < 8);
        static const int V0 = (V1 & 1) ? V1 - 1 : V1;
        static const int V2 = (V3 & 1) ? V3 - 1 : V3;
        return F128PermuteDontCareHelper<V0, V1, V2, V3>::Permute(a, b);
    }
};

template<int V>
struct F128PermuteDontCareHelper<V, 8, 8, 8> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V < 8);
        static const int V1 = ((V & 3) == 0) ? V + 1 : V;
        static const int V2 = ((V & 3) == 0) ? V + 2 : V;
        static const int V3 = ((V & 3) == 0) ? V + 3 : V;
        return F128PermuteDontCareHelper<V, V1, V2, V3>::Permute(a, b);
    }
};

template<int V>
struct F128PermuteDontCareHelper<8, V, 8, 8> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V < 8);
        static const int V0 = ((V & 3) == 1) ? V - 1 : V;
        static const int V2 = ((V & 3) == 1) ? V + 1 : V;
        static const int V3 = ((V & 3) == 1) ? V + 2 : V;
        return F128PermuteDontCareHelper<V0, V, V2, V3>::Permute(a, b);
    }
};

template<int V>
struct F128PermuteDontCareHelper<8, 8, V, 8> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V < 8);
        static const int V0 = ((V & 3) == 2) ? V - 2 : V;
        static const int V1 = ((V & 3) == 2) ? V - 1 : V;
        static const int V3 = ((V & 3) == 2) ? V + 2 : V;
        return F128PermuteDontCareHelper<V0, V1, V, V3>::Permute(a, b);
    }
};

template<int V>
struct F128PermuteDontCareHelper<8, 8, 8, V> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(V < 8);
        static const int V0 = ((V & 3) == 3) ? V - 3 : V;
        static const int V1 = ((V & 3) == 3) ? V - 2 : V;
        static const int V2 = ((V & 3) == 3) ? V - 1 : V;
        return F128PermuteDontCareHelper<V0, V1, V2, V>::Permute(a, b);
    }
};

template <>
struct F128PermuteDontCareHelper<8, 8, 8, 8> {
    static NLIB_M(f128) Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
        NLIB_UNUSED(b);
        return a;
    }
};

}  // namespace detail

template <int V0, int V1, int V2, int V3>
// r[0] = V0 < 4 ? a[V0] : b[V0 - 4], .....
NLIB_M(f128) F128::Permute(f128arg a, f128arg b) NLIB_NOEXCEPT {
#if __has_builtin(__builtin_shufflevector) && !defined(NLIB_F128_SIMD_NOUSE)
    return __builtin_shufflevector(a, b,
                                   (V0 != 8 ? V0 : -1),
                                   (V1 != 8 ? V1 : -1),
                                   (V2 != 8 ? V2 : -1),
                                   (V3 != 8 ? V3 : -1));
#else
    return detail::F128PermuteDontCareHelper <
        V0 != -1 ? V0 : 8,
        V1 != -1 ? V1 : 8,
        V2 != -1 ? V2 : 8,
        V3 != -1 ? V3 : 8>::Permute(a, b);
#endif
}

template <bool SplatLane0, bool SplatLane1, bool SplatLane2, bool SplatLane3>
// r[i] = InsertLane(i) ? insert_value[i] : value[i]
// note that splat[0] = splat[1] = splat[2] = splat[3]
NLIB_M(f128) F128::Splat(f128arg value, f128arg splat) NLIB_NOEXCEPT {
#if defined(NLIB_NEON)
    const int v0 = SplatLane0 ? (SplatLane1 ? 4 : 5) : 0;
    const int v1 = SplatLane1 ? (SplatLane0 ? 5 : 4) : 1;
    const int v2 = SplatLane2 ? (SplatLane3 ? 6 : 7) : 2;
    const int v3 = SplatLane3 ? (SplatLane2 ? 7 : 6) : 3;
#else
    // SSE4.1 has _mm_blend_ps()
    const int v0 = SplatLane0 ? 4 : 0;
    const int v1 = SplatLane1 ? 5 : 1;
    const int v2 = SplatLane2 ? 6 : 2;
    const int v3 = SplatLane3 ? 7 : 3;
#endif
    return F128::Permute<v0, v1, v2, v3>(value, splat);
}

NLIB_M2(f128) F128::Exp2(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.v[0] = powf(2.f, value.vec.v[0]);
    ret.vec.v[1] = powf(2.f, value.vec.v[1]);
    ret.vec.v[2] = powf(2.f, value.vec.v[2]);
    ret.vec.v[3] = powf(2.f, value.vec.v[3]);
    return ret;
#else
    i128 iround = F128::ConvertToI128Round(value);
    f128 fround = F128::ConvertFromI128(iround);
    f128 x = F128::Sub(value, fround);
    f128 xx = F128::Mult(x, x);

    f128 P = F128::LoadA16(F128::exp2_P_);
    f128 Q = F128::LoadA16(F128::exp2_Q_);

    f128 px;
    // px = P[0]
    // px = px * xx + P[1]
    // px = px * xx + P[2]
    // px = x * px;
    px = F128::MultAdd<0>(P, xx, F128::SetValue<1>(P, each_select32), each_select32);
    px = F128::MultAdd(px, xx, F128::SetValue<2>(P, each_select32));
    px = F128::Mult(x, px);

    f128 qx;
    // qx = xx + Q[0]
    // qx = qx * xx + Q[1]
    qx = F128::Add(xx, F128::SetValue<0>(Q, each_select32));
    qx = F128::MultAdd(qx, xx, F128::SetValue<1>(Q, each_select32));

    x = F128::Div(px, F128::Sub(qx, px));
    x = F128::MultAdd<3>(Q, x, F128::SetValue<2>(Q, each_select32), each_select32);

    // x = x * 2^iround
    iround = I128::Add32(iround, I128::SetValue(127, each_int32));
    iround = I128::ShiftLeftLogical32(iround, 23);
    x = F128::Mult(x, F128::CastFromI128(iround));

    // NOTE:
    // overflow not checked
    return x;
#endif
}

NLIB_M(f128) F128::ExpE(f128arg value) NLIB_NOEXCEPT {
    static const float log2e = 1.44269504088896340736f;
    return Exp2(F128::Mult(log2e, value));
}

NLIB_M(f128) F128::SinH(f128arg value) NLIB_NOEXCEPT {
    static const float log2e = 1.44269504088896340736f;
    f128 neg_one = F128::SetValue(-1.f, each_float);
    f128 v0 = F128::MultAdd(log2e, value, neg_one);
    f128 v1 = F128::MultSub(log2e, value, neg_one);
    f128 e0 = Exp2(v0);
    f128 e1 = Exp2(v1);
    return F128::Sub(e0, e1);
}

NLIB_M(f128) F128::CosH(f128arg value) NLIB_NOEXCEPT {
    static const float log2e = 1.44269504088896340736f;
    f128 neg_one = F128::SetValue(-1.f, each_float);
    f128 v0 = F128::MultAdd(log2e, value, neg_one);
    f128 v1 = F128::MultSub(log2e, value, neg_one);
    f128 e0 = Exp2(v0);
    f128 e1 = Exp2(v1);
    return F128::Add(e0, e1);
}

NLIB_M(f128) F128::TanH(f128arg value) NLIB_NOEXCEPT {
    // 1 - 2 * (1 + expE(2x))
    f128 cvalue = F128::LoadA16(tanh_cvalue_);
    f128 e = F128::Mult<0>(cvalue, value, each_select32);
    e = F128::Exp2(e);
    f128 half = F128::SetValue<2>(cvalue, each_select32);
    e = F128::MultAdd(half, e, half);
    e = F128::Recp(e);
    return F128::Sub(F128::SetValue<1>(cvalue, each_select32), e);
}

NLIB_M2(f128) F128::Tan(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    f128 ret;
    ret.vec.v[0] = tanf(value.vec.v[0]);
    ret.vec.v[1] = tanf(value.vec.v[1]);
    ret.vec.v[2] = tanf(value.vec.v[2]);
    ret.vec.v[3] = tanf(value.vec.v[3]);
    return ret;
#else
    // Cody and Waite algorithm
    f128 C = F128::LoadA16(&F128::tan_c_[0]);

    // g = round(value / (pi/2))
    f128 g = F128::Round(F128::Mult<0>(C, value, each_select32));
    f128 nearx_axis;
    {
        i128 t0 = I128::And(F128::ConvertToI128Round(g), I128::SetValue(1U, each_uint32));
        i128 cmp = I128::CmpEq32(t0, I128::SetZero());
        nearx_axis = F128::CastFromI128(cmp);
    }

    // f = value - (pi/2) * g
    f128 f = F128::MultSub<1>(C, g, value, each_select32);
    f = F128::MultSub<2>(C, g, f, each_select32);

    f128 near_axis = F128::CmpNearEqZero(f, F128::SetValue<3>(C, each_select32));

    f128 P = F128::LoadA16(&F128::tan_p_[0]);
    f128 Q = F128::LoadA16(&F128::tan_q_[0]);

    f128 ff = F128::Mult(f, f);
    f128 one = F128::SetValue<3>(P, each_select32);

    f128 p = F128::MultAdd<2>(P, ff, F128::SetValue<1>(P, each_select32), each_select32);
    p = F128::MultAdd(p, ff, F128::SetValue<0>(P, each_select32));
    p = F128::MultAdd(p, ff, one);
    p = F128::Mult(f, p);

    f128 q = F128::MultAdd<3>(Q, ff, F128::SetValue<2>(Q, each_select32), each_select32);
    q = F128::MultAdd(q, ff, F128::SetValue<1>(Q, each_select32));
    q = F128::MultAdd(q, ff, F128::SetValue<0>(Q, each_select32));
    q = F128::MultAdd(q, ff, one);

    p = F128::Select(near_axis, f, p);
    q = F128::Select(near_axis, one, q);

    f128 r0 = F128::Div(p, q);
    f128 r1 = F128::Negate(F128::Recp(r0));

    return F128::Select(nearx_axis, r0, r1);
#endif
}

NLIB_M2(f128) F128::Log2(f128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(CAFE)
    static const float scale = 1.4426950408889634f;  // 1 / LogE(2.0)
    f128 ret;
    ret.vec.v[0] = logf(value.vec.v[0]);
    ret.vec.v[1] = logf(value.vec.v[1]);
    ret.vec.v[2] = logf(value.vec.v[2]);
    ret.vec.v[3] = logf(value.vec.v[3]);
    return F128::Mult(scale, ret);
#else
    // x = frexp(value, &e)
    f128 x = F128::And(F128::SetValue(0x807FFFFFU, each_uint32), value);
    x = F128::Or(F128::SetValue(127U << 23, each_uint32), x);
    i128 e = I128::And(I128::SetValue(0x7F800000U, each_uint32), F128::CastToI128(value));
    e = I128::ShiftRightLogical32(e, 23);
    e = I128::Sub32(e, I128::SetValue(127U, each_uint32));

    x = F128::Sub(x, F128::SetOne());
    f128 z = F128::Mult(x, x);
    f128 y;

    f128 pq0 = F128::LoadA16(&F128::log2_PQ_[0]);
    f128 pq1 = F128::LoadA16(&F128::log2_PQ_[4]);
    f128 pq2 = F128::LoadA16(&F128::log2_PQ_[8]);

    f128 p = F128::SetValue<0>(pq0, each_select32);
    p = F128::MultAdd(p, x, F128::SetValue<1>(pq0, each_select32));
    p = F128::MultAdd(p, x, F128::SetValue<2>(pq0, each_select32));
    p = F128::MultAdd(p, x, F128::SetValue<3>(pq0, each_select32));
    p = F128::MultAdd(p, x, F128::SetValue<0>(pq1, each_select32));
    p = F128::MultAdd(p, x, F128::SetValue<1>(pq1, each_select32));

    f128 q = F128::Add(x, F128::SetValue<2>(pq1, each_select32));
    q = F128::MultAdd(q, x, F128::SetValue<3>(pq1, each_select32));
    q = F128::MultAdd(q, x, F128::SetValue<0>(pq2, each_select32));
    q = F128::MultAdd(q, x, F128::SetValue<1>(pq2, each_select32));
    q = F128::MultAdd(q, x, F128::SetValue<2>(pq2, each_select32));

    y = F128::Mult(z, p);
    y = F128::Div(y, q);
    y = F128::MultAdd(x, y, F128::Mult(-0.5f, z));

    f128 result;
    {
        // do not optimize
        f128 log2ea = F128::SetValue<3>(pq2, each_select32);
        result = F128::Mult(y, log2ea);
        result = F128::MultAdd(log2ea, x, result);
        result = F128::Add(result, y);
        result = F128::Add(result, x);
        result = F128::Add(result, F128::ConvertFromI128(e));
    }

    {
        f128 nan_inf = F128::LoadA16(reinterpret_cast<const float*>(F128::nan_inf_));

        // value is NaN -> NaN
        f128 is_nan = F128::IsNaN(value);
        f128 nan = F128::SetValue<0>(nan_inf, each_select32);
        result = F128::Select(is_nan, nan, result);

        f128 is_inf = F128::IsInfinite(value);
        f128 is_pos = F128::CmpGtZero(value);

        // value == inf -> +inf
        f128 inf = F128::SetValue<1>(nan_inf, each_select32);
        f128 is_pos_inf = F128::And(is_inf, is_pos);
        result = F128::Select(is_pos_inf, inf, result);

        // value == 0 -> -inf
        f128 neg_inf = F128::SetValue<3>(nan_inf, each_select32);
        f128 is_zero = F128::CmpEqZero(value);
        result = F128::Select(is_zero, neg_inf, result);

        // value < 0 -> -NaN
        f128 neg_nan = F128::SetValue<2>(nan_inf, each_select32);
        f128 is_neg = F128::CmpLtZero(value);
        result = F128::Select(is_neg, neg_nan, result);

        // otherwise -> Log2(value)
    }

    return result;
#endif
}

NLIB_M(f128) F128::LogE(f128arg value) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = logf(value.vec.v[0]);
    ret.vec.v[1] = logf(value.vec.v[1]);
    ret.vec.v[2] = logf(value.vec.v[2]);
    ret.vec.v[3] = logf(value.vec.v[3]);
    return ret;
#else
    f128 x = F128::Log2(value);
    static const float recp_log2e = 0.6931471805597018f;
    return F128::Mult(recp_log2e, x);
#endif
}

#undef NLIB_M
#undef NLIB_M2
#endif  // NLIB_DOXYGEN

typedef f128 SimdVector;
typedef f128arg SimdVectorArg;
typedef f128 SimdQuaternion;
typedef f128arg SimdQuaternionArg;
typedef f128 SimdPlane;
typedef f128arg SimdPlaneArg;
typedef f128 SimdSphere;
typedef f128arg SimdSphereArg;

#if !defined(NLIB_DOXYGEN) && !defined(NN_PLATFORM_CTR)
struct NLIB_ALIGNAS(16) SimdMatrix {
#else
struct SimdMatrix {
#endif
 public:
    SimdMatrix() {}
    SimdMatrix(f128arg r0, f128arg r1, f128arg r2, f128arg_ex r3) NLIB_NOEXCEPT {
        r[0] = r0;
        r[1] = r1;
        r[2] = r2;
        r[3] = r3;
    }
    SimdMatrix(float m00, float m01, float m02, float m03, float m10, float m11, float m12,
               float m13, float m20, float m21, float m22, float m23, float m30, float m31,
               float m32, float m33) NLIB_NOEXCEPT;
    explicit SimdMatrix(const float* p) NLIB_NOEXCEPT;

 public:
    f128 r[4];
};

inline SimdMatrix::SimdMatrix(float m00, float m01, float m02, float m03, float m10, float m11,
                              float m12, float m13, float m20, float m21, float m22, float m23,
                              float m30, float m31, float m32, float m33) NLIB_NOEXCEPT {
    r[0] = F128::SetValue(m00, m01, m02, m03);
    r[1] = F128::SetValue(m10, m11, m12, m13);
    r[2] = F128::SetValue(m20, m21, m22, m23);
    r[3] = F128::SetValue(m30, m31, m32, m33);
}

inline SimdMatrix::SimdMatrix(const float* p) NLIB_NOEXCEPT {
    uintptr_t algn = reinterpret_cast<uintptr_t>(p) & 15;
    NLIB_ASSERT((algn & 3) == 0);
    switch (algn >> 2) {
    case 0:
        r[0] = F128::LoadA16(p);
        r[1] = F128::LoadA16(p + 4);
        r[2] = F128::LoadA16(p + 8);
        r[3] = F128::LoadA16(p + 12);
        break;
    case 1:
        r[0] = F128::LoadA4(p);
        r[1] = F128::LoadA4(p + 4);
        r[2] = F128::LoadA4(p + 8);
        r[3] = F128::LoadA4(p + 12);
        break;
    case 2:
        r[0] = F128::LoadA8(p);
        r[1] = F128::LoadA8(p + 4);
        r[2] = F128::LoadA8(p + 8);
        r[3] = F128::LoadA8(p + 12);
        break;
    default:
        NLIB_ASSUME(0);
        break;
    }
}

#if !defined(NLIB_SIMD) || defined(NLIB_F128_SIMD_NOUSE)
typedef const SimdMatrix& SimdMatrixArg;
#else
typedef const SimdMatrix SimdMatrixArg;
#endif

#if defined(NLIB_SSE41) || defined(NLIB_F128_SIMD_NOUSE)
#define NLIB_F128_TRANSPOSE(row0, row1, row2, row3)        \
    {                                                      \
        f128 tmp0 = F128::Permute<0, 1, 4, 5>(row0, row1); \
        f128 tmp2 = F128::Permute<2, 3, 6, 7>(row0, row1); \
        f128 tmp1 = F128::Permute<0, 1, 4, 5>(row2, row3); \
        f128 tmp3 = F128::Permute<2, 3, 6, 7>(row2, row3); \
        row0 = F128::Permute<0, 2, 4, 6>(tmp0, tmp1);      \
        row1 = F128::Permute<1, 3, 5, 7>(tmp0, tmp1);      \
        row2 = F128::Permute<0, 2, 4, 6>(tmp2, tmp3);      \
        row3 = F128::Permute<1, 3, 5, 7>(tmp2, tmp3);      \
    }
#elif defined(NLIB_NEON)
# ifdef __aarch64__
#define NLIB_F128_TRANSPOSE(row0, row1, row2, row3)                    \
    {                                                                  \
        float32x4x2_t trn_f0_ = vtrnq_f32(row0, row1);                 \
        float32x4x2_t trn_f1_ = vtrnq_f32(row2, row3);                 \
        uint64x2_t row0_, row1_, row2_, row3_;                         \
        row0_ = vtrn1q_u64(vreinterpretq_u64_f32(trn_f0_.val[0]),      \
                           vreinterpretq_u64_f32(trn_f1_.val[0]));     \
        row0 = vreinterpretq_f32_u64(row0_);                           \
        row1_ = vtrn1q_u64(vreinterpretq_u64_f32(trn_f0_.val[1]),      \
                           vreinterpretq_u64_f32(trn_f1_.val[1]));     \
        row1 = vreinterpretq_f32_u64(row1_);                           \
        row2_ = vtrn2q_u64(vreinterpretq_u64_f32(trn_f0_.val[0]),      \
                           vreinterpretq_u64_f32(trn_f1_.val[0]));     \
        row2 = vreinterpretq_f32_u64(row2_);                           \
        row3_ = vtrn2q_u64(vreinterpretq_u64_f32(trn_f0_.val[1]),      \
                           vreinterpretq_u64_f32(trn_f1_.val[1]));     \
        row3 = vreinterpretq_f32_u64(row3_);                           \
    }
# else
#define NLIB_F128_TRANSPOSE(row0, row1, row2, row3)                                        \
    {                                                                                      \
        float32x4x2_t trn_f0_ = vtrnq_f32(row0, row1);                                     \
        float32x4x2_t trn_f1_ = vtrnq_f32(row2, row3);                                     \
        row0 = vcombine_f32(vget_low_f32(trn_f0_.val[0]), vget_low_f32(trn_f1_.val[0]));   \
        row1 = vcombine_f32(vget_low_f32(trn_f0_.val[1]), vget_low_f32(trn_f1_.val[1]));   \
        row2 = vcombine_f32(vget_high_f32(trn_f0_.val[0]), vget_high_f32(trn_f1_.val[0])); \
        row3 = vcombine_f32(vget_high_f32(trn_f0_.val[1]), vget_high_f32(trn_f1_.val[1])); \
    }
# endif
#elif defined(CAFE)
#define NLIB_F128_TRANSPOSE(row0, row1, row2, row3)          \
    {                                                        \
        f32x2 tmp0, tmp1;                                    \
        tmp0 = __PS_MERGE00(row0.vec.ps[0], row1.vec.ps[0]); \
        tmp1 = __PS_MERGE11(row0.vec.ps[0], row1.vec.ps[0]); \
        row0.vec.ps[0] = tmp0;                               \
        row1.vec.ps[0] = tmp1;                               \
        tmp0 = __PS_MERGE00(row2.vec.ps[1], row3.vec.ps[1]); \
        tmp1 = __PS_MERGE11(row2.vec.ps[1], row3.vec.ps[1]); \
        row2.vec.ps[1] = tmp0;                               \
        row3.vec.ps[1] = tmp1;                               \
        tmp0 = __PS_MERGE00(row0.vec.ps[1], row1.vec.ps[1]); \
        tmp1 = __PS_MERGE11(row0.vec.ps[1], row1.vec.ps[1]); \
        row0.vec.ps[1] = row2.vec.ps[0];                     \
        row1.vec.ps[1] = row3.vec.ps[0];                     \
        row2.vec.ps[0] = tmp0;                               \
        row3.vec.ps[0] = tmp1;                               \
        tmp0 = __PS_MERGE00(row0.vec.ps[1], row1.vec.ps[1]); \
        tmp1 = __PS_MERGE11(row0.vec.ps[1], row1.vec.ps[1]); \
        row0.vec.ps[1] = tmp0;                               \
        row1.vec.ps[1] = tmp1;                               \
    }
#endif

struct NLIB_VIS_PUBLIC Float3 {
    float x;
    float y;
    float z;
};

struct NLIB_VIS_PUBLIC Float4 {
    float x;
    float y;
    float z;
    float w;
};

struct NLIB_VIS_PUBLIC Float2x3 {
    float m[2][3];
};

struct NLIB_VIS_PUBLIC Float3x3 {
    float m[3][3];
};

#if !defined(NLIB_DOXYGEN) && !defined(NN_PLATFORM_CTR)
struct NLIB_ALIGNAS(16) Float3x4 {
#else
struct Float3x4 {
#endif
    float m[3][4];
};

#if !defined(NLIB_DOXYGEN) && !defined(NN_PLATFORM_CTR)
struct NLIB_ALIGNAS(16) Float4x3 {
#else
struct Float4x3 {
#endif
    float m[4][3];
};

#if !defined(NLIB_DOXYGEN) && !defined(NN_PLATFORM_CTR)
struct NLIB_ALIGNAS(16) Float4x4 {
#else
struct Float4x4 {
#endif
    float m[4][4];
};

}  // namespace simd
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_SIMD_SIMDFLOAT_H_
